package CoPS.agents;
//
//  AdministerProblemSolvingBehaviour.java
//  CoPS
//
//  Created by Kari on 01.02.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

import jade.proto.AchieveREInitiator;
import jade.core.Agent;
import jade.lang.acl.ACLMessage;
import jade.core.AID;

import java.util.Vector;
import java.io.IOException;

import CoPS.FileWriter;

/**
 *	This class represents the behaviour defining the administration of the 
 *	problem solving process. <b>Important!</b> This behaviour is only a 
 *	place holder.
 *
 *	@author Kari Roessland
 */
public class AdministerProblemSolvingProcessBehaviour extends AchieveREInitiator{
	private int transition = PrepareResultBehaviour.PROBLEM_SOLVING_FAILED;
	private FileWriter fw = null;
	
	/**
	 * The constructor.
	 * @param agent The owner of this behaviour.
	 * @param request The default request message.
	 */
	public AdministerProblemSolvingProcessBehaviour(Agent agent, ACLMessage request){
		super(agent, request);
		this.registerHandleAgree(new HandleAgreeBehaviour(myAgent));
	}
	
	
	/**
	 * In this method the request messages are prepared.
	 * @param request The default request message.
	 * @return A Vector containing the request messages.
	 */
	protected Vector prepareRequests(ACLMessage request){

		Vector requests = new Vector();
		AID tsAgent = ((CoPSTaskResponsible)myAgent).getTeamSpaceAgent();
		fw = ((CoPSTaskResponsible)myAgent).getFileWriter();
		
		try{
		
			request.addReceiver(tsAgent);
			request.setLanguage("jcreek:object");
			request.setContentObject(((CoPSTaskResponsible)myAgent).getTMST());
			requests.add(request);
			
		}catch(IOException e){
			e.printStackTrace();
		}
		
		fw.write("    "+myAgent.getLocalName()+" requests (a TEAM SPACE agent) "+tsAgent.getLocalName()+ " to initialize");
		fw.write("    the TEAM SPACE, by sending the TMST, now containing information about the chosen solution");
		
		return requests;
	}
	
	/**
	 * This method handles not-understood messages.
	 * @param notUnderstood The received not-understood message.
	 */
	protected void handleNotUnderstood(ACLMessage notUnderstood){
		fw.write("    "+notUnderstood.getSender().getLocalName()+" did not understand the content of the request from "+myAgent.getLocalName());
	}
	
	/**
	 * This method handles refuse messages.
	 * @param refuse The received refuse message.
	 */
	protected void handleRefuse(ACLMessage refuse){
		fw.write("    "+refuse.getSender().getLocalName()+ " did refuse to initalize the TEAM SPACE, requested from "+myAgent.getLocalName());
	}
	
	
	/**
	 * This method handles the failure messages.
	 * @param failure The received failure message.
	 */
	protected void handleFailure(ACLMessage failure){
		System.out.println("    "+failure.getSender().getLocalName()+ " did fail to solve the problem in the TEAM SPACE, requested from "+myAgent.getLocalName());
	}

	/**
	 * This method handles the inform messages.
	 * @param inform The received inform message.
	 */	
	protected void handleInform(ACLMessage inform){
		fw.write("    "+myAgent.getLocalName()+" got the solution of the problem solving from "+inform.getSender().getLocalName());
		fw.write("      Content of the solution-message: " + inform.getContent());
		((CoPSTaskResponsible)myAgent).setSolution(inform.getContent());
		transition = PrepareResultBehaviour.SUCCESS;
	}
	
	/**
	 * The onEnd method is called at the completion of this behaviour.
	 * @return The transition from this behaviour.
	 */
	public int onEnd(){
		reset();
		return transition;
	}
}
