package CoPS.agents;
//
//  CheckCfpsBehaviour.java
//  CoPS
//
//  Created by Kari on 10.04.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//


import jade.core.*;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.core.behaviours.DataStore;
import jade.domain.FIPANames;
import jade.core.behaviours.ParallelBehaviour;
import java.util.ArrayList;

/**
 * This class implements a behaviour which checks all the incoming cfp-messages
 * received by the owner agent of this behaviour. And a parallel behaviour is added
 * to the agent which handles all of the cfp-messages in parallel.
 * @author Kari Roessland
 */
public class CheckCfpsBehaviour extends OneShotBehaviour{

	/**
	 * The constructor.
	 * @param agent The owner of this behaviour.
	 */
	public CheckCfpsBehaviour(Agent agent){
		super(agent);
	}
	
	/**
	 * This method filters all of the cfp-messages from the agent's pool of messages.
	 * Each of the messages are handled by one of the sub-behaviours of a parallel behaviour.
	 */
	public void action(){
		//myAgent.doWait(5000);
		//Initialise Message Template which is going to be used
		MessageTemplate template = MessageTemplate.and(
													   MessageTemplate.MatchProtocol(FIPANames.InteractionProtocol.FIPA_CONTRACT_NET),
													   MessageTemplate.MatchPerformative(ACLMessage.CFP)
													   );
		
		
		ArrayList cfpMessages = new ArrayList();
		ACLMessage cfpMessage = null;
		boolean hasMoreMsg = true;
		boolean first = true;
		
		while(hasMoreMsg){
			
			if(first){
				cfpMessage = myAgent.blockingReceive(template);
				first = false;
			}else{
				cfpMessage = myAgent.blockingReceive(2000);
			}
			
			if(cfpMessage == null){
				hasMoreMsg = false;
			}else{
				cfpMessages.add(cfpMessage);
			}
		}

		ParallelBehaviour pb = new ParallelBehaviour(myAgent, ParallelBehaviour.WHEN_ALL);
		
		RespondToCfpsBehaviour rtcfp = null;
		
		
		for(int i = 0; i < cfpMessages.size(); i++){
		
			rtcfp = new RespondToCfpsBehaviour(myAgent, template, new DataStore());
			pb.addSubBehaviour(rtcfp);
		
		}
		
		myAgent.addBehaviour(pb);
		
		
		for(int i = 0; i < cfpMessages.size(); i++){
			cfpMessage = (ACLMessage)cfpMessages.get(i);
			
		
			myAgent.putBack(cfpMessage);
		}
		
	}
	
}
