package CoPS.CheckUpExample;

//
//  CheckUpOntology.java
//  CoPS, for now it just has a copy of the transport ontology.
//
//  Created by Kari on 20.11.05.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//	Extended by Kari Roessland 12.04.06

import jade.content.onto.Ontology;
import jade.content.onto.BasicOntology;
import jade.content.onto.OntologyException;
import jade.content.schema.ConceptSchema;
import jade.content.schema.AgentActionSchema;
import jade.content.schema.ObjectSchema;
import jade.content.schema.PrimitiveSchema;
import jade.content.schema.PredicateSchema;
import jade.content.onto.ReflectiveIntrospector;


/**
 * This class implements the CheckUpOntology used in our example application.
 * @author Kari Roessland
 */
public class CheckUpOntology extends Ontology{
	//Name identifying this ontology
	public static final String ONTOLOGY_NAME = "checkup-ontology";
	
	//VOCABULARY
	public static final String CHECKUP_ENTITY = "CheckUpEntity";
	public static final String CHECKUP_ENTITY_NAME = "name";
	
	public static final String PATIENT = "Patient";
	public static final String PATIENT_INFORMATION = "information";
	public static final String PATIENT_SYMPTOMS = "symptoms";
	public static final String PATIENT_DIAGNOSES = "diagnoses";
	public static final String PATIENT_HISTORICAL_INFORMATION = "historical_information";
	public static final String PATIENT_PHYSICAL_EVIDENCES = "physical_evidences";
	public static final String PATIENT_TEST_RESULTS = "test_results";
	public static final String PATIENT_EVIDENCE_CONCLUSIONS = "evidence_conclusions";
	public static final String PATIENT_BLOOD_SAMPLE_REFERENCE = "blood_sample_reference";
	public static final String FINAL_DIAGNOSE = "final_diagnose";
	
	public static final String SYMPTOM = "Symptom";
	public static final String SYMPTOM_DESCRIPTION = "description";

	public static final String DIAGNOSE = "Diagnose";
	public static final String DIAGNOSE_DESCRIPTION = "description";
	
	public static final String UPDATE_STATUS = "UpdateStatus";
	public static final String UPDATE_STATUS_STATUS = "status";

	public static final String EVIDENCE = "Evidence";
	public static final String EVIDENCE_DESCRIPTION = "description";
	
	public static final String PHYSICAL_EVIDENCE = "PhysicalEvidence";
	public static final String TEST_RESULT = "TestResult";

	public static final String CHECKUP_ACTION = "CheckUpAction";
	public static final String CHECKUP_ACTION_INPUT = "input";
	public static final String CHECKUP_ACTION_OUTPUT = "output";
	public static final String CHECKUP_ACTION_ASSUMPTIONS = "assumptions";
	
	public static final String GET_PATIENT_INFO_MA1 = "GetPatientInfoMA1";
	public static final String GET_PATIENT_INFO_DB1 = "GetPatientInfoDB1";
	public static final String UPDATE_PATIENT_INFO_MA1 = "UpdatePatientInfoMA1";
	public static final String UPDATE_PATIENT_INFO_DB1 = "UpdatePatientInfoDB1";
	public static final String INTERROGATE = "Interrogate";
	public static final String GET_ALTERNATIVE_DIAGNOSES_BOOK = "GetAlternativeDiagnosesBook";
	public static final String GET_ALTERNATIVE_DIAGNOSES_DB = "GetAlternativeDiagnosesDB";
	public static final String UPDATE_PATIENT_INFO_MA2 = "UpdatePatientInfoMA2";
	public static final String UPDATE_PATIENT_INFO_DB2 = "UpdatePatientInfoDB2";
	public static final String GET_PATIENT_INFO_DB2 = "GetPatientInfoDB2";
	public static final String GET_PATIENT_INFO_MA2 = "GetPatientInfoMA2";
	public static final String GET_HIST_INFO_MA1 = "GetHistInfoMA1";
	public static final String GET_HIST_INFO_MA2 = "GetHistInfoMA2";
	public static final String GET_HIST_INFO_DB1 = "GetHistInfoDB1";
	public static final String GET_HIST_INFO_DB2 = "GetHistInfoDB2";
	public static final String ANALYZE_BLOODTEST_MECH = "AnalyzeBloodtestMech";
	public static final String ANALYZE_BLOODTEST_MAN = "AnalyzeBloodtestMan";
	public static final String TAKE_BLOODTEST = "TakeBloodtest";
	public static final String PERFORM_PHY_EXAMINATION1 = "PerformPhyExamination1";
	public static final String PERFORM_PHY_EXAMINATION2 = "PerformPhyExamination2";
	
	public static final String PRICE = "Price";
	public static final String PRICE_VALUE = "value";
	public static final String PRICE_VALUTA = "valuta";
	
	public static final String COST = "Cost";
	public static final String COST_ACTION = "action";
	public static final String COST_PRICE = "price";
	
		
	//The singleton instance of this ontology
	private static Ontology theInstance = new CheckUpOntology();
	
	/**
	 * Method for accessing the ontology
	 * @return The ontology defined in an Ontology object.
	 */
	public static Ontology getInstance(){
		return theInstance;
	}
	
	/**
	 * The constructor - used to define the ontology.
	 */
	private CheckUpOntology(){
		//The ontology extends the basic ontology
		super(ONTOLOGY_NAME, BasicOntology.getInstance(), new ReflectiveIntrospector());
		try{
			add(new ConceptSchema(CHECKUP_ENTITY), CheckUpEntity.class);
			add(new ConceptSchema(PATIENT), Patient.class);
			add(new ConceptSchema(SYMPTOM), Symptom.class);
			add(new ConceptSchema(DIAGNOSE), Diagnose.class);
			add(new ConceptSchema(EVIDENCE), Evidence.class);
			add(new ConceptSchema(PHYSICAL_EVIDENCE), Evidence.class);
			add(new ConceptSchema(TEST_RESULT), TestResult.class);
			add(new ConceptSchema(UPDATE_STATUS), UpdateStatus.class);
			add(new ConceptSchema(PRICE), Price.class);
			add(new PredicateSchema(COST), Cost.class);
			add(new AgentActionSchema(CHECKUP_ACTION), CheckUpAction.class);
			add(new AgentActionSchema(GET_PATIENT_INFO_DB1), GetPatientInfoDB1.class);
			add(new AgentActionSchema(GET_PATIENT_INFO_MA1), GetPatientInfoMA1.class);
			add(new AgentActionSchema(UPDATE_PATIENT_INFO_MA1), UpdatePatientInfoMA1.class);
			add(new AgentActionSchema(UPDATE_PATIENT_INFO_DB1), UpdatePatientInfoDB1.class);
			add(new AgentActionSchema(INTERROGATE), Interrogate.class);
			add(new AgentActionSchema(GET_ALTERNATIVE_DIAGNOSES_BOOK), GetAlternativeDiagnosesBook.class);
			add(new AgentActionSchema(GET_ALTERNATIVE_DIAGNOSES_DB), GetAlternativeDiagnosesDB.class);
			
			add(new AgentActionSchema(UPDATE_PATIENT_INFO_MA2), UpdatePatientInfoMA2.class);
			add(new AgentActionSchema(UPDATE_PATIENT_INFO_DB2), UpdatePatientInfoDB2.class);	
			add(new AgentActionSchema(GET_PATIENT_INFO_DB2), GetPatientInfoDB2.class);	
			add(new AgentActionSchema(GET_PATIENT_INFO_MA2), GetPatientInfoMA2.class);				
			add(new AgentActionSchema(GET_HIST_INFO_DB1), GetHistInfoDB1.class);	
			add(new AgentActionSchema(GET_HIST_INFO_DB2), GetHistInfoDB2.class);	
			add(new AgentActionSchema(GET_HIST_INFO_MA1), GetHistInfoMA1.class);	
			add(new AgentActionSchema(GET_HIST_INFO_MA2), GetHistInfoMA2.class);	
			add(new AgentActionSchema(ANALYZE_BLOODTEST_MECH), AnalyzeBloodtestMech.class);														
			add(new AgentActionSchema(ANALYZE_BLOODTEST_MAN), AnalyzeBloodtestMan.class);														
			add(new AgentActionSchema(TAKE_BLOODTEST), TakeBloodtest.class);														
			add(new AgentActionSchema(PERFORM_PHY_EXAMINATION1), PerformPhyExamination1.class);														
			add(new AgentActionSchema(PERFORM_PHY_EXAMINATION2), PerformPhyExamination2.class);														
									
			//Structure of the schema for the CheckUpEntity Concept
			ConceptSchema cs = (ConceptSchema)getSchema(CHECKUP_ENTITY);
			cs.add(CHECKUP_ENTITY_NAME, (PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);//The name attribute is not optional
			
			//Structure of the the schema for the Symptom Concept
			cs = (ConceptSchema)getSchema(SYMPTOM);
			cs.addSuperSchema((ConceptSchema)getSchema(CHECKUP_ENTITY));
			cs.add(SYMPTOM_DESCRIPTION,(PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);//The description attribute is optional

			cs = (ConceptSchema)getSchema(EVIDENCE);
			cs.addSuperSchema((ConceptSchema)getSchema(CHECKUP_ENTITY));
			cs.add(EVIDENCE_DESCRIPTION,(PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);//The description attribute is optional
			
			cs = (ConceptSchema)getSchema(PHYSICAL_EVIDENCE);
			cs.addSuperSchema((ConceptSchema)getSchema(EVIDENCE));

			cs = (ConceptSchema)getSchema(TEST_RESULT);
			cs.addSuperSchema((ConceptSchema)getSchema(EVIDENCE));

			//Structure of the the schema for the Diagnose Concept
			cs = (ConceptSchema)getSchema(DIAGNOSE);
			cs.addSuperSchema((ConceptSchema)getSchema(CHECKUP_ENTITY));
			cs.add(DIAGNOSE_DESCRIPTION,(PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);//The description attribute is optional
			
			//Structure of the schema for the UpdateStatus Concept
			cs = (ConceptSchema)getSchema(UPDATE_STATUS);
			cs.add(UPDATE_STATUS_STATUS, (PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			
			//Structure of the the schema for the Patient Concept
			cs = (ConceptSchema)getSchema(PATIENT);
			cs.addSuperSchema((ConceptSchema)getSchema(CHECKUP_ENTITY));
			cs.add(PATIENT_INFORMATION,(PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(PATIENT_SYMPTOMS, (ConceptSchema) getSchema(SYMPTOM), 0, ObjectSchema.UNLIMITED);
			cs.add(PATIENT_DIAGNOSES, (ConceptSchema) getSchema(DIAGNOSE), 0, ObjectSchema.UNLIMITED);
			cs.add(PATIENT_HISTORICAL_INFORMATION, (PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(PATIENT_PHYSICAL_EVIDENCES, (ConceptSchema) getSchema(PHYSICAL_EVIDENCE), 0, ObjectSchema.UNLIMITED);
			cs.add(PATIENT_TEST_RESULTS, (ConceptSchema) getSchema(TEST_RESULT), 0, ObjectSchema.UNLIMITED);
			cs.add(PATIENT_EVIDENCE_CONCLUSIONS, (PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(PATIENT_BLOOD_SAMPLE_REFERENCE, (PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
			cs.add(FINAL_DIAGNOSE, (ConceptSchema)getSchema(DIAGNOSE), ObjectSchema.OPTIONAL);
			
			//Structucture of the schema for the Price Concept
			cs = (ConceptSchema)getSchema(PRICE);
			cs.add(PRICE_VALUE, (PrimitiveSchema)getSchema(BasicOntology.FLOAT));	
			cs.add(PRICE_VALUTA, (PrimitiveSchema) getSchema(BasicOntology.STRING));
			
			//Structure of schema for the Cost predicate
			PredicateSchema ps = (PredicateSchema)getSchema(COST);
			ps.add(COST_ACTION, (AgentActionSchema)getSchema(CHECKUP_ACTION));
			ps.add(COST_PRICE, (ConceptSchema)getSchema(PRICE));
			
			//Structure of the schema for the CheckUpAction agent action
			AgentActionSchema aas = (AgentActionSchema) getSchema(CHECKUP_ACTION);
			aas.add(CHECKUP_ACTION_INPUT, (ConceptSchema) getSchema(CHECKUP_ENTITY), 0, ObjectSchema.UNLIMITED); //May specify zero or more inputs
			aas.add(CHECKUP_ACTION_OUTPUT, (ConceptSchema) getSchema(CHECKUP_ENTITY), ObjectSchema.OPTIONAL);
			aas.add(CHECKUP_ACTION_ASSUMPTIONS, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);

			//Structure of the schema for the GettingPatientInfoDB agent action
			aas = (AgentActionSchema) getSchema(GET_PATIENT_INFO_DB1);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));

			//Structure of the schema for the GettingPatientInfoMA agent action
			aas = (AgentActionSchema) getSchema(GET_PATIENT_INFO_MA1);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
				
			//Structure of the schema for the UpdatePatientInfoDB agent action
			aas = (AgentActionSchema) getSchema(UPDATE_PATIENT_INFO_MA1);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			//Structure of the schema for the UpdatePatientInfoDB agent action
			aas = (AgentActionSchema) getSchema(UPDATE_PATIENT_INFO_DB1);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));			
			
			//Structure of the schema for the Interrogate agent action
			aas = (AgentActionSchema) getSchema(INTERROGATE);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));			
			
			//Structure of the schema for the GetAlternativeDiagnosesBook agent action
			aas = (AgentActionSchema) getSchema(GET_ALTERNATIVE_DIAGNOSES_BOOK);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));		
			
			//Structure of the schema for the GetAlternativeDiagnosesDB agent action
			aas = (AgentActionSchema) getSchema(GET_ALTERNATIVE_DIAGNOSES_DB);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));					
			
			aas = (AgentActionSchema) getSchema(UPDATE_PATIENT_INFO_MA2);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(UPDATE_PATIENT_INFO_DB2);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(GET_PATIENT_INFO_MA2);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(GET_PATIENT_INFO_DB2);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(GET_HIST_INFO_DB1);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(GET_HIST_INFO_DB2);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(GET_HIST_INFO_MA1);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(GET_HIST_INFO_MA2);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(ANALYZE_BLOODTEST_MECH);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(ANALYZE_BLOODTEST_MAN);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(TAKE_BLOODTEST);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(PERFORM_PHY_EXAMINATION1);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			aas = (AgentActionSchema) getSchema(PERFORM_PHY_EXAMINATION2);
			aas.addSuperSchema((AgentActionSchema)getSchema(CHECKUP_ACTION));
			
			
			
		}catch(OntologyException oe){
			oe.printStackTrace();
		}
	}
}
