package CoPS.CheckUpExample;
//
//  CheckUpTaskResponsible.java
//  CoPS
//
//  Created by Kari on 20.11.05.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

//Jade classes
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;
import jade.content.onto.Ontology;

//CoPS classes
import CoPS.agents.CoPSTaskResponsible;


/**
 * A class extending CoPSTaskResponsible, which implements a task responsible in 
 * our Checkup example application.
 * @author Kari Roessland
 */
public class CheckUpTaskResponsible extends CoPSTaskResponsible{

	private Ontology ontology = null;
	
	
	/**
	 * Method returning the ontology.
	 * @return The ontology defined by an Ontology object.
	 */
	protected Ontology getOntology(){
		return this.ontology;
	}
	
	/**
	 *	This is just a dummy method always returning true.
	 *	@return True.
	 */
	public boolean reviewProblem(String task){
		return true;
	}
	
	/**
	 *	This is just a dummy method always returning true.
	 *	@return True.
	 */
	public boolean problemSolved(){
		return true;
	}
	
	/**
	 *	Method returning an instance of the ontology known by this agent. This agent knows the CheckUpOntology.
	 *	@return Ontology
	 */
	protected Ontology intializeOntology(){
		this.ontology = CheckUpOntology.getInstance();
		return ontology;
	}
	
	/**
	 * Method overridden from the class CoPSAgent.
	 * @return The agents ServiceDescription object to be registered with the DF.
	 */
	public ServiceDescription setServiceDescription(){
        ServiceDescription sd = new ServiceDescription();
        sd.setType("TASK_RESPONSIBLE");
        sd.setName(this.getName());
        sd.setOwnership("KariR");
		
		
		return sd;
	}
	
	/**
	 * Method returning the last PSM to compose
	 * @return The most top-level PSM in the TMST as a String
	 */	
	public String getFinalPSM(){
		return "performingCheckup";
	}
	
	
	/**
	 * Method returning an ACLMessage describing the solution of a psm
	 * @param request - The requested psm
	 * @return The solution to the requested psm
	 */
	public ACLMessage composePSM(ACLMessage request){
		String content = "";

		ACLMessage response = request.createReply();
		content = request.getContent();
		response.setLanguage("plain-text");
	
		if(content.indexOf("findingAlternativeDiagnoses") >= 0){
			if(content.indexOf("paul") >= 0){
				response.setContent("findingAlternativeDiagnoses-(Patient :name paul :diagnoses (Diagnose :description diagnose1))");
			}
			if(content.indexOf("maria") >= 0){
				response.setContent("findingAlternativeDiagnoses-(Patient :name maria :diagnoses (Diagnose :description diagnose1))");
			}
			response.setPerformative(ACLMessage.INFORM);
		}
		
		if(content.indexOf("gatheringSimpleEvidence") >= 0){
			if(content.indexOf("paul") >= 0){		
				response.setContent("gatheringSimpleEvidence-(Patient :name paul :diagnoses (Diagnose :description diagnose1) :evidence_conclusions summary)");
			}
			if(content.indexOf("maria") >= 0){		
				response.setContent("gatheringSimpleEvidence-(Patient :name maria :diagnoses (Diagnose :description diagnose1) :evidence_conclusions summary)");
			}			
			response.setPerformative(ACLMessage.INFORM);
		}
	
		if(content.indexOf("gatheringAdvancedEvidence") >= 0){
			if(content.indexOf("paul") >= 0){				
				response.setContent("gatheringAdvancedEvidence-(Patient :name paul :diagnoses (Diagnose :description diagnose1) :evidence_conclusions summary)");
			}
			if(content.indexOf("maria") >= 0){				
				response.setContent("gatheringAdvancedEvidence-(Patient :name maria :diagnoses (Diagnose :description diagnose1) :evidence_conclusions summary)");
			}			
			response.setPerformative(ACLMessage.INFORM);
		}		

		if(content.indexOf("performingBloodTest") >= 0){
			if(content.indexOf("paul") >= 0){				
				response.setContent("performingBloodTest-(Patient :name paul :test_results (TestResult :description evidence2))");
			}
			if(content.indexOf("maria") >= 0){				
				response.setContent("performingBloodTest-(Patient :name maria :test_results (TestResult :description evidence2))");
			}			
			response.setPerformative(ACLMessage.INFORM);
		}
		
		if(content.indexOf("performingCheckup") >= 0){
			if(content.indexOf("paul") >= 0){				
				response.setContent("performingCheckup-(Patient :name paul :finalDiagnose diagnose)");
			}	
			if(content.indexOf("maria") >= 0){				
				response.setContent("performingCheckup-(Patient :name maria :finalDiagnose diagnose)");
			}	

			response.setPerformative(ACLMessage.INFORM);
		}

		
		return response;
	}
}
