package CoPS.agents;
//
//  CoPSDecomposer.java
//  CoPS
//
//  Created by Odd Erik  Gundersen on Tue May 11 2004.
//  Copyright (c) 2004 NTNU. All rights reserved.
//
//	Changed by Kari 31.01.06 - Moved the behaviour into it's own class
//  Changed by Kari 13.05.06 - The Decomposer gives the TMST a teamId- later, used
//							   to indetify the team of problem solving agents.

//Java classes
import java.io.IOException;

//JADE classes
import jade.core.Agent;
import jade.core.behaviours.SequentialBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.UnreadableException;
import jade.domain.FIPANames;

//jCreek classes
import jcreek.representation.LocalKnowledgeModel;

//TMST classes
import TMST.representation.TMST;

//CoPS classes
import CoPS.Exceptions.NotAbleToDecomposeTaskException;

/**
 *	This class represent a decomposer agent. Modified by Kari Roessland.
 */
public abstract class CoPSDecomposer extends CoPSAgent{
	
	public TMST library = null;
	public static int teamID = 0;
	
	
	/**
	 * The method setup initializes the desomposing agent. First it registers with the 
	 * service description provided by the setSereviceDescription. Then it enters a state 
	 * where it waits for new tasks to decompose.
	 */
	public void setup(){
		
		//Initializing a template. This agent will only respond to messages which follows it.
		MessageTemplate template = MessageTemplate.and(
													   MessageTemplate.MatchProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST),
													   MessageTemplate.MatchPerformative(ACLMessage.REQUEST)
													   );
		
		//initialize the task structure library
		this.library = this.setTaskStructureLibrary();
		
		//Adding behaviours, first register with DF, then wait for problems to decompose
		SequentialBehaviour sb = new SequentialBehaviour(this);
		sb.addSubBehaviour(new DFRegisteringBehaviour(this, this.setServiceDescription()));
		sb.addSubBehaviour(new RespondToDecomposeRequestsBehaviour(this, template));
		this.addBehaviour(sb);

	}
		
	/**
		Abstract method which define how a task is supposed to be decomposed.
		@param Task The problem description which will be decomposed into problem solving methods and subtasks
		@return LocalKnowledgeModel Returns a decomposition which is represented as a jCreek model composed of entities and relations.
	*/
	protected abstract TMST decomposeProblem(String task) throws NotAbleToDecomposeTaskException;
	
	/**
		Method called by the ReviewTask behaviour.
		@return boolean Returns true if, the task is understandable and follows a format the decomposer understands
	*/
	protected abstract boolean correctFormat(String task);
		
	/**
		Method for initializing the knowledge model (task structure tree) of a decomposer. Called in the setup() method.
	*/
	protected abstract TMST setTaskStructureLibrary();
	
	
	/**
	 * Method for getting a new (by incrementing) teamId. Added by Kari Roessland.
	 * @return A team-id that is unique no matter what Decomposer returns it
	 */
	protected static int getTeamID(){
		teamID++;
		return teamID;
	}
}

