package CoPS.agents;
// 
//  CoPSTaskResponsible.java
//  CoPS
//
//  Created by Odd Erik  Gundersen on Tue May 11 2004.
//  Copyright (c) 2004 NTNU. All rights reserved.
// 
// Changed by Kari 31.01.06 - Moved the behaviour into it's own class
// Changed by Kari 10.04.06 - Changed makeInvitations drastically
// Changed by Kari 12.04.06 - The mapping of Executors to Actions is changed, so that executors
//						      that already exeist is not made all over again.
// Changed by Kari 13.05.06 - Added teamID to the invitation to join a team, removed
//							  old stuff about TS (team space)
// Changed by Kari 13.05.06 - While setting the TMST for the TR, the initial input, defined
//							  by the instance-variable task is, is set in the TMST.
//
// Changed by Kari 13.05.06 - Pretty print CoPS
// Changed by Kari 14.05.06 - The make- invitation method also makes reject-proposal messages
//							  for the agents making proposals that were not accepted.
// Changed by kari 14.05.06 - Set and get the solution properly

//java classes
import java.util.Vector;
import java.util.ArrayList;
import java.util.Date;

//jade classes
import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.SequentialBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.domain.FIPANames;
import jade.domain.DFService;
import jade.domain.FIPAAgentManagement.Property;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.content.onto.Ontology;
import jade.content.onto.OntologyException;
import jade.content.lang.Codec.CodecException;
import jade.content.lang.sl.SLCodec;
import jade.content.lang.sl.SLVocabulary;
import jade.content.abs.AbsContentElement;
import jade.content.abs.AbsIRE;
import jade.content.abs.AbsObject;
import jade.content.abs.AbsPredicate;
import jade.content.abs.AbsAggregate;
import jade.content.abs.AbsPrimitive;
import jade.content.schema.PrimitiveSchema;

//tmst classes
import TMST.representation.TMST;
import TMST.representation.Action;
import TMST.representation.Output;
import TMST.representation.Input;
import TMST.representation.Assumption;
import TMST.representation.Executor;
import TMST.Exceptions.AlreadyHasValueException;

//jcreek classes
import jcreek.representation.NameAlreadyExistException;
import jcreek.representation.EntityTypeNotMatchedException;
import jcreek.representation.Entity;


//cops classes
import CoPS.CheckUpExample.*;
import CoPS.*;

/**
 *	Class implementing the Task Responsible role. 
 *	The Task Responsible is responisble for administering the problem solving process.
 *
 */
public abstract class CoPSTaskResponsible extends CoPSAgent{
	private TMST tmst = null;
	private String task = ""; //Dummy variable.
	private String solution = ""; 
	private ACLMessage request = null; //The request recevied from the employer
	private ACLMessage response = null; //The respons which is sent to the employer
	private long waitForMillis = 10000; //Default value
	private Ontology ontology;
	private FileWriter fw = null;	
 	/**
	 * The setup method. Modified by Kari Roessland.
	 */
	public void setup(){
	
		fw = new FileWriter(this.getLocalName()+".txt");
		fw.clear();
	
		//Register language
		
		//Register ontologies
		ontology = intializeOntology();
		
		//Initialise Message Template which is used for filtering which responses are being answered and which are not
		MessageTemplate template = MessageTemplate.and(
														   MessageTemplate.MatchProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST),
														   MessageTemplate.MatchPerformative(ACLMessage.REQUEST)
														   );
		
		//Add Behaviours to this agent
		SequentialBehaviour sb = new SequentialBehaviour(this);
		sb.addSubBehaviour(new DFRegisteringBehaviour(this, this.setServiceDescription()));		
		sb.addSubBehaviour(new HandleProblemSolvingRequestsBehaviour(this, template));
		this.addBehaviour(sb);
	}
	
	
	/**
	 * Method returning the filewriter, associated with the file belonging to this
	 * task responsible agent. New method added by Kari Roessland.
	 * @return The FileWriter.
	 */
	protected FileWriter getFileWriter(){
		return this.fw;
	}
	
	
	/**
	 *	Method which reviews whether a received task has a format that this TR understands. 
	 */
	public abstract boolean reviewProblem(String task);
	
	/**
	 *	Method returning true if the current problem is solved.
	 *	@return boolean True if problem solved, false else.
	 */
	public abstract boolean problemSolved();
	
	/**
	 *	Method for initializing which ontology this agent knows
	 */
	protected abstract Ontology intializeOntology();
	
		/**
	 *	Abstract method to get the ontology this agent knows of.
	 *	@return Ontology The ontology
	 */
	protected abstract Ontology getOntology();
	
	/**
	 * Method returning a list containing list of the different actions of the tmst and the agents performing these actions.
	 * Actions are stored in the list as Action objects and agents are stored as AIDs. Modified by Kari Roessland.
	 * @return Actions and the AID of all agents solving the actions, as described above in an ArrayList
	 */ 
	protected ArrayList getProblemSolvers(){
		ArrayList agents = new ArrayList(); //List containing the agents AID 
		ArrayList actionsAndAgents = new ArrayList(); //List containing all the different actions of the tmst and a list of the agents performing the action
		Action actions[] = this.tmst.getAllActions(); //The names of all the actions part of the tmst
		fw.write("\n    "+this.getLocalName()+ " needs to ask the Matchmaker for agents capable of actions in TMST");
		fw.write("    "+this.getLocalName()+" got a list of actions (action-types) - agents from the Matchmaker:");
		AID aid[] = null;
		
		
		for(int i = 0; i<actions.length; i++){
			ArrayList tempArray = new ArrayList();
			tempArray.add(actions[i]);
			aid = lookupProblemSolvers(actions[i]);
			tempArray.add(aid);
			actionsAndAgents.add(tempArray);
			
			try{
				//this print - anticipates that there are only one agent for each action
				if(aid == null || aid.length == 0){
					fw.write("    "+actions[i].getName()+" (" + actions[i].getType()+") - No available agents");
				}else{
					fw.write("      "+actions[i].getName()+" (" + actions[i].getType()+") - "+aid[0].getLocalName());
				}
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		return actionsAndAgents;
	}
	
	
	/**
	 *	Method that generates a vector of all CFPs needed for solving the decomsposed problem. Modified by Kari Roessland.
	 *	@param cfp a generic CFP for using as a recipe for generating new ones.
	 *	@param actionsAndAgents an ArrayList of the actions needed performing and all agents capable of performing them.
	 *	@return Vector of all the CFPs generated.
	 */
	protected Vector makeCfps(ACLMessage cfp, ArrayList actionsAndAgents){
		Vector cfpVector = new Vector();
		Date date = null;
		
		
		//Make proposals for every agent an action in the actionsAndAgents list.
		for(int i = 0; i<actionsAndAgents.size(); i++){
			//Current Action 
			Action currentAction = ((Action)((ArrayList)actionsAndAgents.get(i)).get(0));

			
			//ArrayList containing all agents which is capable of the current action
			AID agents[] = ((AID[])((ArrayList)actionsAndAgents.get(i)).get(1));			
		
			
			if(agents.length == 0){
				continue;
			}
			
			for(int j = 0; j < agents.length; j++){
				
				//Initialize the date to reply by as the time specified by getReplyByTime. Default == 5 secs
				date = new Date(System.currentTimeMillis() +this.getReplyByTime());

				//Fill all slots of the message
				ACLMessage tempCfp = (ACLMessage)cfp.clone();
				tempCfp.addReceiver(agents[j]);
				tempCfp.setContent(encodeCostOfActionRequest(agents[j], currentAction));
				tempCfp.setReplyByDate(date);
				//Add the newly created message to the vector of cfps this method returns
				cfpVector.add(tempCfp);
			}
		}
		return cfpVector;
	}

	/**
	 *	Method making the invitations used by the Form Team Behaviour as a part of the Contract Net protocol for forming a team.
	 *  Modified by Kari Roessland.
	 *	@param acceptances the messages representing proposals received by a subset of the agents receiving a CFP.
	 *	@return Vector containing all invitations to joining the team. If an exception occurs an empty Vector is returned.
	 */
	protected Vector makeInvitations(Vector responses){
	
		ACLMessage acceptance = null;
		ACLMessage rejection = null;
		ACLMessage response = null;
		AID receiver = null;
		String receiverName = "";
		String responseSenderName = "";
		String actionType = "";
		Action action;
		Action actions[] = tmst.getAllActions();
		ArrayList actionsPartOfSolution = new ArrayList();
		Vector invitations = new Vector();
		String content = "";

	
		fw.write("\n    "+this.getLocalName()+" makes invitations (accept-proposals) to join the team for the agents that");
		fw.write("    made a prosal and is mapped (through an executor) to the actions that are part of");
		fw.write("    the chosen solution. The other agents that made a proposal receives a refuse-proposal message.");
		fw.write("    An agent gets one inivtation(accept-proposal) or refuse-proposal for each of the prosals it has made.\n");
		
		
		//Get all actions part of the solution
		for(int i = 0; i < actions.length; i++){
			if(actions[i].isSolutionPart()){
				actionsPartOfSolution.add(actions[i]);				
			}
		}
		
		fw.write("    These agents receive invitations (accept-proposals), from "+this.getLocalName()+", to join the team:");	
		//Make invitations to the team to all agents executing actions part of the solution
		for(int i = 0; i < actionsPartOfSolution.size(); i++){
			acceptance = null;
			try{
				receiver =  new AID(((Action)actionsPartOfSolution.get(i)).getExecutor().getName());
				receiverName = receiver.getName();
				actionType = ((Action)actionsPartOfSolution.get(i)).getType().getName();
				
				for(int j = 0; j< responses.size(); j++){
					response = (ACLMessage)responses.get(j);
					responseSenderName = response.getSender().getName();
					content = response.getContent();
					
					if (responseSenderName == receiverName && response.getPerformative() == ACLMessage.PROPOSE &&
							content.indexOf(actionType) >= 0){
						acceptance = response.createReply();
						//remove the processed proposal from the vector
						responses.remove(j);
						break;
					}
				}
			}catch(AlreadyHasValueException e){
				e.printStackTrace();
				invitations = new Vector();
				return invitations;
			}catch(Exception e){
				e.printStackTrace();
				invitations = new Vector();
				return invitations;
			}
		
			
			if(acceptance == null){
				fw.write("      "+this.getLocalName()+" did not find response-message from receiver of acceptance");
			}else{
				acceptance.setProtocol(FIPANames.InteractionProtocol.FIPA_CONTRACT_NET);
				acceptance.setPerformative(ACLMessage.ACCEPT_PROPOSAL);
				//Set id
				int teamID = tmst.getTeamID();
			
				//Generate appropriate content 
				content = "(action (:name " +actionType +") (:teamID " + teamID +"))";
				
				//Set content
				acceptance.setContent(content); 
				
				fw.write("      Agent: "+receiver.getLocalName()+"  Invitation: " +content);
				invitations.add(acceptance);
			}
		
		}
		
		//The agents sending proposals that were not accepted receives a reject-proposal message. All the proposals
		//that were accepted were removed from the responses-vector as they were used to make accept-proposal messages
		//meaning that the remaining messages in the responses-vector are proposals that are rejected.
		
		
		
		fw.write("\n    These agents receives reject-proposals, saying they don't get to join the team, from "+this.getLocalName()+":");
		for(int i = 0; i<responses.size();i++){
			response = (ACLMessage)responses.get(i);
			rejection = response.createReply();
			rejection.setProtocol(FIPANames.InteractionProtocol.FIPA_CONTRACT_NET);
			rejection.setPerformative(ACLMessage.REJECT_PROPOSAL);
			invitations.add(rejection);
			//this is a cheesy solution... to get the action in the proposal.
			try{
				actions = getActionsMatchingProposal(response);
				fw.write("      Agent: "+response.getSender().getLocalName()+"  Action in proposal: "+actions[0].getType());
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		fw.write("");
			
		return invitations;
	}
	
	/**
	 *	Method extracting the price of a proposal received. Price is represented using Float. 
	 *  This method returns the String representation of the price. The price (in the ontology) supports currency. 
	 *  Currency is not supported by the TMST and thus is not used. Modified by Kari Roessland.
	 *	@param proposal The message containing the proposal.
	 *	@return A String representation of the price. 
	 */
	private String extractPrice(ACLMessage proposal){
		//Decalare and intialize all variables needed by this method
		SLCodec sl = new SLCodec();
		AbsContentElement content = null;
		AbsPredicate equals = null;
		AbsObject price = null;
		AbsPrimitive value = null;
		
		if(!proposal.getLanguage().equals(sl.getName())){
			fw.write(" |-- TaskResponsible "+this.getLocalName()+" only understand FIPA-SL.\nCurrent message is not encoded using SL.");
			return null;
		}
		
		if(!proposal.getOntology().equals(getOntology().getName())){
			fw.write(" |-- TaskResponsible "+this.getLocalName()+" only understand the ontology: " +getOntology().getName() +". \nCurrent message is not encoded using that ontology.");
			return null;
		}		
		
		try{
			content = sl.decode(getOntology(), proposal.getContent());
		}catch(CodecException e){
			e.printStackTrace();
		}
		
		//Check whether content is an instance of AbsPredicate
		if(!(content instanceof AbsPredicate)){
			return null;
		}
		//Cast content to abspredicate
		equals = (AbsPredicate)content;
		
		//check whether prediacate really is equals. If not return null.
		if(!equals.getTypeName().equals(SLVocabulary.EQUALS)){
			return null;
		}
		
		//If equals, should have both left and rigt side. Only need left side to get all actions. Left side should be an IRE
		if(!(equals.getAbsObject(SLVocabulary.EQUALS_RIGHT) instanceof AbsObject)){
			return null;
		}

		//Get the price object
		price = (AbsObject)equals.getAbsObject(SLVocabulary.EQUALS_RIGHT);
		
		//Check whether price is of type PRICE
		if(!price.getTypeName().equals(CheckUpOntology.PRICE)) return null;

		//Check whether the price value is an instance of absPrimitive
		if(!(price.getAbsObject(CheckUpOntology.PRICE_VALUE) instanceof AbsPrimitive)) return null;
		 
		//Get the value
		value = (AbsPrimitive)price.getAbsObject(CheckUpOntology.PRICE_VALUE);
		
		float floatValue = value.getFloat();

		//Return the string representation of the value (which is a float)
		return Float.toString(floatValue);
	}

	private AbsObject getSLContentAsAbsObject(Ontology ont, String content) throws Exception{
		AbsContentElement absContent = null;
		AbsPredicate absPredicate = null;
		SLCodec sl = new SLCodec();
		String toEncode = "((= " +content +" true))";
		//(equals. 
		try{
			absContent = sl.decode(ont, toEncode);
		}catch(CodecException e){
			e.printStackTrace();
			throw new Exception(e.getMessage());
		}
		absPredicate = (AbsPredicate)absContent;
		return absPredicate.getAbsObject(SLVocabulary.EQUALS_LEFT);
	}
	
	
	/**
	 *	Method matching two abstract objects. Returns true uf they are of the same type and their attributes have the same value.
	 *	@param abs1 
	 *	@param abs2
	 *	@return boolean Returns true if abs1 and abs2 match
	 */
	private boolean matchAbsObjects(AbsObject abs1, AbsObject abs2){
		//If not of the same type, then they are not equal
		if(!abs1.getTypeName().equals(abs2.getTypeName())) return false;
		
		String attributes[] = abs1.getNames();
		
		//If abs1 does not have any attributes, check whether this is the case with abs2. If not the case, return false
		if(attributes == null){
			String attributes1[] = abs2.getNames();
			if(attributes1 == null) return true;
			else return false;
		}
		
		//For all attributes of abs1, check it they are similar to abs2s attributes
		for(int i = 0; i < attributes.length; i++){
			//Try to get the attribute from abs2
			AbsObject currentAttribute = abs2.getAbsObject(attributes[i]);
			
			//If null is returned, the attribute does not exist in abs2, thus they do not match
			if (currentAttribute == null) return false;
			
			//If the attributes are not primitives, do further matching
			if (!(currentAttribute instanceof PrimitiveSchema)){
				if(!matchAbsObjects(abs1.getAbsObject(attributes[i]), currentAttribute)) return false;
			}else{
				//Cast the two attributes as AbsPrimitives as they both should be instances of the class
				AbsPrimitive absPrim1 = (AbsPrimitive)abs1.getAbsObject(attributes[i]);
				AbsPrimitive absPrim2 = (AbsPrimitive)currentAttribute;
				
				//Match the string values of the two absPrimitives. If the primitives are not equal, they do not match. Thus, return false.
				if (absPrim1.getString().equals(absPrim2.getString())) return false;
			}
		}
		
		//If iterating through all attributes does not make the function return, then the two absObjects are equal.
		//Which gives us a reason to celebrat. Hooray!
		return true;
	}
	
	/**
	 *	Metod returning all actions fulfilling the description of the action referenced in the proposal.
	 *  Modified by Kari Roessland.
	 *	@param proposal The ACLMessage containing the proposal
	 *	@return All actions fulfilling the proposal
	 */
	private Action[] getActionsMatchingProposal(ACLMessage proposal){
		//Decalare and intialize all variables needed by this method
		SLCodec sl = new SLCodec();
		AbsContentElement content = null;
		AbsPredicate equals = null;
		AbsIRE ire = null;
		AbsPredicate predicate = null;
		AbsObject absAction = null;
		AbsAggregate input = null;
		AbsObject output = null;
		AbsObject assumptions = null;
		ArrayList actionsMatchingPropose = new ArrayList();
		Action actions[] = null;
		
		if(!proposal.getLanguage().equals(sl.getName())){
			fw.write("    "+this.getLocalName()+" only understands FIPA-SL - found a propose message that is not encoded using SL");
			return null;
		}

		if(!proposal.getOntology().equals(getOntology().getName())){
			fw.write("    "+this.getLocalName()+" only understands the ontology: " +getOntology().getName() +"\n    - found a propose message that is not encoded using that ontology");
			return null;
		}		
		
		try{
			content = sl.decode(getOntology(), proposal.getContent());
		}catch(CodecException e){
			e.printStackTrace();
		}
		
		//Check whether content is an instance of AbsPredicate
		if(!(content instanceof AbsPredicate)){
			return null;
		}
		//Cast content to abspredicate
		equals = (AbsPredicate)content;
		
		//check whether prediacate really is equals. If not return null.
		if(!equals.getTypeName().equals(SLVocabulary.EQUALS)){
			return null;
		}
		
		//If equals, should have both left and rigt side. Only need left side to get all actions. Left side should be an IRE
		if(!(equals.getAbsObject(SLVocabulary.EQUALS_LEFT) instanceof AbsIRE)){
			return null;
		}

		ire = (AbsIRE)equals.getAbsObject(SLVocabulary.EQUALS_LEFT);

		//The IRE should be an iota, if not return null
		if(!ire.getTypeName().equals(SLVocabulary.IOTA)){
			return null;
		}
		
		//Must be a predicate (Cost)
		if(!(ire.getProposition() instanceof AbsPredicate)){
			return null;
		}
		predicate = (AbsPredicate)ire.getProposition();
		
		//Only considers messages asking for price of an action (Cost predicate)
		if(!predicate.getTypeName().equals(CheckUpOntology.COST)){
			return null;
		}
		
		//Get the action the cost object refer to
		absAction = predicate.getAbsObject(CheckUpOntology.COST_ACTION);
		if(absAction == null){
			return null;
		}
		
		//Get input, output and assumptions as specified by the action in the propsal
		input = (AbsAggregate)absAction.getAbsObject(CheckUpOntology.CHECKUP_ACTION_INPUT);
		output = absAction.getAbsObject(CheckUpOntology.CHECKUP_ACTION_OUTPUT); 
		assumptions = absAction.getAbsObject(CheckUpOntology.CHECKUP_ACTION_ASSUMPTIONS);
		
		//Get all actions of the same type as the one referenced in the proposal
		actions = getTMST().getActions(absAction.getTypeName());
		
		//Have to check whether the actions found satisfy the specifications of the action in the proposal (input, output, assumptions)
		for(int i = 0; i < actions.length; i++){
			//Variable keeping track of whether the two actions match
			boolean doMatch = true;
			boolean inputMatch = true;

			//Check whether input match
			if(input != null){	
				Input currentActionsInputs[] = actions[i].getInputs();
				
				//For each of the current actions input, check against all inputs of the action described by the proposal
				for(int j = 0; j < input.size(); j++){
					for(int k = 0; k < currentActionsInputs.length; k++){
						try{
							AbsObject currentValue = getSLContentAsAbsObject(getOntology(), currentActionsInputs[k].getValue().getName());
							if(matchAbsObjects(currentValue, input.get(j))){
								inputMatch = true;
								continue;
							}
						}catch(AlreadyHasValueException e){
							e.printStackTrace();
						}catch(CodecException e){
							e.printStackTrace();
						}catch(Exception e){
							e.printStackTrace();
						}
					}
					if(inputMatch == false) break;
				}
			}
			
			if(inputMatch == false) continue;
			
			//Check whether outputs match
			if(output != null) {
				try{
					AbsObject currentActionOutput =  getSLContentAsAbsObject(getOntology(), actions[i].getOutput().getValue().getName());
					
					if(matchAbsObjects(currentActionOutput, output)){
						doMatch = true;
					}
					else doMatch = false;
				}catch(Exception e){
					e.printStackTrace();
				}
			}
			if(doMatch == false) continue;
			//The current action match the propose action, put it in the array list
			if(doMatch == true) actionsMatchingPropose.add(actions[i]);
		}
		return (Action[])actionsMatchingPropose.toArray(new Action[actionsMatchingPropose.size()]);
	}
	
	/**
	 * Method taking a ACLMessage (a proposal) and maps the sender agent to the action in the tmst the agent has volunteered for.
	 * This method is modified by Kari Roessland: If an executor with the name of the sender-agent of the response-message does not exist,
	 * this one is created. Than the executor (newly created og pre-existing) is related to the action of the response-message.
	 * @param response
	 */	
	protected void mapAgentToExecutor(ACLMessage response) throws Exception{
	
		String actionName = "";
		String costName = "";
		String executorName = "";
		String executorLocalName = "";
		String reference = "";
		String cost = "";
		Executor executor = null;
		Entity ent = null;
		
		Action actionsMatchingPropose[] = getActionsMatchingProposal(response);
		if(actionsMatchingPropose == null) return;

		cost = this.extractPrice(response);
		
		if(cost == null){
			return;
		}
		
		executorName = response.getSender().getName();
		executorLocalName = response.getSender().getLocalName();
		reference = response.getInReplyTo();
		
		//Iterate through all actions mathing the proposed one, make the sender of the response executor of all of these actions
		for(int i = 0; i < actionsMatchingPropose.length; i++){
			actionName = actionsMatchingPropose[i].getName();
			costName = response.getSender().getLocalName() +"-" +actionName +"-cost";
			try{
				ent = tmst.getEntity(executorName);
				
				if(ent == null){
					//Add an executor to the tmst representing the current agent 
					//The executor does not exist and has to be created all over
					new Executor(tmst, executorName, "executor", actionsMatchingPropose[i], cost, reference);
					fw.write("    Executor: "+executorLocalName+" - Action: "+actionName+" - Cost: "+cost);
				}else{
					//The executor does exist and does not have to be created all over
					executor = new Executor(ent);
					executor.addAction(actionName, cost, reference);
					fw.write("    Executor: "+executorLocalName+" - Action: "+actionName+" - Cost: "+cost);
				}
			}catch(NameAlreadyExistException e){
				e.printStackTrace();
				
			}catch(EntityTypeNotMatchedException e){
				e.printStackTrace();
			}		
		}
	}
	
	/**
	 *	Method returning the AID of a decomposer. It picks one if more than one exist - the first one in the list returned.
	 *	@return AID The AID to the selected decomposer
	 */
	public AID getDecomposer(){
		DFAgentDescription possibleAgents[] = lookup("DECOMPOSER");
		
		//We always choose the first agent which match the description
		//The selwection of decomposer may (and shuold) be done otherwise
		if(possibleAgents==null || possibleAgents.length == 0){
			return null;
		}
		return possibleAgents[0].getName();
	}
	
	/**
	 * Method returning the AID of a TeamSpaceAgent. It picks one if more than one exist - the first one in the list returned.
	 * New method added by Kari Roessland.
	 * @return The AID to the selected decomposer
	 */
	public AID getTeamSpaceAgent(){
		DFAgentDescription possibleAgents[] = lookup("TEAM_SPACE_COORDINATOR");
	
		if(possibleAgents == null || possibleAgents.length == 0){
			return null;
		}else{
		
			//hard coding..., for our the two CheckUpExamples.
			if(possibleAgents.length > 1 && this.getTask().indexOf("maria") >= 0){
				return possibleAgents[0].getName();
			}else if(possibleAgents.length > 1 && this.getTask().indexOf("paul") >=0){
				return possibleAgents[1].getName();
			}else{
				return possibleAgents[0].getName();
			}
		}
	}
	
	
	/**
	 *	Method for setting time to wait for starting handling all of the responses from the agents which have been sent cfps. Standart is five seconds (5000 milliseconds).
	 *	@param waitForMillis The time allowed to wait before answering the message
	 */
	protected void setReplyByTime(long waitForMillis){
		this.waitForMillis = waitForMillis;
	}
		
	/**
	 *	Method for getting the time to wait before starting to handle all responses collected in the Contract net part of the Team Formation process step. Default is five seconds.
	 *	@return long Milliseconds to wait
	 */
	protected long getReplyByTime(){
		return this.waitForMillis;
	}
	
	/**
	 *	Method returning all problem solvers registered in the system which carry out a specified actiontype
	 *	@param actiontype Actiontype describing the problem solving capability of an problem solving agent.
	 *	@return AID[] A list containing the AID of all the problem solvers
	 */
	protected AID[] lookupProblemSolvers(Action action){
		//Initialize the instance variables
		DFAgentDescription[] result = null;
		AID pss[] = null;
		String actiontype = null;
		Output output = null;
		Input inputs[] = null;
		
		//initialize the output and actiontype values so that they correspond with those which apply for the current action.
		try{
			inputs = action.getInputs();
			output = action.getOutput();
			actiontype = action.getType().getName();
		}catch(Exception e){
			fw.write("    Something is terribly wrong, an action must have a type");
			e.printStackTrace();
			return null;
		}
		
		//Has to specify the services describing the current action
        DFAgentDescription dfd = new DFAgentDescription();
        ServiceDescription sd = new ServiceDescription();
        sd.setType("PROBLEM_SOLVER"); 
		sd.addProperties(new Property("action-type", actiontype));
		
		try{
			sd.addProperties(new Property("output", output.getValue().getName()));	
		}catch(AlreadyHasValueException e){
			e.printStackTrace();
		}
	
		//Add the service description objet to the DF Agent Description
        dfd.addServices(sd);
		
        //The specified service has to be looked up in the DF
        try{
			result = DFService.search((Agent)this, dfd);
        }catch(Exception e){
            System.out.println("    "+this.getLocalName()+" is not able to register with DF (Matchmaker)fw.write. Shutting down ...");
            e.printStackTrace();
        }
		
		//If there are no agents that offer a service which may be used for performing the current action, return a null reference.
		if(result==null) return null;

		//Initialize an array of AID which is being returned
		pss = new AID[result.length];
		
		for(int i = 0; i<result.length; i++){
			pss[i] = result[i].getName();
		}
		
		//Return all agents offering services which may describe the current action
		return pss;
	}
	
	/**
	 *	Method encoding the content for using in messages of the following kind: ((iota ?x (action (agent-identifier AGENT) (ACTION :input VALUE :output VALUE :assumption VALUE :cost ?x))). 
	 *	Where AGENT equals the parameter agent and ACTION and the VALUE slots are found in the Action parameter. The meaning of the message is "how much does the specified action cost?"
	 *	@param agent The agent which is performing the requested action
	 *	@param action The action to be performed
	 *	@return String 
	 */
	private String encodeCostOfActionRequest(AID receiver, Action action){
		StringBuffer sb = new StringBuffer("((");
		sb.append(SLVocabulary.IOTA);
		sb.append(" ?X (Cost ");
		
		sb.append(encodeAction(receiver, action));
		
		sb.append("?X)))");
		return sb.toString();
	}

	/**
	 *	Method encoding action descriptions to string
	 *	@param receiver The receiver of the action
	 *	@param action The action to encode
	 *	@return String The string encoding of an action
	 */
	private String encodeAction(AID receiver, Action action){
		StringBuffer sb = new StringBuffer("(");		
		try{
			sb.append(action.getType().getName());
		}catch(Exception e){
			e.printStackTrace();
		}
		
		//iterate through all inputs of the action
		Input inputs[] = action.getInputs();
		
		
		if(inputs != null && inputs.length != 0){
			sb.append(" :input");
			
			//Iterate through all of the input, and append their values to to the string
			for(int i = 0; i < inputs.length ; i++){
				try{
					sb.append(" ");
					sb.append(inputs[i].getValue().getName());
				}catch(AlreadyHasValueException e){
					e.printStackTrace();
				}
			}
		}
		
		//iterate through all outputs of the action
		try{
			Output output = action.getOutput();
		
			if(output != null){
				sb.append(" :output ");
				sb.append(output.getValue().getName()); 
			}
		}catch(AlreadyHasValueException e){
			e.printStackTrace();
		}catch(Exception e){
			e.printStackTrace();
		}
		
		//iterate through all assumptions of the action
		Assumption assumptions[] = action.getAssumptions(); //null;
		
		if(assumptions != null && assumptions.length != 0){
			sb.append(" :assumptions");
			
			//Iterate through all of the assumptions, and append their values to to the string
			for(int i = 0; i < assumptions.length ; i++){
				try{
					sb.append(" ");
					sb.append(assumptions[i].getValue().getName());
				}catch(AlreadyHasValueException e){
					e.printStackTrace();
				}catch(Exception e){
					e.printStackTrace();
				}
			}
		}
		sb.append(")");
		return sb.toString();
	}
	
	/**
	 * Method looking up whether there exists any Problem Solvers in the system.
	 * @param agentType String describing the agentType/Service requested.
	 */
    private DFAgentDescription[] lookup(String agentType){
		//return value
		DFAgentDescription[] result = null; 

        //Has to specify the services we are looking for
        DFAgentDescription dfd = new DFAgentDescription();
        ServiceDescription sd = new ServiceDescription();
        sd.setType(agentType);
        dfd.addServices(sd);
		
        //The specified service has to be looked up in the DF
        try{
			result = DFService.search((Agent)this, dfd);
        }catch(Exception e){
            e.printStackTrace();
        }
		return result;
    }
	
	/**
	 *	Method for setting the tmst describing the possible solutions for the task which is hopefully being solved.
	 *  Modified by Kari Roessland. Contains som hard coding that should be removed in a later version.
	 *	@param tmst The task structure tree which describes the possible solutions for the problem at hand
	 */
	protected void setTMST(TMST tmst){
		//hard coding...
		
		if(task.compareTo("(Task :name doCheckup :input (Patient :name paul))") == 0){
			tmst.setInitialInput("(Patient :name paul)");
		}
		if(task.compareTo("(Task :name doCheckup :input (Patient :name maria))") == 0){
			tmst.setInitialInput("(Patient :name maria)");
		}		
		this.tmst = tmst;
		
		tmst.setFileWriter(fw);
	}
	
	/**
	 *	Method returning the tmst describing the possible solutions for the current problem being solved
	 *	@return TMST The task structure tree describing possible solutions for the current problem
	 */
	protected TMST getTMST(){
		return this.tmst;
	}

	
	/**
	 *	This method should be implemented as a protected abstract method which has to be defined by the language the problem is formated as. The solution should be specified in the same language to be shure that the requesting agent understands it (this is not the consulting business).
	 *	@return String The language the solution is specified by, as string.
	 */
	protected String getSolutionLanguage(){
		return FIPANames.ContentLanguage.FIPA_SL;
	}
	
	/**
	 *	Method returning the solution of the problem being solved
	 *	@return String A string representation of the solution, specified in the language returned by the method getSolutionLanguage().
	 */
	protected String getSolution(){
		return this.solution;
	}

	/**
	 *	Method for storing the solution of a problem. The solution is represented using a string.
	 *	@param solution The solution to be stored.
	 */
	protected void setSolution(String solution){
		this.solution = solution;
	}
	
	/**
	 *	Method setting the response for the agent which the CoPSTR works for.
	 *	@param response The response to give
	 */
	protected void setRequest(ACLMessage request){
		this.request = request;
	}
	
	/**
	 *	Method for getting the request gotten from the agent which this agent works for
	 */
	protected ACLMessage getRequest(){
		return this.request;
	}

	/**
	 *	Method setting the response for the agent which the CoPSTR works for.
	 *	@param response The response to give
	 */
	protected void setResponse(ACLMessage response){
		this.response = response;
	}
	
	/**
	 *	Method returning the response stored
	 *	@return ACLMessage The message which is is a response to the agent the CoPSTR works for.
	 */
	protected ACLMessage getResponse(){
		return this.response;
	}
	
	/**
	 *	Method for setting the current task.
	 *	@param task A string representation of the task being solved
	 */
	protected void setTask(String task){
		this.task = task;
	}
	
	/**
	 *	Method for getting the current task which the agent is working with.
	 *	@return String String describing the current task the agent works on solving.
	 */
	protected String getTask(){
		return this.task;
	}
	
	/**
	 *	Method specifying how to compose the psms. New method added by Kari Roessland.
	 *	@param request The request specifying the psm requested
	 *	@return ACLMessage The message specifying the solution
	 */
	protected abstract ACLMessage composePSM(ACLMessage request);
	
	
	/**
	 * Method returning the last PSM to compose. New method added by Kari Roessland.
	 * @return The most top-level PSM in the TMST as a String
	 */
	protected abstract String getFinalPSM();
}