package CoPS.agents;
//
//  ComposePSMBehaviour.java
//  CoPS
//
//  Created by Kari on 18.04.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

//jade
import jade.core.Agent;
import jade.proto.AchieveREResponder;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.core.behaviours.Behaviour;


/**
 * This class assist the task responsible agent in composing partial results connected to a psm 
 * @author Kari Roessland
 */
public class ComposePSMBehaviour extends AchieveREResponder{
	MessageTemplate template;
	String currentRequest;
	
	/**
	 * The constructor only calls the super-constructor
	 * @param agent The owner of this behaviour
	 * @param template A message template that defines the construct of the incoming request message.
	 */
	public ComposePSMBehaviour(Agent agent, MessageTemplate template){
		super(agent, template);
		this.template = template;
	}
	
	/**
	 * In this method the agent desides if it can perform the action and prepares a response to
	 * the incoming request message.
	 * @param request The incoming request message.
	 */
	protected ACLMessage prepareResponse(ACLMessage request){
		
		String psm = request.getContent();
		ACLMessage response = request.createReply();
		currentRequest = psm;
		
		
		//check if the agent understands the content, temporarily check if it is not null
		if(psm != null && psm.compareTo("") != 0){
			response.setPerformative(ACLMessage.AGREE);
		}else{
			response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
		}
		
		return response;
	}
	
	/**
	 * The agent prepares the output from the action and returns it to the requester (TeamSpaceAgent).
	 * @param request The request message.
	 * @param response The response message to the request message, that was sent before.
	 * @return An inform message or a failure message.
	 */
	protected ACLMessage prepareResultNotification(ACLMessage request, ACLMessage response){	
	
		response = ((CoPSTaskResponsible)myAgent).composePSM(request);
		
		
	
		return response;
	}
	
	/**
	 * Information from the last execution of this behaviour is reset. Checks if the composed
	 * PSM is the last one. If it is - do not wait for any more PSMs, else add this behaviour
	 * to the agent's behaviour pool once more (keep waiting for new PSMs to compose).
	 * @return The transition from this state/behaviour.
	 */
	public int onEnd(){
		reset();
		if(currentRequest.indexOf(((CoPSTaskResponsible)myAgent).getFinalPSM()) >= 0){
			myAgent.addBehaviour(new ComposePSMBehaviour(myAgent, template));
		}
			
		return 0;
	}
}
