package CoPS.CheckUpExample;
//
//  DiagnosisAgent.java
//  CoPS
//
//  Created by Kari on 11.04.06
//

//JADE classes
import jade.domain.FIPANames;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.domain.FIPAAgentManagement.Property;
import jade.lang.acl.ACLMessage;
import jade.content.lang.Codec;
import jade.content.lang.leap.LEAPCodec;
import jade.content.onto.Ontology;
import jade.content.abs.AbsContentElement;
import jade.content.abs.AbsIRE;
import jade.content.abs.AbsObject;
import jade.content.abs.AbsPredicate;
import jade.content.abs.AbsVariable;
import jade.content.abs.AbsConcept;
import jade.content.ContentElement;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Equals;
import jade.content.lang.sl.SLVocabulary;

//CoPS classes
import CoPS.CheckUpExample.*;
import CoPS.agents.CoPSProblemSolver;

/**
 * A class extending CoPSProblemSolver, which implements a problem solver in 
 * our Checkup example application.
 * @author Kari Roessland
 */
public class DiagnosisAgent extends CoPSProblemSolver{
	
	/**
	 * Method overridden from the class CoPSAgent.
	 * @return The agents ServiceDescription object to be registered with the DF.
	 */
	public ServiceDescription setServiceDescription(){
	
		//All problem solving agents has (at least) a actiontype they offer 
		Property prop1 = new Property("action-type", "GetAlternativeDiagnosesDB");
		Property prop2 = new Property("output", "(Patient :name x :diagnoses (Diagnose :description k))");
		
		//Initialize the service description describing this agent's services
        ServiceDescription sd = new ServiceDescription();
        sd.setType("PROBLEM_SOLVER");
		sd.setName(this.getName());
        sd.setOwnership("KariR");
		sd.addProperties(prop1);
		sd.addProperties(prop2);
		
		return sd;
	}
	
	/**
	 * Method computing the price of the action performed by this agent.
	 * @return An AbsConcept object describing the price.
	 */
	private AbsConcept getPrice(){
		double maxPrice = 12.0d;
		Float priceAsFloat = new Float(Math.random() * maxPrice);
		
		AbsConcept price = new AbsConcept("Price");
		price.set(CheckUpOntology.PRICE_VALUTA, "GBP");
		price.set(CheckUpOntology.PRICE_VALUE, priceAsFloat.floatValue()); 
		return price;
	}
	
	/**
	 * This method is not an example of how this parcing ought to be done. It is just a mere dummy-method
	 * for creating a response which is working with one specially designed example.
	 * @param request The call-for-proposal message received from CheckUpTaskResponsible.
	 * @return A string describing the proposal.
	 */
	protected String makeProposal(ACLMessage request){
		//Declare and initialize the variables needed
		SLCodec sl = new SLCodec();
		AbsContentElement abs = null;
		AbsIRE ire = null;
		AbsPredicate predicate = null;
		AbsVariable absVariable = null;
		AbsObject absAction = null;
		AbsObject absPrice = null;
		String proposal = null;
		
		String requestString = "";
		
		//If the request is not encoded using fipa-sl, cannot understand it and returns null
		if(!request.getLanguage().equals(sl.getName())) return null;
		
		try{
			//debug
			requestString = request.getContent();
					
			//I only understand messages of the type (iota ?x (Cost :action (Load) :price ?x))
			//The following code checks whether, the request content is as illustrated above
			//Then an answer is made, which is like: (= (iota ?x (Cost :action (Load) :price ?x)) (Price :value 12.0 :valuta GBP))
			//If the price is found to be 12.0 GBP...
			
			
			abs = sl.decode(CheckUpOntology.getInstance(), request.getContent());
			
			//I only understand ire
			if(abs instanceof AbsIRE) ire = (AbsIRE)abs;
			else return null;
			
			
			//Must be an iota message
			if(!ire.getTypeName().equals("iota")) return null;
			
			//Must be a predicate (Cost)
			if(ire.getProposition() instanceof AbsPredicate) predicate = (AbsPredicate)ire.getProposition();
			
			//Only considers messages asking for price of an action (Cost predicate)
			if(!predicate.getTypeName().equals(CheckUpOntology.COST))	return null;
			
			//Get the action the cost object refer to
			absAction = predicate.getAbsObject(CheckUpOntology.COST_ACTION);
			if(absAction == null){
				return null;
			}
			
			//Must check whether the request is for the agent action Load, which is what I know
			if(absAction.getTypeName().equals(CheckUpOntology.GET_ALTERNATIVE_DIAGNOSES_DB)){
				//proceed
			}else{
				return null;
			}
			
			//Get the action the cost object refers to
			absPrice = predicate.getAbsObject(CheckUpOntology.COST_PRICE);
			if(absPrice == null){
				return null;
			}
			
			//Get the variable
			absVariable = ire.getVariable();
			
			//If the variable is the price, we will answer this message
			if(!absVariable.toString().equals(absPrice.toString())) return null;
			
			//The request is a question of what price I take for performing action load
			//Create the reply
			
			AbsPredicate equals = new AbsPredicate(SLVocabulary.EQUALS);
			equals.set(SLVocabulary.EQUALS_LEFT, ire);
			equals.set(SLVocabulary.EQUALS_RIGHT, getPrice());
			
			proposal = sl.encode(CheckUpOntology.getInstance(), equals);
		}catch(Codec.CodecException e){
			e.printStackTrace();
			return null;
		}
		
		
		return proposal;
	}

	/**
	 * This method is used to decide whether the agent should make a proposal or not.
	 * @param request The call-for-proposal message.
	 * @return True or false.
	 */	
	protected boolean deceideWhetherToMakeProposal(ACLMessage request){
		return true;
	}
	
	/**
	 * This method initializes the agent's languages.
	 * @return A list of Codec objects.
	 */	
	protected Codec[] initializeLanguages(){
		Codec langs[] = new Codec[1];
		langs[0] = new SLCodec();		
		return langs;
	}
	
	/**
	 * Method for setting the ontologies this agent knows of.
	 * @return A list of Ontology objects.
	 */
	protected Ontology[] initializeOntologies(){
		Ontology onto[] = {CheckUpOntology.getInstance()};
		return onto;
	}
	
	/**
	 * The method that is called when the agent has received a request for performing
	 * an action from the TeamSpaceAgent.
	 * @param request The request message.
	 * @return A message describing the output from performing the action.
	 */
	public ACLMessage performAction(ACLMessage request){
		ACLMessage response = request.createReply();
		response.setLanguage("plain-text");
		String content = request.getContent();
		if(content.indexOf("paul") >= 0){
			response.setContent("GetAlternativeDiagnosesDB-(Patient :name paul :diagnoses (Diagnose :description diagnose1))");
		}
		if(content.indexOf("maria") >= 0){
			response.setContent("GetAlternativeDiagnosesDB-(Patient :name maria :diagnoses (Diagnose :description diagnose1))");
		}
		response.setPerformative(ACLMessage.INFORM);
		return response;
	}
}



