package CoPS.agents;
//
//  FailureSpecificationBehaviour.java
//  CoPS
//
//  Created by Kari on 02.02.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

import jade.core.Agent;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.DataStore;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;

import CoPS.FileWriter;

/**
 * This represents the failure-state of the FSM represented by PrepareResultBehaviour. The goal of
 * the PrepareResultBehaviour is to place the result of the problem solving process in the datastore
 * of the HandleProblemSovlingRequestBehaviour. If the one of the processes used for preparing this result
 * fails, a FAILURE message should be placed in the HandleProblemSovlingRequestBehaviour datastore instead.
 * In this way our agent will let the agent requesting the solution of a problem know that the problem
 * solving process failed, and that an answer to the problem is not found.
 * 
 * @author Kari Røssland
 *
 */
public class FailureSpecificationBehaviour extends OneShotBehaviour {

	private Behaviour dataStoreBehaviour;
	private FileWriter fw = null;
	
	/**
	 * The constructor.
	 */
	public FailureSpecificationBehaviour() {
		super();
	}

	/**
	 * The constructor.
	 * @param a The owner of this behaviour.
	 * @param datastor The datastore belonging to this behaviour.
	 * @param topBehaviour The behaviour from where this behaviour is added to the agent a.
	 */
	public FailureSpecificationBehaviour(Agent a, DataStore datastore, Behaviour topBehaviour) {
		super(a);
		setDataStore(datastore);
		dataStoreBehaviour = topBehaviour;
	}

	/**
	 * Action method. Adding the failure message to the right datastore, belonging to 
	 * HandleProblemSolvingRequestBehaviour.
	 */
	public void action() {
		fw = ((CoPSTaskResponsible)myAgent).getFileWriter();
		
		ACLMessage response = ((ACLMessage) getDataStore().get(((HandleProblemSolvingRequestsBehaviour)dataStoreBehaviour).REQUEST_KEY)).createReply();
		response.setPerformative(ACLMessage.FAILURE);
		getDataStore().put(((HandleProblemSolvingRequestsBehaviour)dataStoreBehaviour).RESULT_NOTIFICATION_KEY,response);
		
		fw.write("    "+myAgent.getLocalName()+" has entered the failure specification (behaviour) ...");
		System.out.println("    "+myAgent.getLocalName()+" failed to solve the problem, the requester of the problem is informed.");
	}
	
	/**
	 * This method is called when the behaviour has finished.
	 * @return The transition from this method.
	 */
	public int onEnd(){
		reset();
		return 0;
	}

}
