package CoPS.agents;
//
//  FormTeamBehaviour.java
//  CoPS
//
//  Created by Kari on 01.02.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

//java classes
import java.util.Vector;
import java.util.ArrayList;

//jade classes
import jade.proto.ContractNetInitiator;
import jade.core.Agent;
import jade.lang.acl.ACLMessage;
import jade.core.behaviours.DataStore;
import jade.domain.FIPANames;
import jade.core.AID;

//jcreek classes
import jcreek.representation.NameAlreadyExistException;
import CoPS.FileWriter;

/**
 *	Behaviour describing the process of forming a team. 
 *  Team formation achieved through the use of the contract net protocol. Modified by Kari Roessland.
 *
 */
public class FormTeamBehaviour extends ContractNetInitiator{
	private int transition = PrepareResultBehaviour.TEAM_FORMATION_FAILED;
	private FileWriter fw = null;
	
	/**
	 * The constructor only calls the constructor of the superclass of FormTeamBehaviour - ContracntNetInitiator
	 * @param agent The owner of this behaviour.
	 * @param cfp The default call-for-proposal message.
	 */
	public FormTeamBehaviour(Agent agent, ACLMessage cfp){
	
		//kari: lagt til new Datastore()
		super(agent, cfp, new DataStore());
		
	}

	/**
	 * Method preparing all call for proposal messages being sent. Modified by Kari Roessland.
	 * @param cfp The default call-for-proposal message.
	 * @return A vector containing all of the call-for-proposal messages being sent.
	 */
	protected Vector prepareCfps(ACLMessage cfp){
		fw = ((CoPSTaskResponsible)myAgent).getFileWriter();

		//List containing lists of all the actions (specified in the tmst describing the problem at hand) and all agents capable of performing these actions.
		//Thus, first element in each list is the action, the next element in each list contains a list of all agents capable of performing the action.
		ArrayList actionsAndAgents = ((CoPSTaskResponsible)myAgent).getProblemSolvers();


		fw.write("\n    "+myAgent.getLocalName()+" uses the Contract Net Protocol to form the team of problem solvers");
		fw.write("    "+myAgent.getLocalName()+" made Call for Proposals (CFPs) for all agents received from the Matchmaker");
		fw.write("    Agents and content of the CFP-messages are:");
		//Initialize the vector which will contain all the cfps, and will be returned by this method.
		Vector cfpVector = ((CoPSTaskResponsible)myAgent).makeCfps(cfp, actionsAndAgents);
		
		ACLMessage message = null;
		for(int i = 0; i<cfpVector.size(); i++){
			message = (ACLMessage)cfpVector.get(i);
			fw.write("      Agent: "+((AID)(message.getAllReceiver().next())).getLocalName());
			fw.write("      CFP  : "+message.getContent());
		}
		fw.write("");
		
		//If the cfpVector is empty, the current configuration of the system have no solution to the initial problem
		if(cfpVector.isEmpty()){
			transition = PrepareResultBehaviour.TEAM_FORMATION_FAILED;
		}
		
		return cfpVector;
	}
	
	/**
	 * This method is called every time the behaviour/agent receives a propose message. Modified by Kari Roessland.
	 * @param propose The incomming proposal message.
	 * @param acceptances A vector cotaining all acceptances messages.
	 */
	protected void handlePropose(ACLMessage propose, java.util.Vector acceptances){
		fw.write("    "+myAgent.getLocalName()+" received a proposal from "+propose.getSender().getLocalName()+":");
		fw.write("      Proposal: "+propose.getContent());
	}
	
	/**
	 * This method is called every time the behaviour/agent receives a not-understood message. Added by Kari Roessland. 
	 * @param notUnderstood The incomming not-understood message.
	 * @param acceptances A vector cotaining all acceptances messages.
	 */	
	protected void handleNotUnderstood(ACLMessage notUnderstood, Vector acceptances){
		fw.write("    "+myAgent.getLocalName()+" received a not-understood message from "+notUnderstood.getSender().getLocalName()+":");
	}
		
	/**
	 * This method is called every time the behaviour/agent receives a propose message. Added by Kari Roessland. 
	 * @param refuse The incomming refuse message.
	 * @param acceptances A vector cotaining all acceptances messages.
	 */
	protected void handleRefuse(ACLMessage refuse, Vector acceptances){
		fw.write("    "+myAgent.getLocalName()+" received a refuse message from "+refuse.getSender().getLocalName()+":");
	}
	
	
	/**
	 * Method handling all proposals being received. Marks the action as activated and relates it to a node representing 
	 * the agents which has sent the proposal. Modified by Kari Roessland.
	 * @param responses All response - messages in a Vector.
	 * @param acceptances A vector containing all acceptances messages. 	 
	 */
	protected void handleAllResponses(Vector responses, Vector acceptances){

		if(responses.isEmpty()){
			//commented by kari
			//transition = NOT_ENOUGH_CAPABLE_AGENTS_IN_SYSTEM;
			fw.write("    "+myAgent.getLocalName()+" did not receive any proposals in response to the CFPs");
			transition = PrepareResultBehaviour.TEAM_FORMATION_FAILED;
		}else{
		
			ACLMessage response = null;
			ACLMessage reject = null;
			
			
			fw.write("\n    "+myAgent.getLocalName()+" has to handle all the proposals received from the problem solvers");
			fw.write("    Agents sending proposals are used to create Executors, which are mapped to the different");
			fw.write("    Actions in the TMST. Each Executor - Action -relation also has a Cost - value. All the information needed");
			fw.write("    to do this is found in the proposal. Executor - Action realations and their Cost - values are:\n");
			
			
			//Iterate through the vector containing all responses and add all agents as executors of the actions they may perform.
			//Set the ones with lower cost as the executors of the actions (through referring to the cost of the executor - cross reference).
			for(int i = 0; i < responses.size(); i++){
			
				response = (ACLMessage)responses.get(i);
			
				//If the message is not a proposal, we don't bother ...
				if(response.getPerformative() == ACLMessage.PROPOSE){
					//Map all proposing agents to the corresponding action in the tmst
					try{
						((CoPSTaskResponsible)myAgent).mapAgentToExecutor(response);
					}catch(Exception e){
						System.err.print(e.getMessage());
						e.printStackTrace();
					}
				}
			}
			
			
			fw.write("\n    "+myAgent.getLocalName()+" generates the solution space of the TMST, by activating the"); 
			fw.write("    nodes from the bottom and up. All actions connected to an executor is activated, all Tasks");
			fw.write("    connected to an activated action or PSM is activated, all PSMs having all of their sub-Tasks");
			fw.write("    activated are activated. If the initial-Task gets activated, the problem has at least one solution.\n");
			
			//commented for debugging reasons-this part make this method fail. the acceptances are not right, and not returned.
			try{
				//Genereate a solution space
				if(!((CoPSTaskResponsible)myAgent).getTMST().generateSolutionSpace()){

					//transition = NOT_ENOUGH_CAPABLE_AGENTS_IN_SYSTEM;
					transition = PrepareResultBehaviour.TEAM_FORMATION_FAILED;
				}else{
					fw.write("    "+myAgent.getLocalName()+" tries to find the cheepest solution according to action-execution costs");
					fw.write("    One starts at the bottom level (actions) and propagates the costs upwards in the TMST. For all of the");
					fw.write("    Tasks one always choose the chepest alternative of solving it (choose one of the sub-PSMs or sub-Actions).");
					fw.write("    All chosen nodes are marked as part of the chosen solution.");
					//Generate a solution from the possible solutions in the solution space
					if(!((CoPSTaskResponsible)myAgent).getTMST().generateSolution()){
						//commented by kari
						fw.write("    "+myAgent.getLocalName()+" did not find any solution afterall");
						//transition = NO_SOLUTIONS_FOUND;
						transition = PrepareResultBehaviour.TEAM_FORMATION_FAILED;
					}else{
						//Only possible to form team if there exist a solution to the problem
						//This is the main task for this method - to make the invitations being sent
						fw.write("    "+myAgent.getLocalName()+" did find the cheepest solution");
						
						Vector invitations = new Vector();
						invitations = ((CoPSTaskResponsible)myAgent).makeInvitations(responses);
						acceptances.addAll(invitations);
						
					}
				}
				
				//If there does not exist a solution to the problem, send a refuse-proposal messages
				//to the agents that did send a proposal.
				if(acceptances.size() == 0){
					fw.write("    Since there were no solution, "+myAgent.getLocalName()+" sends reject-proposal messages to");
					fw.write("    the agents that did send a proposal");
					for(int i = 0; i < responses.size(); i++){
						response = null;
						reject = null;
						response = (ACLMessage)responses.get(i);
			
						//If the message is not a proposal, we don't bother ...
						if(response.getPerformative() == ACLMessage.PROPOSE){
							reject = response.createReply();
							reject.setProtocol(FIPANames.InteractionProtocol.FIPA_CONTRACT_NET);
							reject.setPerformative(ACLMessage.REJECT_PROPOSAL);
							acceptances.add(reject);
						}
					}				
				
				}
				
			}catch(NameAlreadyExistException e){
				e.printStackTrace();
			}
		}//end responses is not empty
	}
	
	/**
	 * This method is called each time an inform message is received. Added by Kari Roessland.
	 * @param inform An inform message
	 */
	 protected void handleInform(ACLMessage inform){
		fw.write("    "+myAgent.getLocalName()+" got a message that "+inform.getSender().getLocalName()+" accepted the invitation to join the team");
	 }
	
	/**
	 * This method is called each time a failure message is received. Added by Kari Roessland.
	 * @param failure A failure message.
	 */
	 protected void handleFailure(ACLMessage failure){
		fw.write("    "+myAgent.getLocalName()+" got a messaage that "+failure.getSender().getLocalName()+" could not join the team because of a failure");
	 }
	
	/**
	 * This method is called when all of the inform and failure messages are received, it checks if all
	 * of the agents that received an invitation has accepted to join the team. If they have not, re-formation of the team is necessary.
	 * Added by Kari Roessland.
	 * @param resultNotifications A vector containing all of the result-notifications.
	 */
	protected void handleAllResultNotifications(Vector resultNotifications){
		int teamMembers = resultNotifications.size();
		if (teamMembers > 0){
			//to reach this far is the only way to achieve success in this behaviour
			transition = PrepareResultBehaviour.SUCCESS;
			fw.write("    "+myAgent.getLocalName()+" successfully formed a team\n");
		}else{
			transition = PrepareResultBehaviour.TEAM_FORMATION_FAILED;
		}
	}
	
	public int onEnd(){
		reset();			 
		return transition;
	}
}
