package CoPS.agents;
//
//  GetDecompositionBehaviour.java
//  CoPS
//
//  Created by Kari on 01.02.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//
//  Changed by Kari 02.02.06 - 
//  Changed by Kari 13.05.06 - CoPS pretty print

//java classes
import java.util.Vector;

//jade classes
import jade.core.AID;
import jade.core.Agent;
import jade.proto.AchieveREInitiator;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;

//tmst classes
import TMST.representation.TMST;
import CoPS.FileWriter;

/**
 *	This class represents a behaviour specifying the process of getting a decomposition of a problem. The Task Responsible starts a
 *	conversation with the Decomposer agent. Modified by Kari Roessland.
 *	<ul>
 * 		<li>Starts with finding a decomposer by requesting the DF.</li>
 *		<li> Request decomposition of a problem</li>
 *		<li> Expects a acknowledgement message from the decomposer, if not it waits for a specified amount of ms, then sends another request</li>
 *		<li>After an acknowledgement message recieved, it waits for another specified time for a decomposition, if none is received a reminder is sent. </li>
 *	</ul>
 *
 */
public class GetDecompositionBehaviour extends AchieveREInitiator{
	//Modified by Kari Roessland.
	private int transition = PrepareResultBehaviour.DECOMPOSITION_FAILED;
	//Added by Kari Roessland.
	private FileWriter fw = null;
	
	/**
	 * The constructor only calls the super-class AchieveREInitiatior
	 * @param agent The owner of this behaviour
	 * @param message The default request message.
	 */
	public GetDecompositionBehaviour(Agent agent, ACLMessage message){
		super(agent, message);
	}
	
	/**
	 * Method which has to be implemented to set the request message to be sent dynamic in run-time.
	 * Overrides AchieveREInitiators default implementation. Modified by Kari Roessland.
	 * @param request The default request message.
	 * @return A vector containing all of the request messages to be sent.
	 */
	protected Vector prepareRequests(ACLMessage request){
		fw = ((CoPSTaskResponsible)myAgent).getFileWriter();

		fw.write("\n\n################################################################");
		fw.write("############        2. Team Formation             ##############");
		fw.write("################################################################\n\n");		
		
		//Vector contaning the message which is being sent
		Vector requestVector = new Vector();

		//Create the message to send to the decomposer - this contains the task (problem) the TR has taken upon itself to solve as content
		String task = ((CoPSTaskResponsible)myAgent).getTask();
		
		fw.write("    "+myAgent.getLocalName()+ " needs a task decomposition (TMST)");
		
		request.addReceiver(((CoPSTaskResponsible)myAgent).getDecomposer());
		request.setContent(task);
		request.setLanguage("PlainText");
		requestVector.add(request);
		return requestVector;
	}
	
	/**
	 * This method is called when the agent running this behaviour accepts an agree message. Modified by Kari Roessland.
	 * @param agree An agree message.
	 */
	protected void handleAgree(ACLMessage agree){
		fw.write("    "+myAgent.getLocalName()+ " got a message saying that "+agree.getSender().getLocalName()+" agrees to make a TMST");
	}

	
	/**
	 * This method is called when the agent running this behaviour gets a not-understood message. New by Kari Roessland.
	 * @param notUnderstood A not-understood message.
	 */
	protected void handleNotUnderstood(ACLMessage notUnderstood){
		fw.write("    "+myAgent.getLocalName()+ " got a message saying that "+notUnderstood.getSender().getLocalName()+" did not understand the task to decompose");
		transition = PrepareResultBehaviour.DECOMPOSITION_FAILED;
	}

	/**
	 * Called if one of the contacted decomposers refuses decomposing the requested task. Modified by Kari Roessland.
	 * @refuse A refuse message.
	 */
	protected void handleRefuse(ACLMessage refuse){
		fw.write("    "+myAgent.getLocalName()+ " got a message saying that "+refuse.getSender().getLocalName()+" refused to make a TMST");
		transition = PrepareResultBehaviour.DECOMPOSITION_FAILED;
	}
	


	/**
	 * Called if the responder responds with an inform message containing the decomposition. Modified by Kari Roessland.
	 * @param inform An inform message.
	 */
	protected void handleInform(ACLMessage inform){
		fw.write("    "+myAgent.getLocalName()+" got a TMST from "+inform.getSender().getLocalName());
		try{
		
			TMST tmst = (TMST)inform.getContentObject();
			((CoPSTaskResponsible)myAgent).setTMST(tmst);
			
			if(((CoPSTaskResponsible)myAgent).getTMST()==null){ 
				//If the tmst is not set the agent shold not proceed, there return a failure event
				transition = PrepareResultBehaviour.DECOMPOSITION_FAILED;
			}else{
				transition = PrepareResultBehaviour.SUCCESS;
			}
		}catch(UnreadableException e){
			//The decomposition failed because the message was not readable
			e.printStackTrace();
			transition = PrepareResultBehaviour.DECOMPOSITION_FAILED;
		}
	}
	
	
	
	/**
	 * Called if a failure message is received. Modified by Kari Roessland.
	 * @param failure A failure message
	 */
	protected void handleFailure(ACLMessage failure){
		fw.write("    "+myAgent.getLocalName()+ " got a message saying that "+failure.getSender().getLocalName()+" failed to make a TMST");
		transition = PrepareResultBehaviour.DECOMPOSITION_FAILED;
	}
	
	/**
	 * Method called after the behaviour is removed from the behaviour pool.
	 * @return The transition from this behaivour.
	 */
	public int onEnd(){
		reset();
		return transition;
	}	
}




