package CoPS.agents;
//
//  HandleProblemSolvingRequestsBehaviour.java
//  CoPS
//
//  Created by Kari on 31.01.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//
// Changed by Kari 13.06.06 - Pretty print CoPS

import jade.core.Agent;
import jade.core.behaviours.Behaviour;
import jade.proto.AchieveREResponder;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

import CoPS.*;

/**
 *
 *	This class represents the behaviour describing what to do when getting an ACL message
 *  with a problem solving request. Modified by Kari Roessland.
 *  
 *  The prepareResultNotification state of the AchieveREResponder protocol is taken care of
 *  by a new behaviour: PrepareResultBehaviour. By using the method registerPrepareResultNotification
 *  the datastore of this behaviour is set in our PrepareResultBehaviour. It is responsibility of this 
 *  behaviour to put into the datastore (bound at the right key) the proper ACLMessage object to be sent by
 *  this responder. The proper key is RESULT_NOTIFICATION_KEY.
 *
 */
public class HandleProblemSolvingRequestsBehaviour extends AchieveREResponder{
	private MessageTemplate messageTemplate = null;
	
	//added by Kari Roessland
	private FileWriter fw = null;
	
	/**
	 * The constructor
	 * @param agent the owner of this behaviour
	 * @messageTemplate the template specifying the correct request-message.
	 */
	public HandleProblemSolvingRequestsBehaviour(Agent agent, MessageTemplate messageTemplate){
		super(agent, messageTemplate);
		this.messageTemplate = messageTemplate;
		
		this.registerPrepareResultNotification(new PrepareResultBehaviour(agent));
	}
	
	/**
	 * In this method the agent informs the requester of the problem solving
	 * if it is willing to try solving the problem or not. If the agent will try to 
	 * come up with a solution it returns an agree message, else it returns a not-understood
	 * message. Modified by Kari Roessland.
	 * @param request The request message.
	 * @return The response message to the request message.
	 */
	protected ACLMessage prepareResponse(ACLMessage request){
	
		fw = ((CoPSTaskResponsible)myAgent).getFileWriter();
	
		fw.write("\n\n##################################################################################");
		fw.write("####################   Start CoPS Problem Solving Process:   #####################");
		fw.write("####################   1. Problem Analyzis                   #####################");
		fw.write("####################   2. Team Formation                     #####################");
		fw.write("####################   3. Solving of Subtasks	             #####################");
		fw.write("####################   4. Integration of partial results     #####################");
		fw.write("##################################################################################\n\n");


		fw.write("################################################################");
		fw.write("############         1. Problem Analyzis          ##############");
		fw.write("################################################################\n\n");
		
		String task = request.getContent();
		fw.write("    "+myAgent.getLocalName() +" received a task solving request from "+request.getSender().getLocalName()+" and reviews the task");
		System.out.println("\n    "+myAgent.getLocalName()+" received a task solving request from "+request.getSender().getLocalName()+", output from the different\n"+
			"    steps of the problem solvin process are written to "+ fw.getFileName()+" stored where this program is run from.\n    "+
			myAgent.getLocalName()+" received the task: "+task);
		ACLMessage response = request.createReply();
		
		if(((CoPSTaskResponsible)myAgent).reviewProblem(task)){
			fw.write("    "+myAgent.getLocalName()+" did understand the task\n      Task: "+task);
			fw.write("    "+myAgent.getLocalName()+" is not capable of solving the task alone");
			//Store the request and the task in the CoPSTaskResponsible agent for simplicity, even though it is redundant to store both
			((CoPSTaskResponsible)myAgent).setTask(task);
			((CoPSTaskResponsible)myAgent).setRequest(request);
			
			//The requested action (solving a problem) is accepted. Send an agree message to the requesting agent
			response.setPerformative(ACLMessage.AGREE);
						
		}else{
			fw.write("    "+myAgent.getLocalName()+" did not understand the task\n      Task: "+task);
			response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
		}
		
		return response;
	}
	
	/**
	 * Information from the last execution of this behaviour is
	 * reset and the behaviour is added to the agent again. Modified by Kari Roessland.
	 * @return The transition from this behaviour. 
	 */
	public int onEnd(){
		reset();
		//this makes it possible for our agent to receive a new problem solving request. all of the 
		//other behaviours connected to this one need to be reset as they finish, but they should not
		//be added to the agent's behaviour-pool.
		myAgent.addBehaviour((Behaviour)this);
		return 0;
	}
}
