package CoPS.agents;
//
//  PrepareResultBehaviour.java
//  CoPS
//
//  Created by Kari on 01.02.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//
//

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.FSMBehaviour;
import jade.domain.FIPANames;
import jade.lang.acl.ACLMessage;

/**
 * This class represents a finite state machine behaviour. Modified by Kari Roessland.
 * preparing the results of the requested task. It represents the PREPARE_RESULT_NOTIFICATION state 
 * of HandleProblemSolvingRequestBehaviour that extends AchieveREResponder. This behaviour shares the
 * datastore with the HandleProblemSovlingRequestBehaviour.
 */
public class PrepareResultBehaviour extends FSMBehaviour{
	//states
	private static final String GD = "GetDecomposition";
	private static final String FT = "FormTeam";
	private static final String APSP = "AdministerProblemSolvingProcess";
	private static final String TRR = "TaskResponsibleResponse";
	private static final String FSB = "FailureSpecificationBehaviour";
	
	//events - are public since they are used by the state-behaviours of PrepareResultBehaviour
	public static final int SUCCESS = 0;
	public static final int DECOMPOSITION_FAILED = 1;
	public static final int TEAM_FORMATION_FAILED = 2;
	public static final int PROBLEM_SOLVING_FAILED = 3;
	public static final int RESPONSE_FAILED = 4;
	
	/**
	 * The constructor only calls the super-class(FSMBehaviour) constructor.
	 * @param agent The owner of this behaviour.
	 */
	public PrepareResultBehaviour(Agent agent){
		super(agent);
	}
	
	/**
	 * This method is called when the behaviour is started. Modified by Kari Roessland.
	 */
	public void onStart(){
	
		//Instantiate the message the GetDecompositionBehaviour is goiing to send, have to set performative and protocol
		ACLMessage decompositionRequest = new ACLMessage(ACLMessage.REQUEST);
		decompositionRequest.setProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST);
		
		ACLMessage teamSpaceRequest = new ACLMessage(ACLMessage.REQUEST);
		teamSpaceRequest.setProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST);
		
		//Instantiate the call for proposals message Form Team Behaviour is going to send to all agents performing interesting actions
		ACLMessage cfp = new ACLMessage(ACLMessage.CFP);
		cfp.setProtocol(FIPANames.InteractionProtocol.FIPA_CONTRACT_NET);
		cfp.setLanguage(FIPANames.ContentLanguage.FIPA_SL);
		cfp.setOntology(((CoPSTaskResponsible)myAgent).getOntology().getName());
		
		//Register the different state behaviours
		registerFirstState(new GetDecompositionBehaviour(myAgent, decompositionRequest), GD);
		registerState(new FormTeamBehaviour(myAgent, cfp), FT);
		registerState(new AdministerProblemSolvingProcessBehaviour(myAgent, teamSpaceRequest), APSP);
		registerLastState(new TaskResponsibleResponseBehaviour(myAgent, this.getDataStore(), this.getParent()), TRR);
		registerLastState(new FailureSpecificationBehaviour(myAgent, this.getDataStore(), this.getParent()), FSB);

		//Register the default transitions. 
		//These transitions are fired when the first state (ex:GD)
		//returns a number not registered as one of the events. The states should return 0, to fire
		//the default transitions.
		registerDefaultTransition(GD, FT);
		registerDefaultTransition(FT, APSP);
		registerDefaultTransition(APSP, TRR);
		
		//Register the alternative transitions.
		//Thesee are fired when on of the events are returned from the states (represented by behaviours).
		//If none of these transitions occur, the generation of a solution succeded, and the solution
		//is returned from the TRR state.
		registerTransition(GD, FSB, DECOMPOSITION_FAILED);
		registerTransition(FT, FSB, TEAM_FORMATION_FAILED);
		registerTransition(APSP, FSB, PROBLEM_SOLVING_FAILED);
		registerTransition(TRR, FSB, RESPONSE_FAILED);
	}
	
	/**
	 * This method is called when the behaviour has finished. Modified by Kari Roessland.
	 * @return The transition from this behaviour
	 */
	public int onEnd(){
		reset();
		return 0;
	}
}




