package CoPS.agents;
//
//  ProblemSolvingBehaviour.java
//  CoPS
//
//  Created by Kari on 17.03.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//


//jade
import jade.core.Agent;
import jade.proto.AchieveREResponder;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.core.behaviours.Behaviour;


/**
 * This class and behaviour assist a problem solving agent in the problem
 * solving process by supporting a conversation with the TeamSpaceAgent.
 * @author Kari Roessland
 */
public class ProblemSolvingBehaviour extends AchieveREResponder{
	
	/**
	 * The constructor only calls the super-constructor.
	 * @param agent The owner of this behaviour
	 * @param template The template used as a filter to get the correct request message.
	 */
	public ProblemSolvingBehaviour(Agent agent, MessageTemplate template){
		super(agent, template);
	}
	
	/**
	 * The agent desides if it can perform the action and prepares a response telling this.
	 * @param request The incoming request message (request for performing an action).
	 * @return The response stating if the agent will perform the action or not.
	 */
	protected ACLMessage prepareResponse(ACLMessage request){
		
		
		String action = request.getContent();
		ACLMessage response = request.createReply();
		 
		
		//check if the agent understands the content, temporarily check if it is not null
		if(action != null && action.compareTo("") != 0){
			if (!((CoPSProblemSolver)myAgent).getPartOfTeam()){
				response.setPerformative(ACLMessage.REFUSE);
			}else{
				response.setPerformative(ACLMessage.AGREE);
			}
			
		}else{
			response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
		}
		
		return response;
	}
	
	/**
	 * The agent prepares the output from the action and returns it to the requester (TeamSpaceAgent).
	 * @param request The request message.
	 * @param response The agent's response to the request message.
	 * @return A message with the solution/output of performing the action.
	 */
	protected ACLMessage prepareResultNotification(ACLMessage request, ACLMessage response){	
		
		//Here the performAction(request) should be called. The response message is returned from this method.
	
		response = ((CoPSProblemSolver)myAgent).performAction(request);
		
	
		return response;
	}
	
	/**
	 * Information from the last execution of this behaviour is reset.
	 * @return The final transition from this behaviour/state.
	 */
	public int onEnd(){
		reset();
		return 0;
	}
}
