package CoPS.agents;
//
//  RespondToCfps.java
//  CoPS
//
//  Created by Kari on 31.01.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

//Jade classes
import jade.proto.ContractNetResponder;
import jade.core.Agent;
import jade.core.behaviours.SequentialBehaviour;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.ACLMessage;
import jade.content.lang.Codec;
import jade.content.onto.Ontology;
import jade.domain.FIPANames;
import jade.domain.FIPAAgentManagement.NotUnderstoodException;
import jade.domain.FIPAAgentManagement.RefuseException;
import jade.core.behaviours.DataStore;

/**
 * Class taking care of the correspondence between an agent sending out cfps and this problem solver.
 * Modified by Kari Roessland.
 */
public class RespondToCfpsBehaviour extends ContractNetResponder{
	
	String problem = "";
	
	/**
	 * The constructor only calls the RespondToCfpsBehaviour class' superconstructor. Modfied by Kari Roessland.
	 * @param agent The owner of this behaviour.
	 * @param template A message template used to filter the correct cfp-messages.
	 * @param ds The datastore belonging to this behaviour.
	 */
	public RespondToCfpsBehaviour(Agent agent, MessageTemplate template, DataStore ds){
		super(agent, template, ds);
	}
	
	/**
	 * The method prepareResponse handles the making of an answer to a cfp-message. 
	 * The agent makes a proposal to the incoming cfp-message. Modfied by Kari Roessland.
	 * @param request The incoming cfp message.
	 * @return A message which is the response to the incoming cfp message.
	 */
	protected ACLMessage prepareResponse(ACLMessage request){
		
		ACLMessage response = request.createReply();
		request.setProtocol(FIPANames.InteractionProtocol.FIPA_CONTRACT_NET);
		
		if(!((CoPSProblemSolver)myAgent).knownOntology(request)){
			response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
		}else if(!((CoPSProblemSolver)myAgent).knownLanguage(request)){
			response.setPerformative(ACLMessage.NOT_UNDERSTOOD);

		}else{
			//Make a proposal
			if(((CoPSProblemSolver)myAgent).deceideWhetherToMakeProposal(request)){

				//Store the current request
				((CoPSProblemSolver)myAgent).setCurrentRequest(request);
				
				//Make the proposal
				String content = ((CoPSProblemSolver)myAgent).makeProposal(request);
				
				if (content == null){
					response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
				}else{
					response.setPerformative(ACLMessage.PROPOSE);
					response.setContent(content);
					problem = request.getContent();
				}
			//Do not make a proposal
			}else{
				response.setPerformative(ACLMessage.REFUSE);
			}
		}
		return response;
	}
	
	/**
	 * The method handleRejectProposal is called when an reject-proposal-message is received, the agent
	 * did not get an invitation to join the problem solving team. Added by Kari Roessland.
	 * @param cfp The cfp message.
	 * @param propose The agent's proposal message.
	 * @param rejectProposal The received reject-proposal message.
	 */
	 protected void handleRejectProposal(ACLMessage cfp, ACLMessage propose, ACLMessage rejectProposal){
	 }
	
	/**
	 * The method prepareResultNotification is called when an accept-message is received. The result states if an
	 * agent wants to join the team or not. Modfied by Kari Roessland.
	 * @param cfp The cfp message.
	 * @param propose The agent's proposal message.
	 * @param accept The invitation/rejection message received from the Task Responsible agent.
	 * @return A message saying if the agent wants to join the team or not (only if an invitation message was received).
	 */
	protected ACLMessage prepareResultNotification(ACLMessage cfp, ACLMessage propose, ACLMessage accept){
		//the performAction-method only returns the accept-message for now, has to be expanded
		//ACLMessage result = ((CoPSProblemSolver)myAgent).performAction(accept);
		
		//for now - returns an empty result-message, the return-message should be an INFORM message or a FAILURE message
		ACLMessage result = accept.createReply();
		result.setProtocol(FIPANames.InteractionProtocol.FIPA_CONTRACT_NET);
		result.setPerformative(ACLMessage.INFORM);
		result.setLanguage("plain-text");
		result.setContent("I will join the team");
		
		//registers that this agent is part of a team
		((CoPSProblemSolver)myAgent).setPartOfTeam(true);
		
		//Add another behaviour to this agent's behaviour pool. The added behaviour will assist in the 
		//problem solving process.
		
		//Initialise Message Template which is used for filtering which responses are being answered and which are not
		MessageTemplate template = MessageTemplate.and(
														   MessageTemplate.MatchProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST),
														   MessageTemplate.MatchPerformative(ACLMessage.REQUEST)
														   );
		myAgent.addBehaviour(new ProblemSolvingBehaviour(myAgent, template));
		
		
		return result;
	}
	
	/**
	 * This method returns the transition from this state/behaviour.
	 * @return The final transition
	 */
	public int onEnd(){
		this.reset();
		return 0;
	}
}