package CoPS.agents;
//
//  RespondToDecomposeRequestsBehaviour.java
//  CoPS
//
//  Created by Kari on 31.01.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

//Java classes
import java.io.IOException;

//JADE classes
import jade.core.Agent;
import jade.core.behaviours.SequentialBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.UnreadableException;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.domain.FIPAAgentManagement.RefuseException;
import jade.domain.FIPAAgentManagement.FailureException;
import jade.domain.FIPANames;
import jade.proto.AchieveREResponder;

//jCreek classes
import jcreek.representation.LocalKnowledgeModel;

//TMST classes
import TMST.representation.TMST;

//CoPS classes
import CoPS.Exceptions.NotAbleToDecomposeTaskException;
import CoPS.agents.*;

import CoPS.FileWriter;

/** 
 * Behaviour specifying what to do while waiting for a new task to decompose. 
 * Only messages with a request performative is processed. Null messages are ignored, while 
 * all other messages with performative not set to request are answered with a not-understood reply. 
 * Modified by Kari Roessland.
 */
public class RespondToDecomposeRequestsBehaviour extends AchieveREResponder{
	private FileWriter fw = null;
	
	/**
	 * The constructor.
	 * @param agent The owner of this behaviour.
	 * @param template A message template used to filter the incoming  request-messages.
	 */
	public RespondToDecomposeRequestsBehaviour(Agent agent, MessageTemplate template){
        super(agent, template);
    }
	
	/**
	 * Method used to prepare the response, saying if the decomposer wants/can decompose the received
	 * task or not. Modified by Kari Roessland.
	 * @param request The incoming task-decomposition-request message.
	 * @return A message which is an agree-message, refuse-message or not-understood-message.
	 */
	protected ACLMessage prepareResponse(ACLMessage request){
		
		fw = new FileWriter(request.getSender().getLocalName()+".txt");
		
		fw.write("    "+myAgent.getLocalName() +" received a task decomposition request from "+request.getSender().getLocalName());
		
		String task = request.getContent();
		
		ACLMessage response = request.createReply();
		
		if(((CoPSDecomposer)myAgent).correctFormat(task)){
			//The requested action (solving a problem) is accepted
			response.setPerformative(ACLMessage.AGREE);
			fw.write("    "+myAgent.getLocalName()+" did understand the task\n      Task: "+task);
		}else{
			//The requested action (solving a problem) is not understood
			response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
			fw.write("    "+myAgent.getLocalName()+" did not understand the task\n      Task: "+task);
		}
		return response;
	}
	
	
	/**
	 * The method make and return a message containing a TMST. Modified by Kari Roessland.
	 * @param request The received request message.
	 * @param response The response message sent before.
	 * @return A message definin the result of the composition, could either be an inform-message or a failure-message.
	 */
	protected ACLMessage prepareResultNotification(ACLMessage request, ACLMessage response){
	
		try{
			TMST tmst = ((CoPSDecomposer)myAgent).decomposeProblem(request.getContent());
			
			tmst.setTeamID(((CoPSDecomposer)myAgent).getTeamID());
			response.setPerformative(ACLMessage.INFORM);
			response.setLanguage("jcreek:object");			
			response.setContentObject(tmst);
			fw.write("    "+myAgent.getLocalName()+" made the TMST");			
		}catch(IOException e){
			e.getMessage();
			e.printStackTrace();
			fw.write("    "+myAgent.getLocalName()+" was not able to read the content of the task decomposition request");
			response.setPerformative(ACLMessage.FAILURE);		
		}catch(NotAbleToDecomposeTaskException e){
			e.getMessage();
			e.printStackTrace();
			fw.write("    "+myAgent.getLocalName()+" was not able to make the TMST");
			response.setPerformative(ACLMessage.FAILURE);
		}
		return response;
	}
 }
