package CoPS.agents;
//
//  TaskResponsibleResponseBehaviour.java
//  CoPS
//
//  Created by Kari on 01.02.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

//java classes
import java.util.Iterator;

//jade classes
import jade.core.Agent;
import jade.core.behaviours.FSMBehaviour;
import jade.core.behaviours.SimpleBehaviour;
import jade.lang.acl.ACLMessage;
import jade.core.AID;
import jade.domain.FIPANames;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.DataStore;

import CoPS.FileWriter;

/**
 * This class represents a behaviour returning a state describing 
 * how the problem solvig process went.
 *
 * Modified by Kari Roessland: The class is separated from the CoPSTaskResponsibleClass. And it is modified so that
 * the return message - containing the solution to the problem is placed in the right datastore with
 * the right KEY. In the previous working this message was never returned.
 */
public class TaskResponsibleResponseBehaviour extends SimpleBehaviour{ 

	private ACLMessage solution = null;
	private boolean finished = false;
	private int transition = PrepareResultBehaviour.SUCCESS;
	private Behaviour dataStoreBehaviour;
	private FileWriter fw = null;
	
	/** 
	 * The constructor. Modified by Kari Roessland.
	 * @param agent The owner of the behaviour.
	 * @param ds The datastore to be set in this behaviour.
	 * @param topBehaviour The Behviour where this one was added to the behaviour-pool.
	 */
	public TaskResponsibleResponseBehaviour(Agent agent, DataStore ds, Behaviour topBehaviour){
		super(agent);
		setDataStore(ds);
		dataStoreBehaviour = topBehaviour;
	}
	
	/**
	 * The action method prepares and put the result/solution in the proper datastore; the datastore of
	 * the HandleProblemSolvingRequestBehaviour. Modified by Kari Roessland.
	 */
	public void action(){
		fw = ((CoPSTaskResponsible)myAgent).getFileWriter();
		solution = ((ACLMessage) getDataStore().get(((HandleProblemSolvingRequestsBehaviour)dataStoreBehaviour).REQUEST_KEY)).createReply();
		
		solution.setPerformative(ACLMessage.INFORM);
		solution.setContent(((CoPSTaskResponsible)myAgent).getSolution());
		solution.setLanguage(((CoPSTaskResponsible)myAgent).getSolutionLanguage());
		fw.write("    "+myAgent.getName()+" returns the solution for the requester of the problem, "+((AID)solution.getAllReceiver().next()).getLocalName());
		fw.write("      Content of the solution-message: "+solution.getContent());
		System.out.println("    "+myAgent.getName()+" returns the solution for the requester of the problem, "+((AID)solution.getAllReceiver().next()).getLocalName());
		System.out.println("      Content of the solution-message: "+solution.getContent());
		
		getDataStore().put(((HandleProblemSolvingRequestsBehaviour)dataStoreBehaviour).RESULT_NOTIFICATION_KEY,solution);

		finished = true;
	}
		
	/**
	 * The agent scheduler calls this method to see whether a Behaviour still need to be run or it has completed its task. 
	 * Concrete behaviours must implement this method to return their completion state. Finished behaviours are removed from 
	 * the scheduling queue, while others are kept within to be run again when their turn comes again.
	 * @return True or false, depending on if the behaviour is done or not.
	 */
	public boolean done(){
		return finished;
	}
	
	/**
	 * This method is called when the behaviour has finished.
	 * @return The transition from this behaviour.
	 */
	public int onEnd(){
		reset();
		return transition;
	}

}



