//
//  CheckForActionsBehaviour.java
//  Blackboard
//
//  Created by Kari on 16.03.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

package teamSpace.agent;

//jade
import jade.core.*;
import jade.core.behaviours.OneShotBehaviour;

//java
import java.util.Vector;
import java.util.ArrayList;

import CoPS.FileWriter;

/**
 * This behaviour is a behaviour (state) in the {@link FSMBehaviour} (final-state-machine) described by
 * the class {@link TeamSpacePSBehaviour}. In this state one finds out if there are any actions or PSMs 
 * that are ready to execute. If there are one or more actions and/or one or more PSMs the state transits to
 * the behaviour (state) {@link HandleActionsBehaviour}, else the problem solving process is over and the state
 * transits to the behaviour (state) {@link EndProblemSolvingBehaviour}.
 *
 * @author Kari Roessland
 */
public class CheckForActionsBehaviour extends OneShotBehaviour{

	
	private int transition = TeamSpacePSBehaviour.SUCCESS;
	private String teamID = "";
	private FileWriter fw = null;
	
	
	/**
	 * The constructor calls the super-constructor
	 * @param a The agent which is the owner of this behaviour
	 */
	public CheckForActionsBehaviour(Agent a, String teamID){
		super(a);
		this.teamID = teamID;
	}
	
	/**
	 * The method checks if there are any actions or PSMs that are ready to be executed or composed.
	 * The found actions and PSMs are set in the owner agent (of this behaviour) as processing actions and
	 * processing PSMs
	 */
	public void action(){
	
		fw = ((TeamSpaceAgent)myAgent).getFileWriter(teamID);
		
		ArrayList actions = ((TeamSpaceAgent)myAgent).getActionsReadyToExecute(teamID);
		ArrayList psms = ((TeamSpaceAgent)myAgent).getPSMsReadyToCompose(teamID);
		
		//Pretty-print the current state of the problem solving in the team space
		((TeamSpaceAgent)myAgent).printCurrentState(teamID);
		
		if (actions.size() == 0 && psms.size() == 0){
			transition = TeamSpacePSBehaviour.NO_MORE_ACTIONS;
			
		}else{
			((TeamSpaceAgent)myAgent).setProcessingActions(actions, teamID);
			((TeamSpaceAgent)myAgent).setProcessingPSMs(psms, teamID);
		}
	}

	/**
	 * The method resets this behaviour and returns the transition (for the FSM)
	 * @return The transition/result from this behaviour (state)
	 */	
	public int onEnd(){
		reset();
		return transition;
	}
}
