//
//  EndProblemSolvingBehaviour.java
//  Blackboard
//
//  Created by Kari on 16.03.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

package teamSpace.agent;

//java
import java.util.ArrayList;

//jade
import jade.core.*;
import jade.core.behaviours.OneShotBehaviour;

import CoPS.FileWriter;

/**
 * This behaviour is a behaviour (state) in the {@link FSMBehaviour} (final-state-machine) described by
 * the class {@link TeamSpacePSBehaviour}. When we are in this state, there are no more actions or PSMs
 * that are ready to be executed or composed. If there are a solution to the initial problem now, we transit
 * to the state {@link PrepareResultBehaviour}. If there are no solution to the initial problem, something
 * went wrong and we transit to the state {@link FailureBehaviour}.
 *
 * @author Kari Roessland
 */
public class EndProblemSolvingBehaviour extends OneShotBehaviour{

	private int transition = TeamSpacePSBehaviour.SUCCESS;
	private String teamID = "";
	private FileWriter fw = null;
	
	/**
	 * The constructor only calls the super-constructor.
	 * @param a The agent which gets this behaviour
	 */
	public EndProblemSolvingBehaviour(Agent a, String teamID){
		super(a);
		this.teamID = teamID;
	}
	
	/**
	 * The action method find out if the problem solving process is successful (sets the transition
	 * to <code>SUCCESS</code>) or if it is not complete or something are wrong (sets the transition
	 * <code>END_PROBLEM_SOLVING_FAILED</code>). The problem solving process is successful if there is a solution 
	 * to the problem.
	 */
	public void action(){
		fw = ((TeamSpaceAgent)myAgent).getFileWriter(teamID);
		String problemSolution = ((TeamSpaceAgent)myAgent).getProblemSolution(teamID);
		
		if(problemSolution.compareTo("") == 0){
			//there are no solution to the problem
			transition = TeamSpacePSBehaviour.END_PROBLEM_SOLVING_FAILED;
		}else{
			//there are a solution to the problem
			transition = TeamSpacePSBehaviour.SUCCESS;
		}
	}
	
	/**
	 * The method resets this behaviour and returns the transition (for the FSM)
	 * @return The transition/result from this behaviour (state)
	 */
	public int onEnd(){
		reset();
		return transition;
	}
}
