//
//  FailureBehaviour.java
//  Blackboard
//
//  Created by Kari on 16.03.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//
package teamSpace.agent;

//jade
import jade.core.behaviours.OneShotBehaviour;
import jade.core.Agent;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.DataStore;
import jade.lang.acl.ACLMessage;

import CoPS.FileWriter;


/**
 * This class handles any failure that occurs during the process of solving a problem in the team space.
 * The behaviour sees trough that the requester of the TeamSpace service is informed about the failure.
 * And a failure -message is sent as a resultnotification of the request.
 *
 * @author Kari Roessland
 */
public class FailureBehaviour extends OneShotBehaviour{
	
	private Behaviour dataStoreBehaviour;
	private String teamID = "";
	private FileWriter fw = null;

	/**
	 * The constructor only calls the super-constructor.
	 * @param ag The agent which get this behaviour
	 * @param dataStore The {@link DataStore} used in this behaviour
	 * @param topBehaviour A link to the {@link Behaviour} which defines the request protocol that this behaviour is a part of
	 */
	public FailureBehaviour(Agent ag, DataStore dataStore, Behaviour topBehaviour, String teamID){
		super(ag);
		dataStoreBehaviour = topBehaviour;
		setDataStore(dataStore);
		this.teamID = teamID;
	}
	
	/**
	 * The action method reports/specifies the failure, and returns a failure message to the
	 * TaskResponsible requesting for the TeamSpace. This message is part of the Request-protocol
	 * implemented by {@link TeamSpaceAgentBehaviour}.
	 */
	public void action(){
		fw = ((TeamSpaceAgent)myAgent).getFileWriter(teamID);
		ACLMessage response = ((ACLMessage) getDataStore().get(((TeamSpaceAgentBehaviour)dataStoreBehaviour).REQUEST_KEY)).createReply();
		response.setPerformative(ACLMessage.FAILURE);
		response.setLanguage("plain-text");
		getDataStore().put(((TeamSpaceAgentBehaviour)dataStoreBehaviour).RESULT_NOTIFICATION_KEY,response);
	
	}
	
	/**
	 * The method resets this behaviour and returns the transition from this state (behaviour).
	 * @return The transition value
	 */
	public int onEnd(){
		fw.write("\n\n");
		fw.write("################################################################");
		fw.write("####   End problem solving in the Team Space with failure   ####");
		fw.write("################################################################");
		fw.write("\n\n\n");
		this.reset();
		return 0;
	}
}


