//
//  GoalStack.java
//  Blackboard
//
//  Created by Kari on 01.03.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//
package teamSpace.structure;

import java.util.ArrayList;
import java.util.Iterator;
import jess.*;

import CoPS.FileWriter;
/**
 * This class contains the goal stack - meaning the goals that are readily achieved.
 * These goals are actually contained in the <code>Rete</code> rule engine. So this
 * class is only an interface to this rule engine.
 *
 * @author Kari Roessland
 */
public class GoalStack {
	private Rete ruleEngine;
	private FileWriter fw = null;
	
	public GoalStack(Rete ruleEngine){
		this.ruleEngine = ruleEngine;
	}

	/**
	 * This method sets the FileWriter that handles printing the output of the problem solving steps to a file.
	 * @param fileWriter Reference to the FileWriter used by all agents working on the problem of one certain TR
	 */	
	public void setFileWriter(FileWriter fileWriter){
		fw = fileWriter;
	}

	/**
	 * This method returns an ArrayList of achieved goals
	 * @return - An ArrayList of TSGoal
	 */
	public ArrayList getAchievedGoals(){
		ArrayList goals = new ArrayList();

		Iterator facts = ruleEngine.listFacts();
		Fact fact = null;
		Value value = null;
		String goalName = "";
		String output = "";
		
		while (facts.hasNext()){
			try{
				fact = (Fact)(facts.next());
									
				if(fact.getName().compareTo("MAIN::achieved") == 0){
					value = fact.getSlotValue("goal");
					goalName = value.toString();
					value = fact.getSlotValue("output");
					output = value.toString();
							
					goals.add(new TSGoal(goalName, output));
				}
			}catch(JessException e){
				e.printStackTrace();
			}			
		}
		
		return goals;
	}
	
	
	/**
	 * This method returns the output of a goal. The goal has to be achieved to have an output.
	 * @param goalName The name of the goal
	 * @return The output of the specified goal as a <code>String</code>, if the goal is not achieved an
	 * empty <code>String</code> is returned
	 */
	public String getGoalOutput(String goalName){
		ArrayList achievedGoals = getAchievedGoals();
		String output = "";
		TSGoal goal = null;
		for(int i = 0; i < achievedGoals.size(); i++){
			goal = (TSGoal)achievedGoals.get(i);
			if(goal.getName().compareTo(goalName) == 0){
				output = goal.getOutput();
			}
		}
		
		return output;
	}

}
