//
//  HandleActionsBehaviour.java
//  Blackboard
//
//  Created by Kari on 16.03.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//
package teamSpace.agent;

//jade
import jade.proto.AchieveREInitiator;
import jade.lang.acl.ACLMessage;
import jade.core.Agent;
import jade.core.AID;
import jade.domain.FIPANames;

//java
import java.util.Vector;
import java.util.ArrayList;

import teamSpace.structure.TSAction;
import teamSpace.structure.TSPSM;

import CoPS.FileWriter;
/**
 * This behaviour is a behaviour (state) in the {@link FSMBehaviour} (final-state-machine) described by
 * the class {@link TeamSpacePSBehaviour}. When we are in this state, there are one or more actions and/or 
 * one or more PSMs to execute or handle. The behaivour implements a request protocol. Requests are made
 * for executors of the different actions/PSMs. If everything goes OK the state transits to {@link CheckForActionsBehaviour},
 * else it transits to {@link FailureBehaviour}.
 *
 * @author Kari Roessland
 */
public class HandleActionsBehaviour extends AchieveREInitiator{
	
	private int transition = TeamSpacePSBehaviour.SUCCESS;
	private ArrayList actions;
	private ArrayList executedActions;
	private ArrayList psms;
	private ArrayList executedPSMs;
	private String teamID = "";
	private FileWriter fw = null;
	
	/**
	 * The constructor calls the super-constructor.
	 * @param ag The agent which is the owner of this behaviour
	 * @param request The request that are sent as part of the protocol implemented by this behaviour
	 */
	public HandleActionsBehaviour(Agent ag, ACLMessage request, String teamID){
		super(ag, request);
		this.teamID = teamID;
	}
	
	
	/**
	 * This method prepares the requests for executors of the different actions and PSMs
	 * that are ready to execute or compose. Information about action and PSMs are stored
	 * in {@link TSAction} and {@link TSPSM} -classes. This information is extracted and
	 * put into the request messages.
	 * 
	 * @param request The default request message (a templamte)
	 * @return A <code>Vector</code> with request messages
	 */
	protected Vector prepareRequests(ACLMessage request){
		fw = ((TeamSpaceAgent)myAgent).getFileWriter(teamID);
		actions = new ArrayList();
		psms = new ArrayList();
		executedActions = new ArrayList();
		executedPSMs = new ArrayList();
		Vector requests = new Vector();
		TSAction action = null;
		TSPSM psm = null;
		ArrayList partialResults = new ArrayList();
		
		ACLMessage req = null;
		ACLMessage requestCopy = new ACLMessage(ACLMessage.REQUEST);
		requestCopy.setProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST);
		
		
		actions = ((TeamSpaceAgent)myAgent).getProcessingActions(teamID);
		
		psms = ((TeamSpaceAgent)myAgent).getProcessingPSMs(teamID); 
		
		//HANDLING PSMS
		for(int i= 0; i<psms.size(); i++){
			psm = (TSPSM)psms.get(i);
			partialResults = psm.getPartialResults();
			
			req = (ACLMessage)requestCopy.clone();
			req.addReceiver(new AID(psm.getExecutor(), AID.ISGUID));
			req.setLanguage("plain-text");
			req.setContent(psm.getName()+"-"+psm.getInput()+"-"+partialResults.toString());
			requests.add(req);
		}
		
		//HANDLING ACTIONS
		for(int i = 0; i<actions.size(); i++){
			req = (ACLMessage)requestCopy.clone();
			action = (TSAction)actions.get(i);
		
			req.addReceiver(new AID(action.getExecutor(), AID.ISGUID));
			req.setLanguage("plain-text");
			req.setContent(action.getActionType() + "-" + action.getInput());
			requests.add(req);
		}
		return requests;
		
	}
	
	/**
	 * Method is called when a <code>NOT_UNDERSTOOD</code> message is received.
	 * @param notUnderstood The message
	 */
	protected void handleNotUnderstood(ACLMessage notUnderstood){
	}

	/**
	 * Method is called when an <code>AGREE</code> message is received.
	 * @param agree The message
	 */
	protected void handleAgree(ACLMessage agree){
	}
	
	/**
	 * Method is called when a <code>REFUSE</code> message is received.
	 * @param refuse The message
	 */
	protected void handleRefuse(ACLMessage refuse){
	}
		
	/**
	 * Method is called when a <code>FAILURE</code> message is received.
	 * @param failure The message
	 */
	protected void handleFailure(ACLMessage failure){
	}

	/**
	 * The method is called when an <code>INFORM</code> message is received.
	 * If the message is a solution to an action the corresponding {@link TSAction} object
	 * is updated and added to the list <code>executedActions</code>.
	 * If the message is a solution to an PSM the corresponding {@link TSPSM} object
	 * is updated and added to the list <code>executedPSMs</code>.
	 * @param inform The message
	 */	
	protected void handleInform(ACLMessage inform){

		String content = inform.getContent();
		int index = content.indexOf("-");
		String action_psm = content.substring(0, index);
		String output = content.substring(index+1, content.length());
		TSAction tsAction = null;
		TSPSM tspsm = null;
		
		//HANDLING ACTIONS
		for(int i = 0; i<actions.size(); i++){
			tsAction = (TSAction)actions.get(i);
			if(tsAction.getActionType().compareTo(action_psm) == 0){
				tsAction.setOutput(output);
				executedActions.add(tsAction);
			}
		}
		
		//HANDLING PSMS
		for(int i = 0; i<psms.size(); i++){
			tspsm = (TSPSM)psms.get(i);
			if(tspsm.getName().compareTo(action_psm) == 0){
				tspsm.setOutput(output);
				executedPSMs.add(tspsm);
			}
		}
	}
	
	/**
	 * The method is called when all result notifications are received. And
	 * processed actions are updated in the Team Space.
	 * @param resultNotifications A <code>Vector</code> containing all the result notifications
	 */
	protected void handleAllResultNotifications(Vector resultNotifications){
		transition = TeamSpacePSBehaviour.SUCCESS;
		TSAction action = null;
		TSPSM psm = null;
		
		if(((TeamSpaceAgent)myAgent).setProcessedActions(executedActions, teamID)){
			fw.write("############  Executed actions in current state  ###############");
			if(executedActions.size()==0){
				fw.write("    No actions were executed in current state...\n");
			}else{
				for(int i = 0; i < executedActions.size(); i++){
					action = (TSAction)executedActions.get(i);
					fw.write("    Action: "+action.getName()+"\n    Output: "+action.getOutput()+"\n"); 
				}
			}
			
		}else{
			transition = TeamSpacePSBehaviour.HANDLE_ACTIONS_FAILED;
		}
		
		if(((TeamSpaceAgent)myAgent).setProcessedPSMs(executedPSMs, teamID)){
			fw.write("##############  Executed PSMs in current state  ################");
			if(executedPSMs.size() == 0){
				fw.write("    No PSMs were composed in current state...\n");
			}else{
				for(int i = 0; i < executedPSMs.size(); i++){
					psm = (TSPSM)executedPSMs.get(i);
					fw.write("    PSM: "+psm.getName()+"\n    Output: "+psm.getOutput()+"\n");
				}
			}
		}else{
			transition = TeamSpacePSBehaviour.HANDLE_ACTIONS_FAILED;
		}
	}
	

	/**
	 * The method resets this behaviour and returns the transition (for the FSM)
	 * @return The transition/result from this behaviour (state)
	 */
	public int onEnd(){
		if(actions.size() == executedActions.size() && psms.size() == executedPSMs.size()){
			transition = TeamSpacePSBehaviour.SUCCESS;
		}else{
			//if not all of the actions and PMSs that are ready to be processed
			//is processed in this round, something is wrong. this failure should be 
			//handled. for now, we just state that a failure has occured.
			transition = TeamSpacePSBehaviour.HANDLE_ACTIONS_FAILED;
		}
		
		this.reset();
		return transition;
	}

}
