//
//  PrepareResultBehaviour.java
//  Blackboard
//
//  Created by Kari on 16.03.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//
package teamSpace.agent;

//java classes
import java.util.Iterator;

//jade classes
import jade.core.Agent;
import jade.core.behaviours.FSMBehaviour;
import jade.core.behaviours.SimpleBehaviour;
import jade.lang.acl.ACLMessage;
import jade.core.AID;
import jade.domain.FIPANames;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.DataStore;

import CoPS.FileWriter;

/**
 * This behaviour is the last behaviour (state) in the {@link FSMBehaviour} (final-state-machine) described by
 * the class {@link TeamSpacePSBehaviour}. If this state is reached the problem solving in the Team Space
 * was successfull. The solution of the initial problem is plased in the datastore used by the {@link TeamSpaceAgentBehaviour},
 * and is therefor the response in the request-protocol defined by the {@link TeamSpaceAgentBehaviour}.
 *
 * @author Kari Roessland
 */
public class PrepareResultBehaviour extends SimpleBehaviour{ 

	private ACLMessage solution = null;
	private boolean finished = false;
	private int transition = TeamSpacePSBehaviour.SUCCESS;
	private Behaviour dataStoreBehaviour;
	private String teamID = "";
	private FileWriter fw = null;
	
	
	/** 
	 * The constructor
	 * @param agent The agent which get this behaviour
	 * @param ds The {@link DataStore} used in this behaviour
	 * @param topBehaviour A link to the {@link Behaviour} which defines the request protocol that this behaviour is a part of
	 */
	public PrepareResultBehaviour(Agent agent, DataStore ds, Behaviour topBehaviour, String teamID){
		super(agent);
		setDataStore(ds);
		dataStoreBehaviour = topBehaviour;
		this.teamID = teamID;
	}
	
	/**
	 * The action method prepares and put the result/solution in the proper datastore; the datastore of
	 * the HandleProblemSolvingRequestBehaviour.
	 */
	public void action(){
		fw = ((TeamSpaceAgent)myAgent).getFileWriter(teamID);
		
		solution = ((ACLMessage) getDataStore().get(((TeamSpaceAgentBehaviour)dataStoreBehaviour).REQUEST_KEY)).createReply();
		
		solution.setPerformative(ACLMessage.INFORM);
		solution.setContent(((TeamSpaceAgent)myAgent).getProblemSolution(teamID));
		solution.setLanguage("plain-text");

		
		getDataStore().put(((TeamSpaceAgentBehaviour)dataStoreBehaviour).RESULT_NOTIFICATION_KEY,solution);

		finished = true;
	}
		
	/**
	 * The agent scheduler calls this method to see whether a Behaviour still need to be run or it has completed its task. 
	 * Concrete behaviours must implement this method to return their completion state. Finished behaviours are removed from 
	 * the scheduling queue, while others are kept within to be run again when their turn comes again.
	 * @return <code>true</code> if the behaviour is done and <code>false</code> if not
	 */
	public boolean done(){
		return finished;
	}
	
	/**
	 * This method is called when the behaviour has finished.
	 * @return The transition from this behaviour (state)
	 */
	public int onEnd(){
		fw.write("\n\n################################################################");
		fw.write("    Initial task/problem: "+((TeamSpaceAgent)myAgent).getInitialProblem(teamID));
		fw.write("    Input: "+((TeamSpaceAgent)myAgent).getInitialInput(teamID));
		fw.write("    Solution/Output: "+solution.getContent());
		fw.write("################################################################");
	
		fw.write("\n");
		fw.write("################################################################");
		fw.write("####   End problem solving in the Team Space with success   ####");
		fw.write("################################################################");
		fw.write("\n");
				
		reset();
		return transition;
	}

}

