package teamSpace.structure;
//
//  TSPSM.java
//  Blackboard
//
//  Created by Kari on 17.04.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//
import java.util.ArrayList;

/**
 * The TSPSM class represents a PSM (problem solving method) which is defined in the TMST (task structure tree)
 * kept by the {@link PlanLibrary} class. The class will only serve as a keeper of information about a psm.
 * @author Kari Roessland
 */
public class TSPSM extends TSTMSTNode{
	ArrayList partialResults;
	
	/**
	 * The constructor initializes the name, input executor and partial results
	 * of the problem solving method, by calling the super-class and by setting the <code>partialResults</code> 
	 * variable.
	 * @param name - The name of the PSM
	 * @param input - The input of the PSM
	 * @param executor - The executor of the PSM
	 * @param partialResults - The partial results of the subtasks to this PSM
	 */
	public TSPSM(String name, String input, String executor, ArrayList partialResults){
		super(name, input, executor);
		this.partialResults = partialResults;
	}
	
	/**
	 * The method returns the partial results of a PSM
	 * @return The partial results in an {@link ArrayList}
	 */
	public ArrayList getPartialResults(){
		return partialResults;
	}
}
