package teamSpace.structure;
//
//  TSTMSTNode.java
//  Blackboard
//
//  Created by Kari on 17.04.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

/**
 * The TSTMSTNode class represents a executable node (action or psm) which is defined in the TMST (task structure tree)
 * kept by the {@link PlanLibrary} class. The class will only serve as a keeper of information about a node.
 * @author Kari Roessland
 */
public class TSTMSTNode {
	
	String name;
	String input;
	String executor;
	String output;
	
	/**
	 * The constructor instantiate the name, input and executor of the node.
	 * @param name The name of the node
	 * @param input The input of the node
	 * @param executor The executor of the node
	 */
	public TSTMSTNode(String name, String input, String executor){
		this.name = name;
		this.input = input;
		this.executor = executor;
		output = "";
	}
	
	/**
	 * The method returns the name of the node.
	 * @return The name of the node
	 */
	public String getName(){
		return name;
	}
	
	/**
	 * The method returns the input of the node.
	 * @return The input of the node
	 */
	public String getInput(){
		return input;
	}
	
	/**
	 * The method returns the executor of the node.
	 * @return The executor of the node
	 */
	public String getExecutor(){
		return executor;
	}
	
	/**
	 * The method sets the output of the node.
	 * @param output - The output of the node
	 */
	public void setOutput(String output){
		this.output = output;
	}
	
	/**
	 * The method returns the output of the node.
	 * @return The output of the node
	 */
	public String getOutput(){
		return output;
	}
}
