//
//  TeamSpaceAgent.java
//  Blackboard
//
//  Created by Kari on 01.03.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//
//
//  RuleAgent.java
//  Rules
//
//  Created by Kari on 21.02.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//
package teamSpace.agent;

//java
import java.util.ArrayList;
import java.util.Vector;

//jade
import jade.core.*;
import java.util.Vector;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.core.behaviours.SequentialBehaviour;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.DFService;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.ACLMessage;
import jade.domain.FIPANames;

//tmst
import TMST.representation.TMST;

//blackboard
import teamSpace.structure.*;

import CoPS.FileWriter;


/**
 * This class represents the agent that controls the Team Space
 *
 * @author Kari Roessland
 */
public class TeamSpaceAgent extends Agent{

	private Vector teamSpaces = new Vector(); 
	
	/**
	 * The method sets up the agent's behaviours.
	 */
	protected void setup(){
	
		
		//Add Behaviours to this agent
		SequentialBehaviour sb = new SequentialBehaviour(this);
		sb.addSubBehaviour(new DFRegisteringBehaviour(this, this.setServiceDescription()));		
		sb.addSubBehaviour(new CheckTeamSpaceRequestsBehaviour(this));
		this.addBehaviour(sb);

	}
	
	/**
	 * The method returns the actions that are ready to be executed.
	 * @param teamID The ID of the problem solving team using the Team Space
	 * @return The actions in an <code>ArrayList</code>
	 */	
	public ArrayList getActionsReadyToExecute(String teamID){
		return getTeamSpace(teamID).getActionsReadyToExecute();
	}

	/**
	 * The method returns the PSMs that are ready to be composed.
	 * @param teamID The ID of the problem solving team using the Team Space
	 * @return The PSMs in an <code>ArrayList</code>
	 */		
	public ArrayList getPSMsReadyToCompose(String teamID){
		return getTeamSpace(teamID).getPSMsReadyToCompose();
	}
	
	/**
	 * The method returns the processing actions - the actions to be executed
	 * in the current state of the problem solving process
	 * @param teamID The ID of the problem solving team using the Team Space
	 * @return The actions in a list of {@link TSAction} objects
	 */	
	public ArrayList getProcessingActions(String teamID){
		return getTeamSpace(teamID).getProcessingActions();
	}


	/**
	 * The method returns the processing PSMs - the PSMs to be composed
	 * in the current state of the problem solving process
	 * @param teamID The ID of the problem solving team using the Team Space
	 * @return The PSMs in a list of {@link TSPSM} objects
	 */		
	public ArrayList getProcessingPSMs(String teamID){
		return getTeamSpace(teamID).getProcessingPSMs();
	}
	
	/**
	 * The method initializes the Team Space.
	 * @param tmst The task structure tree
	 * @param teamSpaceId The ID of the Team Space
	 * @param requester The name-adress of the requester of the Team Space
	 */
	public boolean initializeTeamSpace(TMST tmst, String teamID, String requester){
		if (tmst == null && (teamID == null || teamID.compareTo("") == 0)){
			return false;
		}else{
			TeamSpace teamSpace = new TeamSpace(tmst, teamID, requester);
			teamSpaces.add(teamSpace);
			
			return teamSpace.initialize();
		}
	}
	

	/**
	 * The method adds a list of actions to the processed actions list 	 
	 * @param actions - The actions that are redily processed
	 * @param teamID The ID of the problem solving team using the Team Space
	 * @return <code>true</code> if the processed actions are successfully added to the result-library and <code>false</code> if not
	 */			
	public boolean setProcessedActions(ArrayList actions, String teamID){
		
		return getTeamSpace(teamID).addProcessedActions(actions);
	}
	
	/**
	 * This method sets the actions that are going to be processed.
	 * @param actions - A list of actions
	 * @param teamID The ID of the problem solving team using the Team Space
	 */
	public void setProcessingActions(ArrayList actions, String teamID){
		getTeamSpace(teamID).setProcessingActions(actions);
	}
	
	
	/**
	 * This method sets the psms that are going to be composed.
	 * @param teamID The ID of the problem solving team using the Team Space
	 * @param psms - A list of psms
	 */
	public void setProcessingPSMs(ArrayList psms, String teamID){
		getTeamSpace(teamID).setProcessingPSMs(psms);
	}
	
	
	/**
	 * The method adds  a list of psms to the processed psms list
	 * @param psms - The list of psms that are processed
	 * @param teamID The ID of the problem solving team using the Team Space
	 * @return True if the processed actions are successfully added to the result-library and False if not
	 */
	public boolean setProcessedPSMs(ArrayList psms, String teamID){
		return getTeamSpace(teamID).addProcessedPSMs(psms);
	}
	
	/**
	 * The method returns the service description of the agent.
	 * @return The service description
	 */
	public ServiceDescription setServiceDescription(){
		//Initialize the service description describing this agent's services
        ServiceDescription sd = new ServiceDescription();
        sd.setType("TEAM_SPACE_COORDINATOR");
		sd.setName(this.getName());
        sd.setOwnership("KariR");  
		
		return sd;
	}
	
	/**
	 * The method prints the current state of the problem solving in the Team Space.
	 * @param teamID The ID of the problem solving team using the Team Space
	 */	
	public void printCurrentState(String teamID){
		getTeamSpace(teamID).printCurrentState();
	}

	/**
	 * The method returns the solution to the initial problem
	 * @param teamID The ID of the problem solving team using the Team Space
	 * @return The solution as a <code>String</code>
	 */	
	public String getProblemSolution(String teamID){
		return getTeamSpace(teamID).getProblemSolution();
	}

	/**
	 * The method returns the initial problem
	 * @param teamID The ID of the problem solving team using the Team Space
	 * @return The initial problem as a <code>String</code>
	 */	
	public String getInitialProblem(String teamID){
		return getTeamSpace(teamID).getInitialProblem();
	}

	/**
	 * The method returns the initial input
	 * @param teamID The ID of the problem solving team using the Team Space
	 * @return The initial input as a <code>String</code>
	 */	
	public String getInitialInput(String teamID){
		return getTeamSpace(teamID).getInitalInput();
	}

	/**
	 * This method returns the FileWriter that handles printing the output of the problem solving steps to a file.
	 * @param teamID The ID of the problem solving team using the Team Space. Each TR has it's own FileWriter, meaning each problem has it's own
	 * filewriter, meaning each Team Space has it's own FileWriter.
	 * @return The file writer - reference
	 */
	public FileWriter getFileWriter(String teamID){
		return getTeamSpace(teamID).getFileWriter();
	}
	
	/**
	 * This method returns the TeamSpace with the specified teamID
	 * @param teamID The teamID
	 * @return A TeamSpace from the list of Team Spaces belonging to this agent
	 */
	public TeamSpace getTeamSpace(String teamID){
		TeamSpace ts = null;
		TeamSpace test = null;
		for(int i = 0; i<teamSpaces.size(); i++){
			test = (TeamSpace)teamSpaces.get(i);
			if(test.getTeamID().compareTo(teamID) == 0){
				ts = test;
				break;
			}
		}
		return ts;
	}

}

