//
//  TeamSpaceAgentBehaviour.java
//  Blackboard
//
//  Created by Kari on 04.03.06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//
package teamSpace.agent;

//java
import java.util.ArrayList;

//jade
import jade.core.Agent;
import jade.proto.AchieveREResponder;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.core.behaviours.Behaviour;
import jade.lang.acl.UnreadableException;
import jade.core.behaviours.DataStore;

import TMST.representation.TMST;

import CoPS.FileWriter;

/**
 * The name of this class should be changed. It will encapsulate all of the TeamSpaceAgent- behaviour
 * from initilizing the team space till the result of the problem solving is present.
 *
 * @author Kari Roessland
 */
public class TeamSpaceAgentBehaviour extends AchieveREResponder{

	private String teamID = "";
	private FileWriter fw = null;
	
	/**
	 * The constructor calls the superconstructor and register a behaviour that should
	 * handle the prepareResultNotification part of the conversation (protocol).
	 * @param agent The agent that this behaviour belongs to.
	 * @param template A template for which messages it should accept as part of this behaviour's protocol.
	 */
	public TeamSpaceAgentBehaviour(Agent agent, MessageTemplate template, DataStore ds){
		super(agent, template, ds);
	}
	
	
	/**
	 * This method is an extention to the same method of the superclass. 
	 * @param request The request message received
	 * @return The response to the request
	 */	
	protected ACLMessage prepareResponse(ACLMessage request){
		
		ACLMessage response = request.createReply();
		
		//NB!also check  the content-language, and if not understood return NOT_UNDERSTOOD performative 
		
		try{
		
			TMST tmst = (TMST)request.getContentObject(); 
			teamID = ""+tmst.getTeamID();
			
			fw = tmst.getFileWriter();
				
			fw.write("\n\n");
			fw.write("################################################################");
			fw.write("########   Start problem solving in the Team Space    ##########");
			fw.write("########   3. Solving of Subtasks                     ##########");
			fw.write("########   4. Integration of Partial Results          ##########");
			fw.write("################################################################");
			fw.write("\n\n");
		
			fw.write("    "+myAgent.getLocalName()+" uses the information in the TMST to initialize the TEAM SPACE structure.");
			fw.write("    Then the problem solving in the TEAM SPACE start. Only the evolution of the problem solving state is");
			fw.write("    printed. Before each of the different states, the rules in the Rule Base are run and the state is updated.");
			fw.write("    Actions ready to execute in a state are communicated to the Problem Solvers(PSs) who return the solution (output)");
			fw.write("    of the actions. PSMs ready to compose in a state are communicated to the Task Responsible(TR) who returns the");
			fw.write("    solution (output). The TEAM SPACE agent receiving solutions from PSs and TR, updates the Result Library with these");
			fw.write("    results. The rules are run again and we get a new problem solving state.\n");

			if (((TeamSpaceAgent)myAgent).initializeTeamSpace(tmst, teamID, request.getSender().getName())){
				response.setPerformative(ACLMessage.AGREE);
				response.setLanguage("PlainText");
				response.setContent("ALL OK - The team-space is successfully created");
				
				fw.write("################################################################");
				fw.write("    Initial task/problem: "+((TeamSpaceAgent)myAgent).getInitialProblem(teamID));
				fw.write("    Input: "+((TeamSpaceAgent)myAgent).getInitialInput(teamID));
				fw.write("    TeamID: "+teamID);
				fw.write("################################################################");
				
				this.registerPrepareResultNotification(new TeamSpacePSBehaviour(myAgent, teamID));

			}else{
				response.setPerformative(ACLMessage.REFUSE);
				response.setLanguage("PlainText");
				response.setContent("FAILURE - The team-space was not possible to create");		 	
			}
		}catch(UnreadableException e){
			e.printStackTrace();
			response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
			response.setLanguage("PlainText");
			response.setContent("FAILURE - The team-space was not possible to create");
		}
		
		
		return response;
	}
	
		
	/**
	 * Information from the last execution of this behaviour is
	 * reset and the behaviour is added to the agent again
	 * @return The transition from this state.
	 */
	public int onEnd(){
		reset();
		myAgent.addBehaviour((Behaviour)this);
		return 0;
	}

}
