package TMST.representation;

//
//  Action.java
//  TMST
//
//  Created by Odd Erik  Gundersen on Wed Feb 25 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

//Java
import java.util.Collection;
import java.util.ArrayList;

//jCreek
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.RelationType;
import jcreek.representation.EntityType;
//import jcreek.representation.IsoPodModel;
//import jcreek.representation.cbr.IsoPodModel;
import jcreek.representation.BasicModel;
import jcreek.representation.Relation;
import jcreek.representation.KnowledgeModel;
//import jcreek.representation.LocalKnowledgeModel;
import jcreek.representation.NameAlreadyExistException;
import jcreek.representation.NoSuchRelationTypeException;
import jcreek.representation.NoSuchRelationException;
import jcreek.representation.EntityTypeNotMatchedException;

//TMST
import TMST.Exceptions.AlreadyHasValueException;

/**
	Class describing the actual, executable bit which may be assigned to an agent. Actions are Problem solving methods which 
	are executable. Actions are described by the type of action it is.Parameters actions may hold:
	<ul>
		<li>Assumption</li>
		<li>Inputs</li>
		<li>Outputs</li>
	</ul>
 
*/
public class Action extends EntityType{ 

	
	/**
		Contains action type as string.
		@param tmst The knowledge model that contains the action
		@param name The name of the action
		@param description Description of the action.
		@param actiontype The type of the action, name as string.b
	 */
	public Action(TMST tmst, String name, String description, String actiontype) throws NameAlreadyExistException{
		super((KnowledgeModel)tmst, name, description);
		System.out.println("Inne i Action-konstruktor"); //Eksekveres
		
		try{
		
			if(!tmst.hasEntity("ACTION")){
				Entity sd = new Entity((KnowledgeModel)tmst, "ACTION", "ACTION");
				sd.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
			}
			this.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity("ACTION"));
						
			if(!tmst.hasEntity("ACTION_TYPE")){
				Entity sd = new Entity((KnowledgeModel)tmst, "ACTION_TYPE", "ACTION_TYPE");
				sd.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
			}

			try{
				new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_ACTION_TYPE", "Relation pointing to the type of a relation", 1.0);
				
			}catch(NameAlreadyExistException e){
				//If the relation type exists, then there is no problem
				System.out.println("HAS_ACTION_TYPE: " +e.getMessage());
			}

			try{
				new ActionType(tmst, actiontype, actiontype);
			}catch(NameAlreadyExistException e){
				System.out.println("ACTION_TYPE: " +e.getMessage());
			}
			new Relation(this, tmst.getRelationType("HAS_ACTION_TYPE"), tmst.getEntity(actiontype));
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/**
	
	 */
	public Action(Entity ent) throws EntityTypeNotMatchedException{
		super(ent);
	}
	
	public void setExecutor(Executor executor){
		//Add relation to the executor
		try{
			new RelationType(this.getKnowledgeModel(), this.getKnowledgeModel().getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_EXECUTOR", "Relation pointing to the type of a relation", 1.0);
		}catch(NameAlreadyExistException e){
		}
		
		new Relation(this, this.getKnowledgeModel().getRelationType("HAS_EXECUTOR"), executor);
		try{
			if(executor.getCostValue(this.getName())<this.getExecutionCostValue()){
				new RelationType(this.getKnowledgeModel(), this.getKnowledgeModel().getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_EXECUTION_COST", "Relation pointing to the type of a relation", 1.0);
			}
		}catch(NameAlreadyExistException e){
		}catch(AlreadyHasValueException e){
			e.printStackTrace();
		}
		new Relation(this, this.getKnowledgeModel().getRelationType("HAS_EXECUTION_COST"), executor.getCost(this.getName()));
	}
	
	/**
		Metohod returning the execution cost as cost entity
		@return Cost The execution cost of this action
	 */
	public Cost getExecutionCost() throws AlreadyHasValueException{
		Cost cost = null;
		Relation rels[] = this.getLocalRelations("HAS_EXECUTION_COST");
		
		if(rels==null) return cost;
		
		if(rels.length>1) throw new AlreadyHasValueException("Actions may only have one execution cost!");
		
		try{
			cost = new Cost(rels[0].getValue());
		}catch(EntityTypeNotMatchedException e){
			e.printStackTrace();
		}
		
		return cost;
	}

	/**
		Method for getting the executor which has been selected to execute the action
		@return Executor The executor of the action
	 **/
	public Executor getExecutor() throws AlreadyHasValueException{
		return this.getExecutionCost().getExecutor();
	}
	
	
	/**
	 * Method for returning the executor which has been selected to execute the action as a String.
	 * @author Kari Roessland
	 * @return the executor as a String
	 */
	public String getExecutorString(){
		String executor = "";
		try{
			executor = this.getExecutor().getName();
		}catch(AlreadyHasValueException e){
			e.printStackTrace();
		}	
		return executor;
	}
	
	/**
	Metohod returning the execution cost as cost entity
	 @return Cost The execution cost of this action
	 */
	public float getExecutionCostValue() throws AlreadyHasValueException{
		float cost = Float.POSITIVE_INFINITY;
		Relation rels[] = this.getLocalRelations("HAS_EXECUTION_COST");
		//Relation rels[] = this.getLocalRelations("HAS_EXECUTION_COST");
		
//		System.out.println("Number of execution costs related to this action: " +rels.length);
		
		if(rels.length == 0) return cost;
		
		if(rels.length>1) throw new AlreadyHasValueException("Action may only have one execution cost!");
		
		try{
			Cost costEntity = new Cost(rels[0].getValue());
			cost = costEntity.getFloatValue();
		}catch(EntityTypeNotMatchedException e){
			//If no cost object, return POSITIVE_INFINITY
		}
		
		//Return the cost as float value
		return cost;
	}	
	
	/**
		Method for checking whether a action is part of the solution space or not.
		@return boolean The method returns true if part of the solution space and false if not.
	 */
	public boolean isSpacePart(){
		try{
			if(this.getLocalRelation("IS_SPACE_PART")!=null){
				return true;
			}
		}catch(NoSuchRelationTypeException e){
			//If there is no such relationtype, this task is not activated.
			//Nothing need to be done and false is returned.
		}catch(NoSuchRelationException e){
			//If there is no such relation, this task is not activated.
			//Nothing need to be done and false is returned.
		}
		return false;
	}
	
	/**
	Method for checking whether a action is part of the solution or not.
	 @return boolean The method returns true if part of the solution and false if not.
	 */
	public boolean isSolutionPart(){
		try{
			if(this.getLocalRelation("IS_SOLUTION_PART")!=null){
				return true;
			}
		}catch(NoSuchRelationTypeException e){
			//If there is no such relationtype, this task is not activated.
			//Nothing need to be done and false is returned.
		}catch(NoSuchRelationException e){
			//If there is no such relation, this task is not activated.
			//Nothing need to be done and false is returned.
		}
		return false;
	}
	
	/**
		Private method for setting the execution cost.
		@param cost The cost entity to relate to
	 */
	protected void setExecutionCost(Cost cost) throws AlreadyHasValueException{
		//Initilization
		KnowledgeModel tmst = this.getKnowledgeModel();
		RelationType hasExecutionCost = null;

		try{
			hasExecutionCost = new RelationType(tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_EXECUTION_COST", "Relation pointing from an action to its execution cost", 1.0);
		}catch(NameAlreadyExistException e){
			//If this relation type already exists, just make new relation of it 
		}
		
		//Get all local has execution cost relations
		Relation rels[] = this.getLocalRelations("HAS_EXECUTION_COST");
		
		//If this action has no has execution cost relation, add this cost as the execution cost
		if(rels.length==0){
			rels = new Relation[1];
			rels[0]=new Relation(this, tmst.getRelationType("HAS_EXECUTION_COST"), cost);
		}
		if(rels.length>2) throw new AlreadyHasValueException("This action has far too many execution cost relations. Only one allowed.");
			
		if(cost.getFloatValue() <= ((Cost)rels[0].getValue()).getFloatValue()){
			rels[0].remove();
			new Relation(this, tmst.getRelationType("HAS_EXECUTION_COST"), cost);
		}
	}
	
	/**
		Method for setting the assumptions related to the action.
		@param ass The assumptions which apply for the action.
	 */ 
	public void setAssumption(Assumption ass){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasAss = tmst.getRelationType("HAS_ASSUMPTION");
		
		if(hasAss==null){
			//Has to make new Relation type
			try{
				hasAss = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_ASSUMPTION", "Relation pointing to an input", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}		
		new Relation(this, hasAss, ass);
	}
	
	/**
		Method for setting the assumptions related to the action.
		@param ass The assumptions which apply for the action.
	 */ 
	public void setAssumptions(Assumption[] ass){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasAss = tmst.getRelationType("HAS_ASSUMPTION");
		
		if(hasAss==null){
			//Has to make new Relation type
			try{
				hasAss = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_ASSUMPTION", "Relation pointing to an input", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		for(int i=0;i<ass.length;i++){
			new Relation(this, hasAss, ass[i]);
		}
	}
	
	/**
		Method for getting the assumptions for the current action.
		@return Assumption[] The assumptions related to the action.
	 */
	public Assumption[] getAssumptions() {
		Assumption[] ass = null;
		
		//Get all relations pointing to realtions of type "has assumption"
		Relation rels[] = this.getLocalRelations("HAS_ASSUMPTION");
		
		/*
			Only one set of assumptions contained by one Assumptions object is allowed, 
			if any assumptions are declared.
			??? Stemmer jo ikke med kode ???
		 */
		if(rels.length==0){
			//Do not do anything, return the null pointer already assigned.
		}else{ 
			ass = new Assumption[rels.length];
			for(int i = 0; i < rels.length; i++){
				try{
					ass[i] = new Assumption(rels[i].getValue());
				}catch(EntityTypeNotMatchedException e){
					e.printStackTrace();
					ass[i] = null;
				}
			}
		}
		return ass;
	}
	
	/**
		Method for adding Input to the action
		@param in The input to add to the action
	 */
	public void setInput(Input in){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasInput = tmst.getRelationType("HAS_INPUT");
		
		if(hasInput==null){
			//Has to make new Relation type
			try{
				hasInput = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_INPUT", "Relation pointing to a input", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		new Relation(this, hasInput, in);
	}
	
	/**
		Method for adding a set of inputs to the action
		@param ins The set of inputs to add to the action
	 */
	public void setInputs(Input[] ins){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasInput = tmst.getRelationType("HAS_INPUT");
		
		if(hasInput==null){
			//Has to make new Relation type
			try{
				hasInput = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_INPUT", "Relation pointing to an input", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		//for every goal in the array goals, make a relation between the goal and this task
		for(int i = 0; i<ins.length; i++){
			try{
				new Relation(this, hasInput, ins[i]);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}
	
	
	/**
		Method for getting all input part the action
		@return Input[] All Input entities related to the action
	*/
	public Input[] getInputs(){
		//Get all relations pointing to input
		Relation rels[] = this.getLocalRelations("INPUT");
	
		if(rels==null){
			return null;
		}
		
		Input ins[] = new Input[rels.length];
	
		//Parse through the relations and get the input they point to ans store them in the reurn value
		for(int i=0; i<rels.length; i++){
			try{
				ins[i] =  new Input(rels[i].getValue());
			}catch(EntityTypeNotMatchedException e){
				e.printStackTrace();
				ins[i] = null;
			}
		}
		
		//Return the array of input values related to this action
		return ins;
	}
	
	/**
	 * Method for getting all input-entities as strings
	 * @author Kari Roessland
	 * @return ArrayList A list of input-entities as strings
	 */
	public ArrayList getInputsString(){
		Relation rels[] = this.getRelations("HAS_INPUT");
		ArrayList retInput = new ArrayList();
		Input input = null;
		
		if (rels != null && rels.length > 0){
			for (int i = 0; i< rels.length; i++){
				try{
					input = new Input(rels[i].getValue());
					retInput.add(input.getName());
				}catch(EntityTypeNotMatchedException e){
					e.printStackTrace();
				}
			}
		}
		return retInput;
	}
	

	/*
		Method for adding output to the action
		@param out The output to add to the action
	*/
	public void setOutput(Output out){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasOutput = tmst.getRelationType("HAS_OUTPUT");
		
		if(hasOutput==null){
			//Has to make new Relation type
			try{
				hasOutput = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_OUTPUT", "Relation pointing to an output", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		new Relation(this, hasOutput, out);
	}
	
	/*
		Method for adding a set of outputs to the action
		@param outs The set of outputs to add to the action
	*/
	public void setOutputs(Output[] outs){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasOutput = tmst.getRelationType("HAS_OUTPUT");
		
		if(hasOutput==null){
			//Has to make new Relation type
			try{
				hasOutput = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_OUTPUT", "Relation pointing to an output", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		//for every goal in the array goals, make a relation between the goal and this task
		for(int i = 0; i<outs.length; i++){
			try{
				new Relation(this, hasOutput, outs[i]);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}
	
	/*
		Method for getting all output entities part the action
		@return Output[] All Output entities related to the action
	*/
	public Output getOutput() throws Exception{
		//Get all relations pointing to problem solving methods
		Relation rels[] = this.getLocalRelations("HAS_OUTPUT");
		
		if(rels==null) return null;
		if(rels.length > 1) throw new Exception("Too many outputs, may only have one ...");
		
		return new Output(rels[0].getValue());
	}
	
	/**
	 * Method for getting the output entity as a string
	 * @author Kari Roessland
	 * @return The name of the output entity
	 */
	public String getOutputString(){
		Relation[] rels = this.getLocalRelations("HAS_OUTPUT");
		String retOutput = "";
		Output output = null;
		if (rels != null && rels.length == 1){
			try{
				output = new Output(rels[0].getValue());
				retOutput = output.getName();
			}catch(EntityTypeNotMatchedException e){
				e.printStackTrace();
			}
		}
		return retOutput;
	}
	
	
	
	/**
	 *	Method for checking whether an action is activated or not.
	 *	@return boolean The method returns true if activated and false if not.
	 */
	public boolean isActivated(){
		try{
			Relation rel = this.getLocalRelation("HAS_ACTIVATION");
			if(rel.getValue().getName().equals("ACTIVATED")){
				return true;
			}
		}catch(NoSuchRelationTypeException e){
			//If there is no such relationtype, this taskis not activated.
			//Nothing need to be done and false is returned.
		}catch(NoSuchRelationException e){
			//If there is no such relation, this task is not activated.
			//Nothing need to be done and false is returned.
		}
		return false;
	}
	
	/**
		Method for setting an action as activated.
	 */
	public void setAsActivated(){
		TMST tmst = (TMST)this.getKnowledgeModel();	
		RelationType isActivated = tmst.getRelationType("HAS_ACTIVATION");
		
		if(isActivated==null){
			//Has to make new Relation type
			try{
				isActivated = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_ACTIVATION", "Relation pointing to an activated entity.", 1.0);
				//System.out.println(" -- Making new relation type: HAS_ACTIVATION");
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		if(!tmst.hasEntity("ACTIVATED")){
			try{
				Entity activated = new Entity((KnowledgeModel)tmst, "ACTIVATED", "Entity for indicating whether a task, a psm or an action is activated.");
				activated.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
				//System.out.println(" -- Making new entity: ACTIVATED");
			}catch(NameAlreadyExistException e){
				//Do nothing, it the entity already exist, there is no problem.
			}catch(NoSuchRelationTypeException e){
				//This should never happen.
				e.printStackTrace();
			}
			
		}

		//When both the relation type HAS_ACTIVATION and the entity ACTIVATED exist, a relation of this type points from this action to the entity ACTIVATED
		new Relation(this, isActivated, tmst.getEntity("ACTIVATED"));
	}
	
	/**
		Method returning the actiontype of this action
		@return ActionType The action type describing this action
	 */
	public ActionType getType() throws Exception{
		//Get all relations pointing to problem solving methods
		Relation rels[] = this.getLocalRelations("HAS_ACTION_TYPE");
		
		if(rels==null){
			throw new Exception("This action does not have a type. It should not be the case! Never.");
		}else if(rels.length>1){
			throw new Exception("This action has more than one type. This should never be the case!");
		}
		try{
			return new ActionType(rels[0].getValue());
		}catch(EntityTypeNotMatchedException e){
			e.printStackTrace();
			return null;
		}
	}
	
	
	/**
	 *  Method returning the task that is executed by this action
	 *  @author Kari Roessland
	 *	@return String A string with the name of the task that is executed by this action
	 */
	public String getTask(){
		Relation rels[] = this.getLocalRelations("EXECUTION_OF");
		String retTask = "";
		Task task = null;
		
		if(rels != null){
			try{
				//The actions can only be connected to one task, and should always be.
				task = new Task(rels[0].getValue());
				retTask = task.getName();
			}catch(EntityTypeNotMatchedException e){
				e.printStackTrace(); 
			}
		
		}
		
		return retTask;
	}
	
	
	
	/**
		Method which has to be overriden (from the class EntityType)
		@param value The entity to match againts the class Action
		@return boolean True if value matches the Action class, and false if not.
	*/
	public static boolean matches(Entity value){
		TMST tmst = (TMST)value.getKnowledgeModel();

		Entity ent = tmst.getEntity("ACTION");
		
		if (ent == null) return false; 
		
		Collection subclasses = ent.getAllSubclassesCollection();
		
		if(value.getName().equals("ACTION")){
			return false;		
		}else if(subclasses.contains(value)){
			return true;
		}
		return false;
	}
}
