package TMST.representation;
//
//  AssumptionType.java
//  TMST
//
//  Created by Odd Erik  Gundersen on Fri Apr 16 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

//Java
import java.util.Collection;

//jCreek
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.EntityType;
//import jcreek.representation.cbr.IsoPodModel;
import jcreek.representation.BasicModel;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.NameAlreadyExistException;
import jcreek.representation.EntityTypeNotMatchedException;

/**
	Class implementing an assumption type.
*/
public class AssumptionType extends EntityType{	
	
	/*FIELDS*/
	final static int EPISTEMOLOGICAL_AVAILABILITY = 0;
	final static int EPISTEMOLOGICAL_PROPERTY = 1;
	final static int PRAGMATIC = 2;
	final static int TELEOLOGICAL = 3;	
	
	/**
		Constructor for setting up an entity for the new assumption type and relation it to a generic assumption type.
		@param tmst The knowledge model this assumption type belongs to
		@param name The name of the assumption type
		@param description The description of the type
	*/
	public AssumptionType(TMST tmst, String name, String description) throws NameAlreadyExistException{
		super((KnowledgeModel)tmst, name, description);
		try{
			if(!tmst.hasEntity("ASSUMPTION_TYPE")){
				Entity cons= new Entity((KnowledgeModel)tmst, "ASSUMPTION_TYPE", "A generic entity all assumption type  entities has to relate to, as a indication that they are assumption type entities.");
				cons.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
			}
			this.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity("ASSUMPTION_TYPE"));
		}catch(Exception e){
			e.printStackTrace();
		}
	}

	public AssumptionType(Entity ent) throws EntityTypeNotMatchedException{
		super(ent);
	}
	
	/**
		Method which has to be overriden (from the class EntityType)
	*/
	public static boolean matches(Entity value){		
		TMST tmst = (TMST)value.getKnowledgeModel();
		Entity ent = tmst.getEntity("ASSUMPTION_TYPE");
		
		if (ent == null) return false; 

		Collection subclasses = ent.getAllSubclassesCollection();

		if(value.getName().equals("ASSUMPTION_TYPE")){
			return false;		
		}else if(subclasses.contains(value)){
			return true;
		}
		return false;
	}
}