package TMST.CheckUpExample;

//
//  CheckUpTMST.java
//  TMST
//
//  Created by Kari on 17.11.05. 
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

//Standard java bibliotek
import java.util.*;
import java.lang.Object;

//jCreek 
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.LocalKnowledgeModel;

//TMST
import TMST.representation.Action;
import TMST.representation.Task;
import TMST.representation.ProblemSolvingMethod;
import TMST.representation.Goal;
import TMST.representation.Assumption;
import TMST.representation.ControlInformation;
import TMST.representation.Input;
import TMST.representation.Output;
import TMST.representation.Value;
import TMST.representation.TMST;
import jcreek.representation.EntityTypeNotMatchedException;



/**
 * This class defines and implements the TMST used in our checkup-example application.
 * @author Kari Roessland
 */
public class CheckUpTMST{


	/**
	 * The constructor.
	 * @param filename The name of the file where the TMST is stored.
	 */
	public CheckUpTMST(String filename){
		long time = System.currentTimeMillis();
		TMST tmst = generateModel();
		
		
		
		System.out.println("Time to generate model: "+(System.currentTimeMillis()-time)+" ms");
		
		tmst.dependencyAnalyzation();
		tmst.analyzeAndPrint();
														
		try{
			tmst.saveAs(filename);
		}
		catch(java.io.IOException e){
			e.printStackTrace();
		}
		
	}


	/**
	 * Method for generating a task-method-subtask tree of the checkup-example application.
	 */
	public static TMST generateModel(){
		
		String test;
		TMST tmst = new TMST();
		
		tmst.setName("CHECKUP_EXAMPLE");
		tmst.setTitle("A TMST Describing a Checkup at the Health Center");
		tmst.setAuthor("Kari Roessland");
		tmst.setDescription("This is an example of how a TMST for performing a checkup at a health center may be built."); 
		try{
			
			
			//FIND ALTERNATIVE DIAGNOSES
			
			//All Actions
			Action gettingPatientInfoDB1 = new Action(tmst, "gettingPatientInfoDB1", "action", "GetPatientInfoDB1");
			gettingPatientInfoDB1.setInput(new Input(tmst, "gettingPatientInfoDB1_input1", "input", "(Patient :name x)"));
			gettingPatientInfoDB1.setOutput(new Output(tmst, "gettingPatientInfoDB1_output1", "output", "(Patient :name x :information y)"));

			Action gettingPatientInfoMA1 = new Action(tmst, "gettingPatientInfoMA1", "action", "GetPatientInfoMA1"); 
			gettingPatientInfoMA1.setInput(new Input(tmst, "gettingPatientInfoMA1_input1", "input", "(Patient :name x)"));
			gettingPatientInfoMA1.setOutput(new Output(tmst, "gettingPatientInfoMA1_output1", "output", "(Patient :name x :information y)"));
						
			Action interrogatingPatient = new Action(tmst, "interrogatingPatient", "action", "Interrogate");
			interrogatingPatient.setInput(new Input(tmst, "interrogatingPatient_input1", "input", "(Patient :name x :information y)")); 
			interrogatingPatient.setOutput(new Output(tmst, "interrogatingPatient_output1", "output", "(Patient :name x :information y :symptoms (Symptom :description z))"));
			
			Action updatingPatientInfoDB1 = new Action(tmst, "updatingPatientInfoDB1", "action", "UpdatePatientInfoDB1");
			updatingPatientInfoDB1.setInput(new Input(tmst, "updatingPatientInfoDB1_input1", "input", "(Patient :name x :information y :symptoms (Symptom :description z))"));
			updatingPatientInfoDB1.setOutput(new Output(tmst, "updatingPatientInfoDB1_input2", "output", "(UpdateStatus :status k)"));
			
			Action updatingPatientInfoMA1 = new Action(tmst, "updatingPatientInfoMA1", "action", "UpdatePatientInfoMA1");
			updatingPatientInfoMA1.setInput(new Input(tmst, "updatingPatientInfoMA1_input1", "input", "(Patient :name x :information y :symptoms (Symptom :description z))"));
			updatingPatientInfoMA1.setOutput(new Output(tmst, "updatingPatientInfoMA1_input2", "output", "(UpdateStatus :status k)"));
			
			Action gettingAlternativeDiagnosesBook = new Action(tmst, "gettingAlternativeDiagnosesBook", "action", "GetAlternativeDiagnosesBook");
			gettingAlternativeDiagnosesBook.setInput(new Input(tmst, "gettingAlternativeDiagnosesBook_input1", "input", "(Patient :name x :information y :symptoms (Symptom :description z))"));
			gettingAlternativeDiagnosesBook.setOutput(new Output(tmst, "gettingAlternativeDiagnosesBook_output1", "output", "(Patient :name x :diagnoses (Diagnose :description k))"));
			
			Action gettingAlternativeDiagnosesDiseaseDB = new Action(tmst, "gettingAlternativeDiagnosesDiseaseDB", "action", "GetAlternativeDiagnosesDB");
			gettingAlternativeDiagnosesDiseaseDB.setInput(new Input(tmst, "gettingAlternativeDiagnosesDiseaseDB_input1", "input", "(Patient :name x :information y :symptoms (Symptom :description z))"));
			gettingAlternativeDiagnosesDiseaseDB.setOutput(new Output(tmst, "gettingAlternativeDiagnosesDiseaseDB_output1", "output", "(Patient :name x :diagnoses (Diagnose :description k))"));
			
			//All tasks
			Task findAlternativeDiagnoses = new Task(tmst, "findAlternativeDiagnoses", "task");
			findAlternativeDiagnoses.setGoal(new Goal(tmst, "findAlternativeDiagnoses_goal", "goal", "(Patient :name x :diagnoses (Diagnose :description y))"));
			findAlternativeDiagnoses.setInput(new Input(tmst, "findAlternativeDiagnoses_input1", "input", "(Patient :name x)"));
			findAlternativeDiagnoses.setOutput(new Output(tmst, "findAlternativeDiagnoses_output1", "output", "(Patient :name x :diagnoses (Diagnose :description y))"));
			
			Task getPatientInfo1 = new Task(tmst, "getPatientInfo1", "task");
			getPatientInfo1.setGoal(new Goal(tmst, "getPatientInfo1_goal", "goal",  "(Patient :name x :information y)"));
			getPatientInfo1.setInput(new Input(tmst, "getPatientInfo1_input1", "input", "(Patient :name x)"));
			getPatientInfo1.setOutput(new Output(tmst, "getPatientInfo1_output1", "output",  "(Patient :name x :information y)"));
			
			Task interrogatePatient = new Task(tmst, "interrogatePatient", "task");
			interrogatePatient.setGoal(new Goal(tmst, "interrogatePatient_goal", "goal", "(Patient :name x :information y :symptoms (Symptom :description z))"));
			interrogatePatient.setInput(new Input(tmst, "interrogatePatient_input1", "input", "(Patient :name x :information y)"));
			interrogatePatient.setOutput(new Output(tmst, "interrogatePatient_output1", "output", "(Patient :name x :information y :symptoms (Symptom :description z))"));
			
			Task updatePatientInfo1 = new Task(tmst, "updatePatientInfo1", "task");
			updatePatientInfo1.setGoal(new Goal(tmst, "updatePatientInfo1_goal", "goal", "(UpdateStatus :status k)"));
			updatePatientInfo1.setInput(new Input(tmst, "updatePatientInfo1_input1", "input", "(Patient :name x :information y :symptoms (Symptom :description z))"));
			updatePatientInfo1.setOutput(new Output(tmst, "updatePatientInfo1_output1", "output", "(UpdateStatus :status k)"));

			Task getAlternativeDiagnoses = new Task(tmst, "getAlternativeDiagnoses", "task");
			getAlternativeDiagnoses.setGoal(new Goal(tmst, "getAlternativeDiagnoses_goal", "goal", "(Patient :name x :diagnoses (Diagnose: description: y))"));
			getAlternativeDiagnoses.setInput(new Input(tmst, "getAlternativeDiagnoses_input1", "input", "(Patient :name x :information y :symptoms (Symptom :description z))"));
			getAlternativeDiagnoses.setOutput(new Output(tmst, "getAlternativeDiagnoses_output1", "output", "(Patient :name x :diagnoses (Diagnose :description y))"));			
			
			//All problem solving methods
			ProblemSolvingMethod findingAlternativeDiagnoses = new ProblemSolvingMethod(tmst, "findingAlternativeDiagnoses", "PSM");
			findingAlternativeDiagnoses.setInput(new Input(tmst, "findingAlternativeDiagnoses_input1", "input", "(Patient :name x)"));
			findingAlternativeDiagnoses.setOutput(new Output(tmst, "findingAlternativeDiagnoses_output1", "output", "(Patient :name x :diagnoses (Diagnose :description y))"));
			
			
			//Relating the structures to each other
			findAlternativeDiagnoses.setProblemSolvingMethod(findingAlternativeDiagnoses);
			
			findingAlternativeDiagnoses.setSubtask(getPatientInfo1);
			findingAlternativeDiagnoses.setSubtask(interrogatePatient);
			findingAlternativeDiagnoses.setSubtask(updatePatientInfo1);
			findingAlternativeDiagnoses.setSubtask(getAlternativeDiagnoses); 			
			getPatientInfo1.setAction(gettingPatientInfoDB1);
			getPatientInfo1.setAction(gettingPatientInfoMA1);
			interrogatePatient.setAction(interrogatingPatient);
			updatePatientInfo1.setAction(updatingPatientInfoDB1);
			updatePatientInfo1.setAction(updatingPatientInfoMA1);
			getAlternativeDiagnoses.setAction(gettingAlternativeDiagnosesDiseaseDB);
			getAlternativeDiagnoses.setAction(gettingAlternativeDiagnosesBook);
			
			
			
			//GATHER EVIDENCE
			//All actions
			Action gettingHistInfoDB1 = new Action(tmst,"gettingHistInfoDB1", "action", "GetHistInfoDB1");
			gettingHistInfoDB1.setInput(new Input(tmst, "gettingHistInfoDB1_input1", "input", "(Patient :name x :diagnoses (Diagnose :description y))"));
			gettingHistInfoDB1.setOutput(new Output(tmst, "gettingHistInfoDB1_output1", "output", "(Patient :name x :historical_information z :diagnoses (Diagnose :description y))"));
			
			Action gettingHistInfoMA1 = new Action(tmst, "gettingHistInfoMA1", "action", "GetHistInfoMA1");
			gettingHistInfoMA1.setInput(new Input(tmst, "gettingHistInfoMA1_input1", "input", "(Patient :name x :diagnoses (Diagnose :description y))"));
			gettingHistInfoMA1.setOutput(new Output(tmst, "gettingHistInfoMA1_output1", "output", "(Patient :name x :historical_information z :diagnoses (Diagnose :description y))"));
			
			Action performingPhysicalExamination1 = new Action(tmst, "performingPhysicalExamination1", "action", "PerformPhyExamination1");
			performingPhysicalExamination1.setInput(new Input(tmst, "performingPhysicalExamination1_input1", "input", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z))"));
			performingPhysicalExamination1.setOutput(new Output(tmst, "performingPhysicalExamination1_output1", "output", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z) :physical_evidences (PhysicalEvidence :description k))"));


			Action gettingHistInfoDB2 = new Action(tmst,"gettingHistInfoDB2", "action", "GetHistInfoDB2");
			gettingHistInfoDB2.setInput(new Input(tmst, "gettingHistInfoDB2_input1", "input", "(Patient :name x :diagnoses (Diagnose :description y))"));
			gettingHistInfoDB2.setOutput(new Output(tmst, "gettingHistInfoDB2_output1", "output", "(Patient :name x :historical_information z :diagnoses (Diagnose :description y))"));
			
			Action gettingHistInfoMA2 = new Action(tmst, "gettingHistInfoMA2", "action", "GetHistInfoMA2");
			gettingHistInfoMA2.setInput(new Input(tmst, "gettingHistInfoMA2_input1", "input", "(Patient :name x :diagnoses (Diagnose :description y))"));
			gettingHistInfoMA2.setOutput(new Output(tmst, "gettingHistInfoMA2_output1", "output", "(Patient :name x :historical_information z :diagnoses (Diagnose :description y))"));

			Action performingPhysicalExamination2 = new Action(tmst, "performingPhysicalExamination2", "action", "PerformPhyExamination2");
			performingPhysicalExamination2.setInput(new Input(tmst, "performingPhysicalExamination2_innput1", "input", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z))"));
			performingPhysicalExamination2.setOutput(new Output(tmst, "performingPhysicalExamination2_output1", "output", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z) :physical_evidences (PhysicalEvidence :description k))"));
			
			Action gettingPatientInfoDB2 = new Action(tmst, "gettingPatientInfoDB2", "action", "GetPatientInfoDB2");
			gettingPatientInfoDB2.setInput(new Input(tmst, "gettingPatientInfoDB2_input1", "input", "(Patient :name x)"));
			gettingPatientInfoDB2.setOutput(new Output(tmst, "gettingPatientInfoDB2_output1", "output", "(Patient :name x :information y)"));

			Action gettingPatientInfoMA2 = new Action(tmst, "gettingPatientInfoMA2", "action", "GetPatientInfoMA2"); 
			gettingPatientInfoMA2.setInput(new Input(tmst, "gettingPatientInfoMA2_input1", "input", "(Patient :name x)"));
			gettingPatientInfoMA2.setOutput(new Output(tmst, "gettingPatientInfoMA2_output1", "output", "(Patient :name x :information y)"));
			
			Action takingBloodtest = new Action(tmst, "takingBloodtest", "action", "TakeBloodtest");
			takingBloodtest.setInput(new Input(tmst, "takingBloodtest_input1", "input", "(Patient :name x :information y)"));
			takingBloodtest.setOutput(new Output(tmst, "takingBloodtest_output1", "output", "(Patient :name x :information y :blood_sample_reference z)"));
			
			Action analyzingBloodtestMech = new Action(tmst, "analyzingBloodtestMech", "action", "AnalyzeBloodtestMech");
			analyzingBloodtestMech.setInput(new Input(tmst, "analyzingBloodtestMech_input1", "input", "(Patient :name x :information y :blood_sample_reference z)"));
			analyzingBloodtestMech.setOutput(new Output(tmst, "analyzingBloodtestMech_output1", "output", "(Patient :name x :information y :blood_sample_reference z :test_results (TestResult :description k))"));			

			Action analyzingBloodtestMan = new Action(tmst, "analyzingBloodtestMan", "action", "AnalyzeBloodtestMan");
			analyzingBloodtestMan.setInput(new Input(tmst, "analyzingBloodtestMan_input1", "input", "(Patient :name x :information y :blood_sample_reference z)"));
			analyzingBloodtestMan.setOutput(new Output(tmst, "analyzingBloodtestMan_output1", "output", "(Patient :name x :information y :blood_sample_reference z :test_results (TestResult :description k))"));
			
			
			Action updatingPatientInfoDB2 = new Action(tmst, "updatingPatientInfoDB2", "action", "UpdatePatientInfoDB2");
			updatingPatientInfoDB2.setInput(new Input(tmst, "updatingPatientInfoDB2_input1", "input", "(Patient :name x :information y :blood_sample_reference z :test_results (TestResult :description m))"));
			updatingPatientInfoDB2.setOutput(new Output(tmst, "updatingPatientInfoDB2_input2", "output", "(UpdateStatus :status k)"));
			
			Action updatingPatientInfoMA2 = new Action(tmst, "updatingPatientInfoMA2", "action", "UpdatePatientInfoMA2");
			updatingPatientInfoMA2.setInput(new Input(tmst, "updatingPatientInfoMA2_input1", "input", "(Patient :name x :information y :blood_sample_reference z :test_results (TestResult :description m))"));
			updatingPatientInfoMA2.setOutput(new Output(tmst, "updatingPatientInfoMA2_input2", "output", "(UpdateStatus :status k)"));
			
			

			//All tasks
			Task gatherEvidence = new Task(tmst, "gatherEvidence", "task");
			gatherEvidence.setGoal(new Goal(tmst, "gatherEvidence_goal", "goal", "(Patient :name x :diagnoses (Diagnose :description y) :evidence_conclusions z)"));
			gatherEvidence.setInput(new Input(tmst, "gatherEvidence_input1", "input", "(Patient :name x :diagnoses (Diagnose :description y))"));
			gatherEvidence.setOutput(new Output(tmst, "gatherEvidence_output1", "output", "(Patient :name x :diagnoses (Diagnose :description y) :evidence_conclusions z)"));			
			
			Task getHistInfo1 = new Task(tmst, "getHistInfo1", "task");
			getHistInfo1.setGoal(new Goal(tmst, "getHistInfo1_goal", "goal", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z))"));
			getHistInfo1.setInput(new Input(tmst, "getHistInfo1_input1", "input", "(Patient :name x :diagnoses (Diagnose :description z))"));
			getHistInfo1.setOutput(new Output(tmst, "getHistInfo1_output1", "output", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z))"));
			
			Task performPhysicalExamination1 = new Task(tmst, "performPhysicalExamination1", "task");
			performPhysicalExamination1.setGoal(new Goal(tmst, "performPhysicalExamination1_goal", "goal", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z) :physical_evidences (PhysicalEvidence :description k))"));
			performPhysicalExamination1.setInput(new Input(tmst, "performPhysicalExamination1_input1", "input", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z))"));
			performPhysicalExamination1.setOutput(new Output(tmst, "performPhysicalExamination1_output1", "output", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z) :physical_evidences (PhysicalEvidence :description k))"));			
			
			Task getHistInfo2 = new Task(tmst, "getHistInfo2", "task");
			getHistInfo2.setGoal(new Goal(tmst, "getHistInfo2_goal", "goal", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z))"));
			getHistInfo2.setInput(new Input(tmst, "getHistInfo2_input1", "input", "(Patient :name x :diagnoses (Diagnose :description z))"));
			getHistInfo2.setOutput(new Output(tmst, "getHistInfo2_output1", "output", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z))"));
			
			Task performPhysicalExamination2 = new Task(tmst, "performPhysicalExamination2", "task");
			performPhysicalExamination2.setGoal(new Goal(tmst, "performPhysicalExamination2_goal", "goal", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z) :physical_evidences (PhysicalEvidence :description k))"));
			performPhysicalExamination2.setInput(new Input(tmst, "performPhysicalExamination2_input1", "input", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z))"));
			performPhysicalExamination2.setOutput(new Output(tmst, "performPhysicalExamination2_output1", "output", "(Patient :name x :historical_information y :diagnoses (Diagnose :description z) :physical_evidences (PhysicalEvidence :description k))"));				
			
			Task takeLaboratoryTest = new Task(tmst, "takeLaboratoryTest", "task");
			takeLaboratoryTest.setGoal(new Goal(tmst, "takeLaboratoryTest_goal", "goal", "(Patient :name x :test_results (TestResult :description y))"));
			takeLaboratoryTest.setInput(new Input(tmst, "takeLaboratoryTest_input1", "input", "(Patient :name x)"));
			takeLaboratoryTest.setOutput(new Output(tmst, "takeLaboratoryTest_output1", "output", "(Patient :name x :test_results (TestResult :description y))"));

			Task getPatientInfo2 = new Task(tmst, "getPatientInfo2", "task");
			getPatientInfo2.setGoal(new Goal(tmst, "getPatientInfo2_goal", "goal",  "(Patient :name x :information y)"));
			getPatientInfo2.setInput(new Input(tmst, "getPatientInfo2_input1", "input", "(Patient :name x)"));
			getPatientInfo2.setOutput(new Output(tmst, "getPatientInfo2_output1", "output",  "(Patient :name x :information y)"));
			
			Task getBlood = new Task(tmst, "getBlood", "task");
			getBlood.setGoal(new Goal(tmst, "getBlood_goal", "goal", "(Patient :name x :information y :blood_sample_reference z)"));
			getBlood.setInput(new Input(tmst, "getBlood_input1", "input", "(Patient :name x :information y)"));
			getBlood.setOutput(new Output(tmst, "getBlood_output1", "output", "(Patient :name x :information y :blood_sample_reference z)"));

			Task analyzeBlood = new Task(tmst, "analyzeBlood", "task");
			analyzeBlood.setGoal(new Goal(tmst, "analyzeBlood_goal", "goal", "(Patient :name x :information y :blood_sample_reference z :test_results (TestResult :description m))"));
			analyzeBlood.setInput(new Input(tmst, "analyzeBlood_input1", "input", "(Patient :name x :information y :blood_sample_reference z)"));
			analyzeBlood.setOutput(new Output(tmst, "analyzeBlood_output1", "output", "(Patient :name x :information y :blood_sample_reference z :test_results (TestResult :description m))"));
			
			Task updatePatientInfo2 = new Task(tmst, "updatePatientInfo2", "task");
			updatePatientInfo2.setGoal(new Goal(tmst, "updatePatientInfo2_goal", "goal", "(UpdateStatus :status k)"));
			updatePatientInfo2.setInput(new Input(tmst, "updatePatientInfo2_input1", "input", "(Patient :name x :information y :blood_sample_reference z :test_results (TestResult :description m))"));
			updatePatientInfo2.setOutput(new Output(tmst, "updatePatientInfo2_output1", "output", "(UpdateStatus :status k)"));

			
			//All problem solving methods
			ProblemSolvingMethod gatheringSimpleEvidence = new ProblemSolvingMethod(tmst, "gatheringSimpleEvidence", "PSM");
			gatheringSimpleEvidence.setInput(new Input(tmst, "gatheringSimpleEvidence_input1", "input", "(Patient :name x :diagnoses (Diagnose :description z))"));
			gatheringSimpleEvidence.setOutput(new Output(tmst, "gatheringSimpleEvidence_output1", "output", "(Patient :name x :diagnoses (Diagnose :description z) :evidence_conclusions y)"));

			ProblemSolvingMethod gatheringAdvancedEvidence = new ProblemSolvingMethod(tmst, "gatheringAdvancedEvidence", "PSM");
			gatheringAdvancedEvidence.setInput(new Input(tmst, "gatheringAdvancedEvidence_input1", "input", "(Patient :name x :diagnoses (Diagnose :description z))"));
			gatheringAdvancedEvidence.setOutput(new Output(tmst, "gatheringAdvancedEvidence_output1", "output", "(Patient :name x :diagnoses (Diagnose :description z) :evidence_conclusions y)"));
			
			ProblemSolvingMethod performingBloodTest = new ProblemSolvingMethod(tmst, "performingBloodTest", "PSM");
			performingBloodTest.setInput(new Input(tmst, "performingBloodTest_input1", "input", "(Patient :name x)"));
			performingBloodTest.setOutput(new Output(tmst, "performingBloodTest_output1", "output", "(Patient :name x :test_results (TestResult :description y))"));
		
			
			//Relating the structures to each other
			
			gatherEvidence.setProblemSolvingMethod(gatheringSimpleEvidence);
			
			gatheringSimpleEvidence.setSubtask(getHistInfo1);
			getHistInfo1.setAction(gettingHistInfoDB1);
			getHistInfo1.setAction(gettingHistInfoMA1);
			gatheringSimpleEvidence.setSubtask(performPhysicalExamination1);
			performPhysicalExamination1.setAction(performingPhysicalExamination1);
			
			gatherEvidence.setProblemSolvingMethod(gatheringAdvancedEvidence);
			
			gatheringAdvancedEvidence.setSubtask(getHistInfo2);
			getHistInfo2.setAction(gettingHistInfoDB2);
			getHistInfo2.setAction(gettingHistInfoMA2);
			gatheringAdvancedEvidence.setSubtask(performPhysicalExamination2);
			performPhysicalExamination2.setAction(performingPhysicalExamination2);
			
			gatheringAdvancedEvidence.setSubtask(takeLaboratoryTest);
			
			takeLaboratoryTest.setProblemSolvingMethod(performingBloodTest);
			
			performingBloodTest.setSubtask(getPatientInfo2);
			getPatientInfo2.setAction(gettingPatientInfoDB2);
			getPatientInfo2.setAction(gettingPatientInfoMA2);
			performingBloodTest.setSubtask(getBlood);
			getBlood.setAction(takingBloodtest);
			performingBloodTest.setSubtask(analyzeBlood);
			analyzeBlood.setAction(analyzingBloodtestMan);
			analyzeBlood.setAction(analyzingBloodtestMech);
			performingBloodTest.setSubtask(updatePatientInfo2);
			updatePatientInfo2.setAction(updatingPatientInfoDB2);
			updatePatientInfo2.setAction(updatingPatientInfoMA2);
			
			
			
			//DO CHECKUP
			//All tasks
			Task doCheckup = new Task(tmst, "doCheckup", "task");
			doCheckup.setGoal(new Goal(tmst, "doCheckup_goal", "goal", "(Patient :name x :finalDiagnose y"));
			doCheckup.setInput(new Input(tmst, "doCheckup_input1", "input", "(Patient :name x)"));
			doCheckup.setOutput(new Output(tmst, "doCheckup_output1", "output", "(Patient :name x :finalDiagnose y)"));			
			
			//All problem solving methods
			ProblemSolvingMethod performingCheckup = new ProblemSolvingMethod(tmst, "performingCheckup", "PSM");
			performingCheckup.setInput(new Input(tmst, "performingCheckup_input1", "input", "(Patient :name x)"));
			performingCheckup.setOutput(new Output(tmst, "performingCheckup_output1", "output", "(Patient :name x :finalDiagnose y)"));
			
			
			//Relating the structures to eachother
			
			doCheckup.setProblemSolvingMethod(performingCheckup);
			performingCheckup.setSubtask(findAlternativeDiagnoses);
			performingCheckup.setSubtask(gatherEvidence);
			
			try{
				tmst.setAsInitialProblem(new Task(tmst.getEntity("doCheckup")));
			}catch(EntityTypeNotMatchedException e ){
				System.out.println("FEIL VED SETTING AV INITIAL PROBLEM");
				e.printStackTrace();
			}
			
		}catch(Exception e){
			e.printStackTrace();
		}
		
		return tmst;
	}


	/**
	 * The main method used to run this class. When the class is run - the TMST defined here is
	 * written to the file that was specified in this class' constructor.
	 */
	public static void main(String[] argv){
		//Takes filname as input
		new CheckUpTMST(argv[0]);
		System.exit(0);
	}


}
