package TMST.representation;

//
//  ControlInformation.java
//  TMST
//
//  Created by Odd Erik  Gundersen on Tue Feb 17 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

import java.util.Collection;
import jcreek.representation.Entity;
import jcreek.representation.EntityType;
import jcreek.representation.RelationType;
import jcreek.representation.Relation;
//import jcreek.representation.cbr.IsoPodModel;
import jcreek.representation.BasicModel;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.NameAlreadyExistException;
import jcreek.representation.EntityTypeNotMatchedException;


import java.util.ArrayList;

/**
	Wrapper class for the Control Information 
*/

public class ControlInformation extends EntityType{

	public ControlInformation(TMST tmst, String name, String description) throws NameAlreadyExistException{
		super((KnowledgeModel)tmst, name, description);
		try{
			if(!tmst.hasEntity("CONTROL_INFORMATION")){
				Entity cons= new Entity((KnowledgeModel)tmst, "CONTROL_INFORMATION", "A generic entity all control information entityies has to relate to, as a indication that they are control information entities.");
				cons.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
			}
			this.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity("CONTROL_INFORMATION"));
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	public ControlInformation(Entity ent) throws EntityTypeNotMatchedException{
		super(ent);
	}
	/**
		Method for relating an io dependency to this control information entity
		@param ioDependency The io dependecy to relate to as String.
	*/ 
	public void setIODependency(String ioDependency){
		//Get the TMST this entity is a part of
		TMST tmst = (TMST)this.getKnowledgeModel();
		
		//Relate the value to this output			
		RelationType hasDependency = tmst.getRelationType("HAS_IO_DEPENDENCY");
		
		if(hasDependency==null){
			//Has to make new Relation type
			try{
				hasDependency = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_IO_DEPENDENCY", "Relation pointing to a io dependency", 1.0);
			}catch(NameAlreadyExistException e){
				e.printStackTrace();
			}
		}
		
		try{
			new Value(tmst, ioDependency, "Value repesenting the io dependency " +ioDependency);
		}catch(NameAlreadyExistException e){
			System.out.println("GOAL: " +e.getMessage());
		}
		new Relation(this, hasDependency, tmst.getEntity(ioDependency));
	}
	
	/**
		Method for relating an io dependency to this control information entity
		@param ioDependency The io dependecy to relate to as a Value class instance.
	 */ 
	public void setIODependency(Value ioDependency){
		//Get the TMST this entity is a part of
		TMST tmst = (TMST)this.getKnowledgeModel();
		
		//Relate the value to this output			
		RelationType hasDependency = tmst.getRelationType("HAS_IO_DEPENDENCY");
		
		if(hasDependency==null){
			//Has to make new Relation type
			try{
				hasDependency = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_IO_DEPENDENCY", "Relation pointing to a io dependency", 1.0);
			}catch(NameAlreadyExistException e){
				e.printStackTrace();
			}
		}
				
		new Relation(this, hasDependency, ioDependency);
	}
	
	/**
	 * Method for getting the IODependecies of this ControlInformation
	 * @return ArrayList a list of IODependecies related to this ControlInformation - the list contains Strings
	 * @author Kari Roessland
	 */
	public ArrayList getIODependencies(){
		Relation rels[] =this.getLocalRelations("HAS_IO_DEPENDENCY");
		Entity ent = null;
		Value ioDependecy = null;
		ArrayList retIODependencies = new ArrayList();
		
		
		if(rels != null){
			for(int i=0; i<rels.length; i++){
				try{
					ent = rels[i].getValue();
					ioDependecy = new Value(ent);
					retIODependencies.add(ioDependecy.getName());
				
				}catch(EntityTypeNotMatchedException e){
					e.printStackTrace();
				}
			}
		}
		return retIODependencies;
	}

	
	/*
		Method which has to be overriden (from the class EntityType)
	*/
	public static boolean matches(Entity value){
		TMST tmst = (TMST)value.getKnowledgeModel();
		Entity ent = tmst.getEntity("CONTROL_INFORMATION");
		
		if (ent == null) return false; 
		
		Collection subclasses = ent.getAllSubclassesCollection();
		
		if(value.getName().equals("CONTROL_INFORMATION")){
			return false;		
		}else if(subclasses.contains(value)){
			return true;
		}
		return false;
	}
}
