package TMST.representation;

//
//  Cost.java
//  TMST
//
//  Created by Odd Erik  Gundersen on Sat May 22 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

//Java
import java.util.Collection;

//jCreek
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.RelationType;
import jcreek.representation.EntityType;
//import jcreek.representation.cbr.IsoPodModel;
import jcreek.representation.BasicModel;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.NameAlreadyExistException;
import jcreek.representation.EntityTypeNotMatchedException;

//TMST
import TMST.Exceptions.AlreadyHasValueException;
	
/**
	Class representing the cost of having an executor performing an action.
*/
public class Cost extends EntityType{

	public Cost(TMST tmst, String name, String description, Entity owner, String costValue) throws NameAlreadyExistException{
		super((KnowledgeModel)tmst, name, description);	
		
		//START DEBUGGING CODE
		//System.out.println("\n\n\n ### COST CONSTRUCTOR");
		//System.out.println("	|-- name: " +name);
		//System.out.println("	|-- owner: " +owner.getName());
		//END DEBUGGING CODE
		
		try{
			//The model need a Value-object 
			if(!tmst.hasEntity("COST")){
				Entity value = new Entity((KnowledgeModel)tmst, "COST", "COST");
				value.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
			}		
			this.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity("COST"));
			
			//Relate the executor to this output			
			RelationType hasCost = tmst.getRelationType("HAS_COST");
				
			if(hasCost==null){
				//Has to make new Relation type
				hasCost = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_COST", "Relation pointing from an executor to a cost entity", 1.0);
			}
			
			new Relation(this, hasCost, owner);	

			//Relate the value to this cost			
			RelationType hasValue = tmst.getRelationType("HAS_VALUE");
			
			if(hasValue==null){
				//Has to make new Relation type
				hasValue = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_VALUE", "Relation pointing to a value", 1.0);
			}
			
			try{
				new Value(tmst, costValue, costValue);
			}catch(NameAlreadyExistException e){
				//If the value already exist, no problem just relate to it.
			}
			
			new Relation(this, hasValue, tmst.getEntity(costValue));
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public Cost(Entity ent) throws EntityTypeNotMatchedException{
		super(ent);
	}
	
	/**
		Method for adding a reference to the Cost object.
		@param reference The refrerence which is being addded to the costw.
	 */
	public void setReplyReference(ReplyReference reference) throws AlreadyHasValueException{
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasReplyReference = tmst.getRelationType("HAS_REFERENCE");
		
		if(hasReplyReference == null){
			//Has to make new Relation type
			try{
				hasReplyReference = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_REFERENCE", "Relation pointing to a goal", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		Relation rels[] = getLocalRelations("HAS_REFERENCE");
		
		if(rels.length > 1) throw new AlreadyHasValueException("A cost entity may only have one reference, if any.");
		else new Relation(this, hasReplyReference, reference);
	}
	
	/**
		Method returning the reply reference related to this cost entity
		@return String the name of the reply reference
	 **/
	public ReplyReference getReplyReference() throws Exception{
		Relation rels[]=this.getLocalRelations("HAS_REFERENCE");
		
		if(rels.length == 0) return null;
		if(rels.length>1) throw new Exception("A cost entity may only have one reference, if any");
		
		return (ReplyReference)rels[0].getValue();
	}
	
	/**
		Returns the value of this cost entity as a float. If the cost entity has no value, return positive infinity
		@return float The float value of the cost.
	 */
	public float getFloatValue() throws AlreadyHasValueException{
		Relation rels[] = this.getLocalRelations("HAS_VALUE");

		//If the cost entity has no value, return positive infinity
		if(rels==null){
			return Float.POSITIVE_INFINITY;
		}
		
		//A cost object shall only have one value
		if(rels.length>1){
			throw new AlreadyHasValueException("An cost entity may only have one value. Something is very wrong");
		}
		
		return new Float(rels[0].getValue().getName()).floatValue();
	}

	/**
		Returns the value of this cost entity as a string. If the cost entity has no value, return positive infinity
		@return String The float value of the cost.
	 */
	public String getStringValue() throws AlreadyHasValueException{
		Relation rels[] = this.getLocalRelations("HAS_VALUE");
		
		//If the cost entity has no value, return positive infinity
		if(rels==null){
			return new Float(Float.POSITIVE_INFINITY).toString();
		}
		
		//A cost object shall only have one value
		if(rels.length>1){
			throw new AlreadyHasValueException("An cost entity may only have one value. Something is very wrong");
		}

		return rels[0].getValue().getName();
	}
	
	/**
		Method returning the owner related to this cost.
		@return Entity The owner of this cost entity
	 */
	public Executor getExecutor() throws AlreadyHasValueException{
		Entity ent = null;
		Executor exec = null;
		Relation rels[] = this.getLocalRelations("HAS_COST");
		
		//If the cost entity has no executor, return null
		if(rels==null){
			return exec;
		}
		
		//A cost object shall only have one executor
		if(rels.length>1){
			//debugging:kari
			//for (int i =0; i<rels.length; i++){
			//	System.out.println("#### DEBUGGING ####: "+rels[i].getValue().getName()+" cost: "+this.getName());
			//}
		
			throw new AlreadyHasValueException("An cost entity may only have one executor. Something is very wrong");
		}
		
		//Try getting the owner of the cost entity
		ent = rels[0].getValue(); 
		try{
			exec = new Executor(ent);
		}catch(EntityTypeNotMatchedException e){
			e.printStackTrace();
		}
		
		//Return the owner of this cost entity.
		return exec;
	}


	//Method overridden from EntityType
	public static boolean matches(Entity value){
		TMST tmst = (TMST)value.getKnowledgeModel();
		Entity ent = tmst.getEntity("COST");
		
		if (ent == null) return false; 
		
		Collection subclasses = ent.getAllSubclassesCollection();
		
		if(value.getName().equals("COST")){
			return false;
		}else if(subclasses.contains(value)){
			return true;
		}
		return false;
	}
}
