package TMST.representation;
//
//  Input.java
//  TMST
//
//  Created by Odd Erik  Gundersen on Tue Feb 17 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

//Java
import java.util.Collection;
import java.lang.Exception;
//jCreek
import jcreek.representation.Entity;
import jcreek.representation.EntityType;
import jcreek.representation.Relation;
import jcreek.representation.RelationType;
//import jcreek.representation.IsoPodModel;
//import jcreek.representation.cbr.IsoPodModel;
import jcreek.representation.BasicModel;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.NameAlreadyExistException;
import jcreek.representation.EntityTypeNotMatchedException;

//TMST
import TMST.Exceptions.AlreadyHasValueException;

/**
	Class for representing a input of a task or a problem solving method.
*/
public class Input extends EntityType{

	/**
		Constructor setting up the relation to a generic input entity all inputs are related to 
		(makes explisit that the current entity is an input) and relates to the value of the input.
		Takes value as a string.
		@param tmst The knowledge model which the input belongs to
		@param name The name of the input 
		@param description The description of the input
		@param value The value of the input as a string
	*/
	public Input(TMST tmst, String name, String description, String value) throws NameAlreadyExistException{		
		super((KnowledgeModel)tmst, name, description);
		System.out.println("Inni konstruktor til input");
		
		try{
			//The model need a Input-object 
			if(!tmst.hasEntity("INPUT")){
				Entity in = new Entity((KnowledgeModel)tmst, "INPUT", "INPUT");
				in.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
			}
			this.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity("INPUT"));
		}catch(Exception e){
			e.printStackTrace();
		}

		RelationType hasValue = tmst.getRelationType("HAS_VALUE");
		
		if(hasValue==null){
			//Has to make new Relation type
			hasValue = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_VALUE", "Relation pointing to a value", 1.0);
			//kari sin:
			//hasValue = new RelationType((KnowledgeModel)tmst, tmst.getRelationType("STRUCTURAL_RELATION"), "HAS_VALUE", "Relation pointing to a value", 1.0);
		}
		
		try{
			new Value(tmst, value, value);
		}catch(NameAlreadyExistException e){
			System.out.println("GOAL: " +e.getMessage());
		}
		
		new Relation(this, hasValue, tmst.getEntity(value));		
	}
	
	
	public Input(Entity ent) throws EntityTypeNotMatchedException{
		super(ent);
	}

	/**
	Constructor setting up the relation to a generic input entity all inputs are related to 
	 (makes explisit that the current entity is an input) and relates to the value of the input.
	 Takes value as a value object.
	 @param tmst The knowledge model which the input belongs to
	 @param name The name of the input 
	 @param description The description of the input
	 @param value The value of the input as a value object.
	 */
	public Input(TMST tmst, String name, String description, Value value) throws NameAlreadyExistException{		
		super((KnowledgeModel)tmst, name, description);
		
		try{
			//The model need a Input-object 
			if(!tmst.hasEntity("INPUT")){
				Entity in = new Entity((KnowledgeModel)tmst, "INPUT", "INPUT");
				in.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
			}
			this.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity("INPUT"));
		}catch(Exception e){
			e.printStackTrace();
		}
		
		RelationType hasValue = tmst.getRelationType("HAS_VALUE");
		
		if(hasValue==null){
			System.out.println("INPUT: HAS_VALUE==NULL");
			//Has to make new Relation type
			hasValue = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_VALUE", "Relation pointing to a value", 1.0);
		}
		new Relation(this, hasValue, value);		
	}
	
	
	
	/**
		Method returning the value of the input
		@return Value The value of the input.
	*/
	public Value getValue() throws AlreadyHasValueException{
		
		Value value = null;
		Relation rels[] = this.getLocalRelations("HAS_VALUE");

		if(rels==null){
			return value;
		}
		if(rels.length>1){
			throw new AlreadyHasValueException("An output may only have one value. Something is terrible wrong");
		}
		
		try{
			value = new Value(rels[0].getValue());
		}catch(EntityTypeNotMatchedException e){
			System.out.println("Something is terrible wrong. An output should have a value!");
			e.printStackTrace();
		}
		return value;		
	}
	
	/**
		Method which has to be overriden (from the class EntityType)
	*/
	public static boolean matches(Entity value){
		TMST tmst = (TMST)value.getKnowledgeModel();
		Entity ent = tmst.getEntity("INPUT");
		
		if (ent == null) return false; 
		
		Collection subclasses = ent.getAllSubclassesCollection();

		if(value.getName().equals("INPUT")){
			return false;
		}else if(subclasses.contains(value)){
			return true;
		}
		return false;
	}
}