package TMST.representation;
//
//  Output.java
//  TMST
//
//  Created by Odd Erik  Gundersen on Tue Feb 17 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

//Java
import java.util.Collection;
import java.lang.Exception;

//jCreek
//import jcreek.representation.cbr.IsoPodModel;
import jcreek.representation.BasicModel;
import jcreek.representation.EntityType;
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.RelationType;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.NameAlreadyExistException;
import jcreek.representation.EntityTypeNotMatchedException;

//TMST
import TMST.Exceptions.AlreadyHasValueException;


public class Output extends EntityType{

	/**
		Constructor sets up an output entity by relating it to a generic output entity, and then relating the 
		instace of an ouput (with a given name) to its value. Here represented using a string.
		@param tmst The knowledge model the input belongs to
		@param name The name of the model
		@param description The description of the input
		@param value The value of the input represented using a string
	*/
	public Output(TMST tmst, String name, String description, String value) throws NameAlreadyExistException{
		super((KnowledgeModel)tmst, name, description);
		System.out.println("Inni konstruktor til output");
		
		//The model need a generic Output-object 
		try{
			if(!tmst.hasEntity("OUTPUT")){
				Entity out = new Entity((KnowledgeModel)tmst, "OUTPUT", "OUTPUT");
				out.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
			}
			this.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity("OUTPUT"));
			RelationType hasValue = tmst.getRelationType("HAS_VALUE");
			
			if(hasValue==null){
				//Has to make new Relation type
				hasValue = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_VALUE", "Relation pointing to a value", 1.0);
			}

			try{
				new Value(tmst, value, value);
			}catch(NameAlreadyExistException e){
				System.out.println("OUTPUT: " +e.getMessage());
			}
			new Relation(this, hasValue, tmst.getEntity(value));		
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public Output(Entity ent) throws EntityTypeNotMatchedException{
		super(ent);
	}

	/**
		Constructor sets up an output entity by relating it to a generic output entity, and then relating the 
		instace of an ouput (with a given name) to its value. Here represented using a value object.
		@param tmst The knowledge model the input belongs to
		@param name The name of the model
		@param description The description of the input
		@param value The value of the input represented using a value object
	*/
	public Output(TMST tmst, String name, String description, Value value) throws NameAlreadyExistException{
		super((KnowledgeModel)tmst, name, description);
		
		//The model need a Output-object 
		try{
			if(!tmst.hasEntity("OUTPUT")){
				Entity out = new Entity((KnowledgeModel)tmst, "OUTPUT", "OUTPUT");
				out.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
			}
			this.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity("OUTPUT"));
			
			//Relate the value to this output
			RelationType hasValue = tmst.getRelationType("HAS_VALUE");
			
			if(hasValue==null){
				//Has to make new Relation type
				hasValue = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_VALUE", "Relation pointing to a value", 1.0);
			}
			new Relation(this, hasValue, value);
			
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/**
		Method returning the output value
		@return Value The value of the Output object
	*/
	
	public Value getValue() throws AlreadyHasValueException{
		Relation rels[] = this.getLocalRelations("HAS_VALUE");
		Value value = null;
		
		if(rels==null){
			return null;
		}
		if(rels.length>1){
			throw new AlreadyHasValueException("An output may only have one value. Something is terrible wrong");
		}
		
		try{
			value = new Value(rels[0].getValue());
		}catch(EntityTypeNotMatchedException e){
			System.out.println("Something is terrible wrong. An output should have a value!");
			e.printStackTrace();
		}
		return value;
	}
	
	/**
		Method which has to be overriden (from the class EntityType)
		@param value The entity to match againts
	*/
	public static boolean matches(Entity value){	
		TMST tmst = (TMST)value.getKnowledgeModel();
		Entity ent = tmst.getEntity("OUTPUT");
		
		if (ent == null) return false; 
		
		Collection subclasses = ent.getAllSubclassesCollection();

		if(value.getName().equals("OUTPUT")){
			return false;
		}else if(subclasses.contains(value)){
			return true;
		}
		return false;
	}
}
