package TMST.representation;
//
//  ReplyReference.java
//  TMST
//
//  Created by Odd Erik  Gundersen on Sat Jun 12 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

//Java
import java.util.Collection;
import java.lang.Exception;

//jCreek
//import jcreek.representation.cbr.IsoPodModel;
import jcreek.representation.BasicModel;
import jcreek.representation.EntityType;
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.RelationType;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.NameAlreadyExistException;
import jcreek.representation.EntityTypeNotMatchedException;
import jcreek.representation.NoSuchRelationTypeException;

//TMST
import TMST.Exceptions.AlreadyHasValueException;

public class ReplyReference extends EntityType{

	public ReplyReference(TMST tmst, String name, String description, String value) throws NameAlreadyExistException {
		super((KnowledgeModel)tmst, name, description);
		try{
			//The model need a Value-object 
			if(!tmst.hasEntity("REPLY_REFERENCE")){
				Entity ref = new Entity((KnowledgeModel)tmst, "REPLY_REFERENCE", "REPLY_REFERENCE");
				ref.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
			}
			this.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity("REPLY_REFERENCE"));
			
			//Check whether the relation  type exist
			RelationType hasValue = tmst.getRelationType("HAS_VALUE");
			
			//If it does not exist, it has to be made
			if(hasValue==null){
				//Has to make new Relation type
				hasValue = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_VALUE", "Relation pointing to a value", 1.0);
			}
			
			//Make a new value entity out of the value given
			try{
				new Value(tmst, value, value);
			}catch(NameAlreadyExistException e){
				System.out.println("OUTPUT: " +e.getMessage());
			}
			
			//Relate to the value
			new Relation(this, hasValue, tmst.getEntity(value));
			
		}catch(NoSuchRelationTypeException e){
			e.printStackTrace();
		}
	}
	
	/**
		Method returning the value of the reference
		@return The value assigned to the reference
	 */
	public Value getValue() throws AlreadyHasValueException{
		Value value = null;
		
		Relation rels[] = this.getLocalRelations("HAS_VALUE");
		if(rels==null){
			return value;
		}
		if(rels.length>1){
			throw new AlreadyHasValueException("An output may only have one value. Something is very wrong");
		}
		try{
			value = new Value(rels[0].getValue());
		}catch(EntityTypeNotMatchedException e){
			e.printStackTrace();
		}
		return value;		
	}
	
	/**
		Method which has to be overriden (from the class EntityType)
	 */
	public static boolean matches(Entity value){
		TMST tmst = (TMST)value.getKnowledgeModel();
		Entity ent = tmst.getEntity("REPLY_REFERENCE");
		
		if (ent == null) return false; 
		
		Collection subclasses = ent.getAllSubclassesCollection();
		
		if(value.getName().equals("REPLY_REFERENCE")){
			return false;
		}else if(subclasses.contains(value)){
			return true;
		}
		return false;
	}
}