package TMST.representation;

//
//  TMST.java
//  TMST
//
//  Created by Odd Erik  Gundersen on Mon May 03 2004.
//  Copyright (c) 2004 NTNU. All rights reserved.
//
//  Changed by Kari 11.02.06 -	removed the tagging of the solutions entities from 
//								generateSolutionRecursively to tagSolutionEntities
//	Changed by Kari 13.02.06 -  the addSolutions method used to generate the solution space
//								is not correct. only available solutions according to proposing agents
//								should be marked as part of the solution space. added new method isSpacePart
//  Changed by Kari 13.05.06 -  added a teamID to the TMST. each TMST that is made gets it's own teamID, that
//								later is used as an identifier for the Problem Solver agents that joins in solving
//								the problem represented by a TMST.
//  Changed by Kari 13.05.06 -  added an intial input to the TMST. each TMST may solve general problems, to make them
//								specific to a certain problem one has to apply some problem - specific information
//  Changed by Kari 14.05.06 -  removed debugging code - a lot of system.out.println, and made pretty-print

//Java
import java.util.ArrayList;   
import java.util.Arrays;
import java.util.Iterator;

//jCreek
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.RelationType;
import jcreek.representation.LocalKnowledgeModel;
import jcreek.representation.KnowledgeModel;
//import jcreek.representation.cbr.IsoPodModel;
import jcreek.representation.BasicModel;
import jcreek.representation.NoSuchRelationException;
import jcreek.representation.NoSuchRelationTypeException;
import jcreek.representation.EntityTypeNotMatchedException;
import jcreek.representation.NameAlreadyExistException;

//TMST
import TMST.Exceptions.AlreadyHasValueException;


import CoPS.FileWriter;

public class TMST extends LocalKnowledgeModel{


	private String initialInput = "";
	private int teamID;
	private FileWriter fw = null;
	
	public TMST(){
		super();
	}
	
	public TMST(String file) throws java.io.IOException, java.lang.ClassNotFoundException{
		super(file);
	}
	
	/**
	 * Method setting the filewriter, by giving the name of a file
	 * New method added by Kari Roessland.
	 * @param file The name of the file this class should write to
	 */
	public void setFileWriter(FileWriter fileWriter){
		fw = fileWriter;
	}
	
	/**
	 * Method returning the filewriter of this TMST.
	 * New method added by Kari Roessland.
	 * @return The filewriter
	 */
	public FileWriter getFileWriter(){
		return fw;
	}
	

	/**
	 * Method returning the teamID of this TMST
	 * New method added by Kari Roessland.
	 * @return The teamID
	 */
	public int getTeamID(){
		return teamID;
	}
	
	/**
	 * Method setting the teamID of this TMST
	 * New method added by Kari Roessland.
	 * @param id - the teamID
	 */
	public void setTeamID(int id){
		teamID = id;
	}
	
	
	/**
	 * Method setting the initial input of the problem solving.
	 * New method added by Kari Roessland.
	 * @param input The initial input
	 */
	public void setInitialInput(String input){
		initialInput = input;
	}
	
	/**
	 * Method getting the initial input of the problem solving.
	 * New method added by Kari Roessland.
	 * @return The initial input
	 */
	public String getInitialInput(){
		return initialInput;
	}
	
	
	
	/**
		Method returning the specified action, or null if it does not exist.
	*/
	public Action getAction(String actionname){		
		Entity action = this.getEntity(actionname);
		
		try{
			return new Action(action);
		}catch(EntityTypeNotMatchedException e){
			e.printStackTrace();
			return null;
		}
	}
	
	
	/**
		Method for getting all actions with a specified type
	 */
	public Action[] getActions(String actionType){
		ArrayList actions = new ArrayList();
		ActionType at;

		try{
			at = new ActionType(this.getEntity(actionType));			
		}catch(EntityTypeNotMatchedException e){
			e.printStackTrace();
			return null;
		}
		
		Relation rels[] = at.getLocalRelations("HAS_ACTION_TYPE");
		
		for(int i = 0; i < rels.length; i++){
			try{
				actions.add(new Action(rels[i].getValue()));
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		return (Action[]) actions.toArray(new Action[actions.size()]);
	}
	
	/**
		Method returning the specified task, or null if it does not exist.
		@param taskname 
		@return Task The task specified.
	 */
	public Task getTask(String taskname){		
		Entity task = this.getEntity(taskname);
		
		try{
			return new Task(task);
		}catch(EntityTypeNotMatchedException e){
			e.printStackTrace();
			return null;
		}
	}
	

	/**
		Method returning all the problem solving methods part of a TMST.
		@return ArrayList The name of all PSMs in the TMST stored in an ArrayList as Strings.
	 */
	public ArrayList getAllProblemSolvingMethods(){	
		//The name of the psms are stored in an ArrayList which is returned by the method.
		ArrayList psms	= new ArrayList();
		Entity allEntities[] = this.getEntities();
		
		for(int i = 0; i < allEntities.length; i++){
			//If the current entity is a PSM add its name to the psms ArrayList
			if(ProblemSolvingMethod.matches(allEntities[i])){
				psms.add(allEntities[i].getName());
			}
		}
		
		return psms;
	}
	
	/**
		Method returning the names of all actions part of the TMST as Strings.
		@return ArrayList The name of the actions stored in an ArrayList as Strings.
	*/
	public Action[] getAllActions(){	
		//The name of the psms are stored in an ArrayList which is returned by the method.
		ArrayList actions	= new ArrayList();
		Entity allEntities[] = this.getEntities();
		
		for(int i = 0; i < allEntities.length; i++){
			//If the current entity is a action add its name to the psms ArrayList
			if(Action.matches(allEntities[i])){
				try{
					actions.add(new Action(allEntities[i]));
				}catch(EntityTypeNotMatchedException e){
					e.printStackTrace();
					continue;
				}
			}
		}
		
		if(actions.size()==0) return null;
		
		return (Action[])actions.toArray(new Action[actions.size()]);
	}
	
	
	
	
	
	/**
		Method returning all tasks part of the TMST.
		@return ArrayList An ArrayList containing the name of all tasks in the TMST.
	 */
	public ArrayList getAllTasks(){	
		//The name of the psms are stored in an ArrayList which is returned by the method.
		ArrayList tasks	= new ArrayList();
		Entity allEntities[] = this.getEntities();
		
		for(int i = 0; i < allEntities.length; i++){
			//If the current entity is a PSM add its name to the psms ArrayList
			if(Task.matches(allEntities[i])){
				tasks.add(allEntities[i].getName());
			}
		}
		
		return tasks;
	}

	/**
		Method returning all executors part of the TMST.
		@return ArrayList An Array of all the executors part of the TMST.
	*/
	public Executor[] getAllExecutors(){
		//Initialization
		Executor executors[] = null;
		ArrayList exeAL = new ArrayList();
		Entity allEntities[] = this.getEntities();
		
		//Iterate through al entities and add those whom match the Executor class
		for(int i = 0; i < allEntities.length; i++){
			if(Executor.matches(allEntities[i])){
				try{
					exeAL.add(new Executor(allEntities[i]));
				}catch(EntityTypeNotMatchedException e){
					e.printStackTrace();
				}
			}
		}
		
		//Return as array and not ArrayList
		executors = (Executor[])exeAL.toArray(new Executor[exeAL.size()]);
		
		return executors;
	}
	
	/**
		Method returning the specified problem solving method, or null if it does not exist.
	 */
	public ProblemSolvingMethod getProblemSolvingMethod(String psmname){		
		Entity psm = this.getEntity(psmname);
		
		try{
			return new ProblemSolvingMethod(psm);
		}catch(EntityTypeNotMatchedException e){
			e.printStackTrace();
			return null;
		}
	}
	
	/**
		Method returning the initial problem of this task structure tree, or null if it does not exist.
		@return Task The task which has been specified as the initial problem of the task structure tree.
	 */
	public Task getInitialProblem(){
		Task initialProblem = null;

		try{
			initialProblem = new Task((this.getEntity("INITIAL_PROBLEM").getRelation("IS_INITIAL_PROBLEM").getValue()));
		}catch(NoSuchRelationException e){
			//If there exist no such relation, the initial problem do not exist. Thus, return null.
			e.printStackTrace();
		}catch(NoSuchRelationTypeException e){
			//If there exist no such relation type, the initial problem do not exist. Thus, return null.
			e.printStackTrace();			
		}catch(EntityTypeNotMatchedException e){
			//If there exist no such entity type, the initial problem do not exist. Thus, return null.
			e.printStackTrace();			
		}
	
		return initialProblem;
	}
	
	
	/**
	 * Method printing enitities part of the solution and the cost related to them.
	 * Modified by Kari Roessland.
	 */
	public void printSolutionAndCost(){
		fw.write("\n    The nodes of the TMST part of the solution and their costs:");
		
		Entity solutionPart = this.getEntity("SOLUTION_PART");
		
		if(solutionPart == null){
			return;
		}
		
		Relation rels[] = solutionPart.getLocalRelations("IS_SOLUTION_PART");
		
		if(rels == null){
			fw.write("    NO ENTITIES ARE PART OF THE SOLUTION!!!");
			return;
		}
		
		for(int i = 0; i <rels.length; i++){
			Entity ent = rels[i].getValue();
			
			try{
				if(Task.matches(ent)){
					Task task = new Task(ent);
					fw.write("    Task: " +task.getName() +"  Cost: " +task.getCostValue());
				}else if(ProblemSolvingMethod.matches(ent)){
					ProblemSolvingMethod psm = new ProblemSolvingMethod(ent);
					fw.write("    PSM: " +psm.getName() +" Cost: " +psm.getCostValue());
				}else if(Action.matches(ent)){
					Action action = new Action(ent);
					fw.write("    Action: " +action.getName() +" Cost: " +action.getExecutionCostValue());
				}else{
					//do nothing
				}
			}catch(EntityTypeNotMatchedException e){
				e.printStackTrace();
			}catch(AlreadyHasValueException e){
				e.printStackTrace();
			}
		}
	}
	
	/**
		Method for setting a task as the initial problem to solve - the root node in the task-method tree structure.
		Every TMST structure may only have one Initial Problem, so setting two or more tasks as inital problems
		throws exception.
	 */
	public void setAsInitialProblem(Task initialProblem) throws Exception{
		//TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType initialProblemType = this.getRelationType("IS_INITIAL_PROBLEM");
		
		if(initialProblemType!=null){
			throw new Exception("The model already has already defined an inital problem.\n Only one may exist at any time.");
		}
		
		initialProblemType = new RelationType((KnowledgeModel)this, this.getRelationType(BasicModel.HAS_VALUE_TYPE), "IS_INITIAL_PROBLEM", "Points to a root entity", 1.0);
		
		try{
			//The model needs an entity representing initial problem, which this task shall have a relation to.
			if(!this.hasEntity("INITIAL_PROBLEM")){
				Entity task = new Entity((KnowledgeModel)this, "INITIAL_PROBLEM", "INITIAL_PROBLEM");
				task.addRelation(BasicModel.INSTANCE_OF, this.getEntity(BasicModel.SYMBOL));
			}
			
			//A relationship between the intial problem and this instance of task is made to specify the fact that
			//_this_ task represents the initial problem this TMST structure is supposed to solve.
			//==> Denne funker ikke, hvorfor? this.addRelation(initialProblemType, tmst.getEntity("INITIAL_PROBLEM"));
			new Relation(initialProblem, initialProblemType, this.getEntity("INITIAL_PROBLEM"));
		}catch(Exception e){
			e.printStackTrace();
		}
	}

	/**
		Method for extracting input/output dependencies from all PSMs in the TMST.
	*/
	public void dependencyAnalyzation(){
		//ProblemSolvingMethod psms[] = this.getAllProblemSolvingMethods(tmst);
		ArrayList psms = this.getAllProblemSolvingMethods();
		
		//Iterate through 
		for(int i = 0; i < psms.size(); i++){
			System.out.println("\n\n#### CURRENT PROBLEM SOLVING METHOD: " +psms.get(i) +" ####\n\n");
			//this.analyzeIODependencies((ProblemSolvingMethod)this.getEntity(psms.get(i)));
			try {
				this.analyzeIODependencies(new ProblemSolvingMethod(this.getEntity(psms.get(i))));
			}catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
		Method for analyzing the io dependencies of a problem solving method. All dependencies are stored 
		in the control information entitywhich the psm relate to.
		@param psm The psm to analyze for io dependencies.
	 */
	private void analyzeIODependencies(ProblemSolvingMethod psm){
		Task subtasks[] = null;

		try{
			subtasks = psm.getSubtasks();
		}catch(Exception e){
			e.printStackTrace();
		}
		
		//ArrayList containing ArrayLists. 
		//Each arrayList in it contains a subtask, the subtasks inputs, the subtasks outputs
		ArrayList allTasks = new ArrayList();
		
		for(int i = 0; i < subtasks.length; i++){
			ArrayList taskInfo = new ArrayList();
			taskInfo.add(subtasks[i]);
			taskInfo.add(subtasks[i].getInputs());
			taskInfo.add(subtasks[i].getOutputs());
			allTasks.add(taskInfo);
		}
		
		//Iterate through all tasks checking for dependencies by checking 
		for(int i = 0; i <allTasks.size(); i++){
			//Gather info
			ArrayList taskInfo = (ArrayList)allTasks.get(i);
			Task currentTask = (Task)(taskInfo.get(0));
			Input ins[] = (Input[])taskInfo.get(1);

			System.out.println("Current TASK: " +currentTask.getName());
			
			for(int j = 0; j < allTasks.size(); j++){
				//Do not need to check a task against itself 
				if(j==i){
					break;
				}
				
				ArrayList checkingAgainstInfo = (ArrayList)allTasks.get(j);
				Task checkingAgainstTask = ((Task)checkingAgainstInfo.get(0));
				Output outs[] = checkingAgainstTask.getOutputs();

				System.out.println("	|--Checking against TASK: " +checkingAgainstTask.getName());
				
				for(int k = 0; k< ins.length; k++){
					Input currentInput = ins[k];
					Value inputValue  = null;

					try{
						inputValue = currentInput.getValue();
					}catch(Exception e){
						e.printStackTrace();
					}
					
					System.out.println("		|--Input value checking against " +inputValue.getName());
					
					for(int l = 0; l < outs.length; l++){
						Output currentOutput = outs[l];
						Value outputValue = null;
	
						try{
							outputValue = currentOutput.getValue();
						}catch(Exception e){
							e.printStackTrace();
						}
						
						System.out.println("			|--Output value: " +outputValue.getName());						

						//If the name of the input which the outputs are checked againts equals the name of the current output, there is a dependency
						if(outputValue.getName().equals(inputValue.getName())){
							String depString = "<" +currentInput.getName() +">(" +currentTask.getName() +") :- <" +currentOutput.getName() +">(" +checkingAgainstTask.getName() +")";
							System.out.println("				|--Output value equals Input value: DEPENDECY FOUND!");
							System.out.println("					|-- DEPENDENCY: " +depString);
							psm.setControlInformation(depString);

						}
					}
				}
			}
		}
	}
	
	/**
	 * Method for generating the solution space of a tmst. It does not only return the solution space, 
	 * but store it as a private variable in the tmst as well. Modified by Kari Roessland.
	 * @return true if there are a solution and false if not.
	 */
	public boolean generateSolutionSpace() throws NameAlreadyExistException{
		//Activate all actions, tasks and psms which are solvable
		this.activate();
		
		//Debugging code
//		this.analyzeAndPrint();
		
		//There are only solutions in the solution space if the intial task has been activated
		if(!this.getInitialProblem().isActivated()) return false; 
			
		//Start the recursive method addSolutions
		//debug
		fw.write("    The nodes of the TMST part of the solution space, and the nodes that are not activated are:");
		this.addSolutions((Entity)this.getInitialProblem());
		fw.write("\n");
		//Return true when finished
		return true;
	}
	
	/**
	 *	Private method used by the method generate solution space to find all the possible solutions of the current (activated) tmst. 
	 *  This is done by traversing the three in a top-down approach. Modified by Kari Roessland
	 *	@param ent the entity one should try to add to the solution space.
	 */
	private void addSolutions(Entity ent){
		//Initialization
		Entity next[] = null;
		
	
		try{
			if(ProblemSolvingMethod.matches(ent)){
				ProblemSolvingMethod psm = new ProblemSolvingMethod(ent);
				//debug
				
				if(psm.isFulFilled()){
					next = (Entity[])psm.getSubtasks();
					//set as SPACE_PART
					isSpacePart(ent);
					fw.write("    PSM: "+psm.getName());
				}else{
					fw.write("    PSM: "+psm.getName()+ " is NOT fulfilled!!!");
					return;
				}
			}else if(Task.matches(ent)){
				//set as SPACE_PART
				isSpacePart(ent);
				Task task = new Task(ent);
				//debug
				fw.write("    Task: "+task.getName());
				Entity nextPSMs[] = (Entity[])task.getProblemSolvingMethods();
				Entity nextActions[] = (Entity[])task.getActions();
				
				ArrayList temp = new ArrayList(Arrays.asList(nextPSMs));
				temp.addAll(Arrays.asList(nextActions));
				
				next = (Entity[])temp.toArray(new Entity[nextPSMs.length + nextActions.length]);
			}else if(Action.matches(ent)){
				Action action = new Action(ent);
				
				if(action.isActivated()){
					//set as SPACE_PART
					isSpacePart(ent);
					//debug
					fw.write("    Action: "+ent.getName());
					return;
				}else{
					fw.write("    Action: "+ent.getName()+ " is NOT activated!!!");
					return;
				}
			}else{
				throw new Exception("TMST.addSolutions: This should never happen. Ever!");
			}
		}catch(EntityTypeNotMatchedException e){
			e.printStackTrace();
		}catch(Exception e){
			e.getMessage();
			e.printStackTrace();
		}

		//Continue the recursion
		for(int i = 0; i < next.length; i++){
			this.addSolutions(next[i]);
		}
	}
	
	
	/**
	 * Method for tagging an entity part of the solution space. New method added by Kari Roessland.
	 * @param ent - the entity to be tagged
	 */
	public boolean isSpacePart(Entity ent){
		try{
			if(!this.hasEntity("SPACE_PART")){
				Entity sd = new Entity(this, "SPACE_PART", "SPACE_PART");
				sd.addRelation(BasicModel.INSTANCE_OF, this.getEntity(BasicModel.SYMBOL));
			}
		
			if(!this.hasEntity("IS_SPACE_PART")){
				Entity sd = new Entity(this, "IS_SPACE_PART", "IS_SPACE_PART");
				sd.addRelation(BasicModel.INSTANCE_OF, this.getEntity(BasicModel.SYMBOL));
			}
		}catch(NameAlreadyExistException e){
			e.printStackTrace();
			return false;
		}catch(NoSuchRelationTypeException e){
			e.printStackTrace();
			return false;	
		}
		
		//Add a relation between the current entity and the the SPACE_PART tagging all entities part of the solution space.
		new Relation(ent, this.getRelationType("IS_SPACE_PART"), this.getEntity("SPACE_PART"));
	
		return true;
	}
	
	
	
	
	/**
	 *	Method for generating the solution of a tmst. It does not only return the solution but store it as a private variable as well.
	 *	Modified by Kari Roessland
	 *	@return The status of the solution generation.
	 */	
	public boolean generateSolution() throws NameAlreadyExistException{
		//Totoal cost
		float totalCost = 0.0f;

		
		//If the initial problem is not part of the solution space, there are no solutions
		if(!this.getInitialProblem().isSpacePart()){
			fw.write("\n    The initial-Task (problem) is not activated, therefore it does NOT have a solution");
			return false;
		}
		fw.write("\n    The initial-Task (problem) is activated, therefore it has a solution");

		//Compute the cost while generating the solution
		totalCost = this.generateSolutionRecursively((Entity)this.getInitialProblem());
		
		fw.write("    The solution of the initial-Task (problem) has a total cost of:" +totalCost+"\n\n");
		
		//tag the solution entities
		
		this.tagSolutionEntities((Entity)this.getInitialProblem(), true);
		
			
		//Print ell entities part of the solution with cost values, for debugging purposes
		printSolutionAndCost();
		return true;
	}
	
	/**
	 * Method used by the generateSolutions method to tag the solution entities, by traversing the lowest cost path.
	 * New method added by Kari Roessland.
	 * @param ent The current entity to mark
	 * @param initialTask Whether the entity is the initialTask
	 */
	private void tagSolutionEntities(Entity ent, boolean initialTask){
	
	
		float cost = 0;
		Entity temp = null;
		Task task = null;
		ProblemSolvingMethod psm = null;
		Action action = null;
		
		
		
		
		try{
			//The current temp object is being given a solution part tag.
			if(!this.hasEntity("SOLUTION_PART")){
				Entity sd = new Entity(this, "SOLUTION_PART", "SOLUTION_PART");
				sd.addRelation(BasicModel.INSTANCE_OF, this.getEntity(BasicModel.SYMBOL));
			}
		
			//To give the solution part tag, a relation is needed
			if(!this.hasEntity("IS_SOLUTION_PART")){
				Entity sd = new Entity(this, "IS_SOLUTION_PART", "IS_SOLUTION_PART");
				sd.addRelation(BasicModel.INSTANCE_OF, this.getEntity(BasicModel.SYMBOL));
			}
		
			new Relation(ent, this.getRelationType("IS_SOLUTION_PART"), this.getEntity("SOLUTION_PART"));
		
			if (Task.matches(ent)){
				task = new Task(ent);
				cost = task.getCostValue();
				
				ProblemSolvingMethod psms[] = task.getProblemSolvingMethods();
				Action actions[] = task.getActions();
				
				
				
				for(int i = 0; i < psms.length; i++ ){	
					//check if psm is part of the solution space
				
					if (psms[i].isSpacePart() && psms[i].getCostValue() == cost){
						temp = psms[i];
					}
				}
				
				for(int i =0; i< actions.length; i++){
					if(actions[i].getExecutionCostValue() == cost){
						temp = actions[i];
					}
				}
				
				tagSolutionEntities(temp, false);

			}
			
			if (ProblemSolvingMethod.matches(ent)){
				psm = new ProblemSolvingMethod(ent);
				Task tasks[] = psm.getSubtasks();
				
				for(int i = 0; i<tasks.length; i++){
					tagSolutionEntities(tasks[i], false);
				}
			
			}
			
		}catch(NoSuchRelationTypeException e){
			e.printStackTrace();
			return;
		}catch(NameAlreadyExistException e){
			e.printStackTrace();
			return;
		}catch(EntityTypeNotMatchedException e){
			e.printStackTrace();
			return;
		}catch(AlreadyHasValueException e){
			e.printStackTrace();
			return;		
		}

	
	}
	
	/**
	 *	Method used by the generateSolutions method to recursively generating a solution based on the entities with lowest cost related to it.
	 *	Modified by Kari Roessland.
	 *	@param ent the current entity to check analyse the cost of.
	 *	@return The cost of the current entity as a float number.
	 */
	private float generateSolutionRecursively(Entity ent){
		//The cost of the current entity
		float cost = Float.POSITIVE_INFINITY;
		Entity temp = null;
		
		try{
			if(Task.matches(ent)){
				Task task = new Task(ent);
				ProblemSolvingMethod psms[] = task.getProblemSolvingMethods();
				Action actions[] = task.getActions();
				
				//Loop Through PSMs
				for(int i = 0; i < psms.length; i++ ){
					float tempCost;

					//If the PSM is not in the solution space, it is certainly not a part of the solution, then it is just to continue to the next PSM
					if(!psms[i].isSpacePart()){
						continue;
					}
					
					//Continue the recursive cost generation with the PSMs solving the current task
					tempCost = generateSolutionRecursively((Entity)psms[i]);
					
					//Store the cost of and the psm with the lowest price
					if(tempCost <= cost){
						cost = tempCost;
						temp = (Entity)psms[i];
					}					
				}
			
	
				//Loop through actions
				for(int i = 0; i < actions.length; i++ ){
					float tempCost;
					//If the action is not in the solution space, it is certainly not a part of the solution, then it is just to continue to the next action
					if(!actions[i].isSpacePart()){
						continue;
					}
					
					//Continue the recursive cost generation with the actions solving the current task
					tempCost = generateSolutionRecursively((Entity)actions[i]);

					//Store the cost of and the action with the lowest price
					if(tempCost <= cost){
						cost = tempCost;
						temp = (Entity)actions[i];
					}
				}
				task.setCost(cost);
			}else if(ProblemSolvingMethod.matches(ent)){
				ProblemSolvingMethod psm = new ProblemSolvingMethod(ent);
				Task subtasks[] = psm.getSubtasks();
				float allCosts[] = new float[subtasks.length];

				for(int i = 0; i < subtasks.length; i++ ){
					float tempCost;
					
					//If the action is not in the solution space, it is certainly not a part of the solution, then it is just to continue to the next action
					if(!subtasks[i].isSpacePart()){
						continue;
					}
					
					//Continue the recursive cost generation with the subtasks solving the current PSM
					allCosts[i] =  generateSolutionRecursively((Entity)subtasks[i]);
				}
				
				//The cost of this psm is the sum of all its subtasks costs
				cost = addAll(allCosts);
				
				//Set the computed cost as the cost of the current task
				psm.setCost(cost);
			}else if(Action.matches(ent)){
				
				
				Action action = new Action(ent);
				cost = action.getExecutionCostValue();
				temp = ent;
			}else{
				//do nothing
			}
		
		}catch(EntityTypeNotMatchedException e){
			e.printStackTrace();
		}catch(AlreadyHasValueException e){
			e.printStackTrace();
		}

		return cost;
	}
	
	/**
		Method adding a list of floats and returning the sum
		@param list List of floats
		@return float The sum
	 */
	private float addAll(float list[]){
		float sum = 0.0f;
		
		for(int i = 0; i < list.length; i++){
			sum = sum + list[i];
		}
		return sum;
	}
	
	/**
		Method for activating the all entities in the tmst which are solvable. Starts with collecting all the executors, and activates all actions and all related entities which are solvable when an action is performed.
	 */
	public void activate(){
		//All the activities being taken care of by executors is being activated
		Executor executors[] = this.getAllExecutors();
		//Iterate through all executors part of this tmst
		for(int i = 0; i < executors.length; i++){
			Action  actions[] = executors[i].getActions();
			//Activate all actions and the entities connected to them which also are solvable
			for(int j = 0; j < actions.length; j++){
				this.activateFrom((Entity)actions[j]);
			}
		}
	}

	/**
		Method for activating a path between one action and the initial problem of a knowledge model.
		@param tmst The knowledge model which contains the action which path to the initial problem si to be activated
		@param action The action which path to the initial problem of the knowledge model is to be activated
		@return KnowlegeModel A knowledge model containing the the activated path.
	*/
	private void activateFrom(Entity entity){
		//Next enitity is the next entity for this recursive algorithm to chew
		Entity next = null;
		
		if(Task.matches(entity)){
			//Initialize a bit
			Task current = null;
			ProblemSolvingMethod psm = null;
			
			//Find the problem solving method pointing to this task.
			try{
				//"Cast" the entity to a task
				current = new Task(entity);
				
				//If the task is not activated, activate it
				if(!current.isActivated()){
					current.setAsActivated();
				}else return;
				
				//The initial problem is not pointed to by a psm, thus return
				if(current.isInitialProblem()) return;
				
				psm =new ProblemSolvingMethod(current.getLocalRelation("HAS_SOLUTION").getValue());
			}catch(NoSuchRelationException e){
				e.printStackTrace();
			}catch(NoSuchRelationTypeException e){
				e.printStackTrace();
			}catch(EntityTypeNotMatchedException e){
				e.printStackTrace();				
			}
			
			//Continue the recursion
			next = (Entity)psm;
		}else if(ProblemSolvingMethod.matches(entity)){
			//Initialization
			ProblemSolvingMethod current = null;
			Task task = null;

			try{
				//encapsulate the entity in an psm object, an find the task pointing to this problem solving method
				current = new ProblemSolvingMethod(entity);
				
				//For a psm to be activated, it must be full-filled - all subtask must be activated.
				if(!current.isActivated()){
					
					//PSM has to be fulfilled to continue the recursion
					if(current.isFulFilled()){
						current.setAsActivated();
					}else return;
				}else return;

				task = new Task(current.getLocalRelation("IS_SOLVED_BY").getValue());
			}catch(NoSuchRelationException e){
				e.printStackTrace();
			}catch(NoSuchRelationTypeException e){
				e.printStackTrace();
			}catch(EntityTypeNotMatchedException e){
				e.printStackTrace();				
			}

			//Contine the recusion
			next = (Entity)task;
			
		}else if(Action.matches(entity)){	
			//Initialize some usefull variables
			Action current = null;
			Task task = null;

			//Find the task pointing to this action
			try{
				current = new Action(entity);				
				task = new Task(current.getLocalRelation("EXECUTION_OF").getValue());
			}catch(NoSuchRelationException e){
				e.printStackTrace();
			}catch(NoSuchRelationTypeException e){
				e.printStackTrace();
			}catch(EntityTypeNotMatchedException e){
				e.printStackTrace();				
			}
			
			//Activate the current action if it's not already activated
			if(!current.isActivated()){
				current.setAsActivated();
				//activated.add(current);
			}else{
				return;
			}
			
			//Continue the recursion
			next = (Entity)task;
		}		
		//Continue the recursion
		activateFrom(next);
	}
	
	/**
		Method printing all entities part of the tmst to stdout.
	 */
	public void analyzeAndPrint(){
		System.out.println("\n#### ANALYZE AND PRINT ####\n");
		
		System.out.println("Name: " +this.getName());
		Iterator entities = this.getEntityIterator();
		int test = 0;
		
		while(entities.hasNext()){
			Entity ent = ((Entity)entities.next());
			System.out.println("Name of Entity #" +test +": " +ent.getName());
			
			if(ProblemSolvingMethod.matches(ent)){
				System.out.println("   |-- PROBLEM SOLVING METHOD");
			}else if(Task.matches(ent)){
				System.out.println("   |-- TASK");
			}else if(Action.matches(ent)){
				System.out.println("   |-- ACTION");
			}else if(ActionType.matches(ent)){
				System.out.println("   |-- ACTION TYPE");
			}else if(Assumption.matches(ent)){
				System.out.println("   |-- ASSUMPTION");
			}else if(Output.matches(ent)){
				System.out.println("   |-- OUTPUT");
			}else if(Input.matches(ent)){
				System.out.println("   |-- INPUT");		
			}else if(ReplyReference.matches(ent)){
				System.out.println("   |-- REPLY_REFERENCE");				
			}else if(Input.matches(ent)){
				System.out.println("   |-- COST");
			}else if(AssumptionType.matches(ent)){
				System.out.println("   |-- ASSUMPTION_TYPE");
			}else if(ControlInformation.matches(ent)){
				System.out.println("   |-- CONTROL_INFORMATION");
			}else if(Executor.matches(ent)){
				System.out.println("   |-- EXECUTOR");  
			}else if(Goal.matches(ent)){
				System.out.println("   |-- GOAL");
			}else if(Value.matches(ent)){
				System.out.println("   |-- VALUE"); 
			}else{
				System.out.println("   |-- " +ent.getDescription());	
			}
			test++;
		}	
	}	

}