package TMST.representation;

//
//  Task.java
//  TMST
//
//  Created by Odd Erik  Gundersen on Mon Feb 16 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//
// Changed by Kari 18.04.06 : getGoals caused a class-cast exception.

//Java
import java.util.Collection;
import java.util.ArrayList;

//jCreek
import jcreek.representation.Entity;
import jcreek.representation.EntityType;
//import jcreek.representation.cbr.IsoPodModel;
import jcreek.representation.BasicModel;
import jcreek.representation.Relation;
import jcreek.representation.RelationType;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.NameAlreadyExistException;
import jcreek.representation.NoSuchRelationTypeException;
import jcreek.representation.NoSuchRelationException;
import jcreek.representation.EntityTypeNotMatchedException;

//TMST
import TMST.Exceptions.AlreadyHasValueException;

/**
	Class describing tasks. Task describes what to do. Parameters a task may hold:
	<ul>
		<li>Goal</li>
		<li>Inputs</li>
		<li>Outputs</li>
		<li>PSMs</li>
		<li>Actions</li>
	</ul>
*/
public class Task extends EntityType{

	/**
		Constructor, calls the super class constructor only with knowledge model and name as parameters
		Parameters a task may takes:	 
		@param tmst The TMST this Task is a part of.
		@param name The name of the task.
		@param description The description of the task.

	*/
	public Task(TMST tmst, String name, String description) throws NameAlreadyExistException{
		super((KnowledgeModel)tmst, name, description);
		try{
			//The model needs a generic task 
			if(!tmst.hasEntity("TASK")){
				Entity task = new Entity((KnowledgeModel)tmst, "TASK", "TASK");
				task.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));
			}
		
			//A relationship between the generic task an this instance of task is made to specify the fact that
			//_this_ instance of the class task is a task.
			this.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity("TASK"));
		}catch(Exception e){
			e.printStackTrace();
		}
	}


	/**
		Cosntructor for encpsulating an entity as a psm (to be allowed to use psm methods on it)
		@param ent The entity to encapsulate.
	 */
	public Task(Entity ent) throws EntityTypeNotMatchedException {
		super(ent);
	}


	/**
		Method for adding a goal to the Task.
		@param goal The goal which is being addded to the task.
	*/
	public void setGoal(Goal goal){
		TMST tmst = (TMST)this.getKnowledgeModel();		
		RelationType hasGoal = tmst.getRelationType("HAS_GOAL");
		
		if(hasGoal==null){
			//Has to make new Relation type
			try{
				hasGoal = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_GOAL", "Relation pointing to a goal", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		new Relation(this, hasGoal, goal);
	}

	/**
		Method for adding a set of goals to the Task.
		@param goals The set of goals which is to be added to the task.
	*/
	public void setGoals(Goal goals[]){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasGoal = tmst.getRelationType("HAS_GOAL");
		
		if(hasGoal==null){
			//Has to make new Relation type
			try{
				hasGoal = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_GOAL", "Relation pointing to a goal", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		//for every goal in the array goals, make a relation between the goal and this task
		for(int i = 0; i<goals.length; i++){
			try{
				new Relation(this, hasGoal, goals[i]);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}

	/**
		Method returning all goals of the Task. Changed by Kari because it caused a class-cast exception
		@return Goal[] The goals belonging to the task.
		
	*/
	public Goal[] getGoals() throws Exception{
				
		//Hente ut alle HasGoal relasjoner som denne tasken peker til
		Relation rels[] = this.getLocalRelations("HAS_GOAL");
		
		//All tasks must have goals throw exception if this is not the case
		if(rels==null){
			throw new Exception("The task has no goals! Something is terribly wrong!\n");
		}
		
		//Declare and instatiate the array which is to be returned
		Goal retGoals[] = new Goal[rels.length];
	
		//Get all goals that the HasGoal relations point to
		for(int i=0; i<rels.length; i++){
			retGoals[i]= new Goal(rels[i].getValue());
		}
		return retGoals;
	}
		
	/**
		Method for setting the assumptions related to the task.
		@param ass The assumptions which apply for the task.
	 */ 
	public void setAssumption(Assumption ass){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasAss = tmst.getRelationType("HAS_ASSUMPTION");
		
		if(hasAss==null){
			//Has to make new Relation type
			try{
				hasAss = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_ASSUMPTION", "Relation pointing to an input", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}		
		new Relation(this, hasAss, ass);
	}
	
	/**
		Method for setting the assumptions related to the task.
		@param ass The assumptions which apply for the task.
	 */ 
	public void setAssumptions(Assumption[] ass){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasAss = tmst.getRelationType("HAS_ASSUMPTION");
		
		if(hasAss==null){
			//Has to make new Relation type
			try{
				hasAss = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_ASSUMPTION", "Relation pointing to an input", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		for(int i=0;i<ass.length;i++){
			new Relation(this, hasAss, ass[i]);
		}
	}
	
	/**
		Method for adding one problem solving method to the Task.
		@param psm The problem solving method to add to the task.
	*/
	public void setProblemSolvingMethod(ProblemSolvingMethod psm){
		TMST tmst = (TMST)this.getKnowledgeModel();

		try{
			new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_PROBLEM_SOLVING_METHOD", "Relation pointing to a problem solving method, usually from a task", 1.0, "IS_SOLVED_BY", "Negation of HAS_PROBLEM_SOLVING_METHOD", 1.0);
		}catch(Exception e){
			e.getMessage();
		}
		
//		if(tmst.getRelationType("HAS_PROBLEM_SOLVING_METHOD")==null) System.out.println("HELVETE ER LS");
		
		new Relation(this, tmst.getRelationType("HAS_PROBLEM_SOLVING_METHOD"), psm);
	}
	
	
	/**
		Method for adding a set of problem solving methods to the task.
		@param psms The set of problem solving methods to add to the task.
	*/
	public void setProblemSolvingMethods(ProblemSolvingMethod[] psms){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasPSM = tmst.getRelationType("HAS_PROBLEM_SOLVING_METHOD");
		
		if(hasPSM==null){
			//Has to make new Relation type
			try{
				hasPSM = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_PROBLEM_SOLVING_METHOD", "Relation pointing to a problem solving method, usually from a task.", 1.0, "IS_SOLVED_BY", "Negation of HAS_PROBLEM_SOLVING_METHOD", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		//for every goal in the array goals, make a relation between the goal and this task
		for(int i = 0; i<psms.length; i++){
			try{
				new Relation(this, hasPSM, psms[i]);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}
	
	/**
		Method which returns all of the problem solving methods related to this particular task
		@return ProblemSolvingMethod[] The problem solving methods.
	*/
	public ProblemSolvingMethod[] getProblemSolvingMethods(){
		//Initialization
		ProblemSolvingMethod psms[] = null;
		Relation rels[] = this.getLocalRelations("HAS_PROBLEM_SOLVING_METHOD");
	
		//Throw exception if there exist no problem solving methods
		if(rels==null){
			return psms;
		}	
		
		//Initialize the psm array
		psms = new ProblemSolvingMethod[rels.length];
	
		//Parse through the relations and get the psm they point to ans store them in the reurn value
		for(int i=0; i<rels.length; i++){
			try{
				psms[i] =  new ProblemSolvingMethod(rels[i].getValue());
			}catch(EntityTypeNotMatchedException e){
				e.printStackTrace();
			}
		}
		return psms;
	}
	
	/**
		Method for adding Input to the task
		@param in The input to add to the task
	*/
	public void setInput(Input in){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasInput = tmst.getRelationType("HAS_INPUT");
		
		if(hasInput==null){
			//Has to make new Relation type
			try{
				hasInput = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_INPUT", "Relation pointing to a input", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		new Relation(this, hasInput, in);
	}
	
	/**
		Method for adding a set of inputs to the task
		@param ins The set of inputs to add to the task
	*/
	public void setInputs(Input[] ins){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasInput = tmst.getRelationType("HAS_INPUT");
		
		if(hasInput==null){
			//Has to make new Relation type
			try{
				hasInput = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_INPUT", "Relation pointing to an input", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		//for every goal in the array goals, make a relation between the goal and this task
		for(int i = 0; i<ins.length; i++){
			try{
				new Relation(this, hasInput, ins[i]);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}
	
	/**
		Method for getting all input part the task
		@return Input[] All Input entities related to the task
	*/
	public Input[] getInputs(){
		//Get all relations pointing to problem solving methods
		Relation rels[] = this.getLocalRelations("HAS_INPUT");
	
		//Throw exception if there exist no problem solving methods
		if(rels==null){
			return null;
		}
		Input ins[] = new Input[rels.length];
	
		//Parse through the relations and get the psm they point to ans store them in the reurn value
		for(int i=0; i<rels.length; i++){
			ins[i] = (Input)rels[i].getValue();
		}
		
		//Return the array of input values related to this task
		return ins;
	}
	
	
	/**
	 * Method for getting all input-entities as strings
	 * @author Kari Roessland
	 * @return ArrayList A list of input-entities as strings
	 */
	public ArrayList getInputsString(){
		Relation rels[] = this.getRelations("HAS_INPUT");
		ArrayList retInput = new ArrayList();
		Input input = null;
		
		if (rels != null && rels.length > 0){
			for (int i = 0; i< rels.length; i++){
				try{
					input = new Input(rels[i].getValue());
					retInput.add(input.getName());
				}catch(EntityTypeNotMatchedException e){
					e.printStackTrace();
				}
			}
		}
		return retInput;
	}
	

	

	/**
		Method for adding output to the task
		@param out The output to add to the task
	*/
	public void setOutput(Output out){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasOutput = tmst.getRelationType("HAS_OUTPUT");
		
		if(hasOutput==null){
			//Has to make new Relation type
			try{
				hasOutput = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_OUTPUT", "Relation pointing to an output", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		new Relation(this, hasOutput, out);
	}
	
	/**
		Method for adding a set of outputs to the task
		@param outs The set of outputs to add to the task
	*/
	public void setOutputs(Output[] outs){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasOutput = tmst.getRelationType("HAS_OUTPUT");
		
		if(hasOutput==null){
			//Has to make new Relation type
			try{
				hasOutput = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_OUTPUT", "Relation pointing to an output", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		//for every goal in the array goals, make a relation between the goal and this task
		for(int i = 0; i<outs.length; i++){
			try{
				new Relation(this, hasOutput, outs[i]);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}
	
	/**
		Method for getting all output entities part the task
		@return Output[] All Output entities related to the task
	*/
	public Output[] getOutputs(){
		//Get all relations pointing to problem solving methods
		Relation rels[] = this.getLocalRelations("HAS_OUTPUT");
	
		//Throw exception if there exist no problem solving methods
		if(rels==null){
			return null;
		}
		Output outs[] = new Output[rels.length];
	
		//Parse through the relations and get the psm they point to ans store them in the reurn value
		for(int i=0; i<rels.length; i++){
			outs[i] = (Output)rels[i].getValue(); 
		}
		return outs;
	}
	
	
	/**
	 * Method for getting the output entity as a string - this method assume that there is only one output.
	 * @author Kari Roessland
	 * @return String The name of the output entity
	 */
	public String getOutputString(){
		Relation[] rels = this.getLocalRelations("HAS_OUTPUT");
		String retOutput = "";
		Output output = null;
		if (rels != null && rels.length == 1){
			try{
				output = new Output(rels[0].getValue());
				retOutput = output.getName();
			}catch(EntityTypeNotMatchedException e){
				e.printStackTrace();
			}
		}
		return retOutput;
	}
	
	
	

	/**
		Method for adding an action to the task
		@param action The action to add to the task
	*/
	public void setAction(Action action){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasAction = tmst.getRelationType("HAS_ACTION");
		
		if(hasAction==null){
			//Has to make new Relation type
			try{
				hasAction = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_ACTION", "Relation pointing to an action, usually from a task.", 1.0, "EXECUTION_OF", "Negation of the HAS_ACTION relation type.", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		new Relation(this, hasAction, action);
	}
	
	/**
		Method for adding a set of actions to the task
		@param actions The set of actions to add to the task
	*/
	public void setActions(Action[] actions){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType hasAction = tmst.getRelationType("HAS_ACTION");
		
		if(hasAction==null){
			//Has to make new Relation type
			try{
				hasAction = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_ACTION", "Relation pointing to an action, usually from a task.", 1.0, "EXECUTION_OF", "Negation of the HAS_ACTION relation type.", 1.0);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		//for every goal in the array goals, make a relation between the goal and this task
		for(int i = 0; i<actions.length; i++){
			try{
				new Relation(this, hasAction, actions[i]);
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}
	
	/**
		Method for getting all actions part the task
		@return Action[] All action entities related to the task
	*/
	public Action[] getActions(){
		//Get all relations pointing to actions
		Relation rels[] = this.getLocalRelations("HAS_ACTION");
	
		if(rels==null){
			return null;
		}
		
		Action actions[] = new Action[rels.length];
	
		//Parse through the relations and get the actions they point to ans store them in the reurn value
		for(int i=0; i<rels.length; i++){
			try{
				actions[i] = new Action(rels[i].getValue());
			}catch(EntityTypeNotMatchedException e){
				e.printStackTrace();
			}
		}
		
		//Return the set of actions related to this task
		return actions;
	}
	
	

	/**
		Method for setting the cost of a task. Takes a float as parameter.
		@param cost The cost value as flaot which is being related to the task.
	 */
	protected void setCost(float costValue) throws AlreadyHasValueException{
		Cost cost = null;
		RelationType hasCost = getKnowledgeModel().getRelationType("HAS_COST");
		
		//Make a has cost relation type if it does not exist
		try{
			if(hasCost == null) hasCost = new RelationType(getKnowledgeModel(), getKnowledgeModel().getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_COST", "Relation pointing to an a cost entity.", 1.0);
		}catch(NameAlreadyExistException e){
			e.printStackTrace();
		}
		
		//Get all local relations of type has cost
		Relation rels[] = this.getLocalRelations("HAS_COST");
		
		//If no local relation of type has cost, relate the cost specified to this task
		if(rels.length == 0){
			String costName = getName() +"-cost";
			try{
				cost = new Cost((TMST)getKnowledgeModel(), costName, "psm cost", (Entity)this, Float.toString(costValue));
			}catch(NameAlreadyExistException e){
				e.printStackTrace();
			}
			//if(cost != null) new Relation(this, hasCost, cost);
		}else throw new AlreadyHasValueException("A task may only hvae one cost related to it. Something is very wrong!");
	}
	
	/**
		Method returning the cost entity of this task.
		@return Cost
	 */
	public Cost getCost() throws AlreadyHasValueException{
		Relation rels[] = this.getLocalRelations("HAS_COST");
		
		if(rels.length==1) return (Cost)rels[0].getValue();
		else throw new AlreadyHasValueException("A task may only have one cost related to it. Something is very wrong!");
	}

	/**
		Method returning the cost entity of this task.
		@return Cost
	 */
	public float getCostValue() throws AlreadyHasValueException{
		Relation rels[] = this.getLocalRelations("HAS_COST");
		
		//If this task does not have a cost, it has to be generated
//		if(rels.length == 0)  CostNotPossibleToComputeException
		
//		System.out.println("Number of cost entities related to this task: " +rels.length);
		if(rels.length==1) return ((Cost)rels[0].getValue()).getFloatValue();
		else{
			System.out.println("THIS TASK HAS MORE THAN ONE COST RELATED TO IT:");
			for(int i = 0; i < rels.length; i++){
				System.out.println("	|-- Cost name #" +i +" " +rels[i].getValue().getName());
			}
			throw new AlreadyHasValueException("A task may only hvae one cost related to it. Something is very wrong!");
		}
	}
	
	/**
		Method for setting a task as activated.
	*/
	public void setAsActivated(){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType isActivated = tmst.getRelationType("HAS_ACTIVATION");
		
		if(isActivated==null){
			//Has to make new Relation type
			try{
				isActivated = new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "HAS_ACTIVATION", "Relation pointing to an activated entity.", 1.0);
				//System.out.println(" -- Making new relation type: HAS_ACTIVATION");
			}catch(Exception e){
				e.printStackTrace();
			}
		}

		if(!tmst.hasEntity("ACTIVATED")){
			try{
				Entity activated = new Entity((KnowledgeModel)tmst, "ACTIVATED", "Entity for indicating whether a task, a psm or an action is activated.");
				activated.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));			
				//System.out.println(" -- Making new entity: ACTIVATED");
			}catch(NameAlreadyExistException e){
				//Do nothing, it the entity already exist, there is no problem.
			}catch(NoSuchRelationTypeException e){
				//This should never happen.
				e.printStackTrace();
			}
			
		}
		new Relation(this, isActivated, tmst.getEntity("ACTIVATED"));
	}
	
	/**
	Method for setting a task as activated.
	 
	 */
	public void setAsSolutionPart(){
		TMST tmst = (TMST)this.getKnowledgeModel();
		RelationType isSolutionPart = tmst.getRelationType("IS_SOLUTION_PART");
		
		if(isSolutionPart == null){
			//Has to make new Relation type
			try{
				isSolutionPart= new RelationType((KnowledgeModel)tmst, tmst.getRelationType(BasicModel.HAS_VALUE_TYPE), "IS_SOLUTION_PART", "Relation pointing to an entity part of the solution.", 1.0);
				//System.out.println(" -- Making new relation type: HAS_ACTIVATION");
			}catch(Exception e){
				e.printStackTrace();
			}
		}
		
		if(!tmst.hasEntity("SOLUTION_PART")){
			try{
				Entity solutionPart = new Entity((KnowledgeModel)tmst, "SOLUTION_PART", "Entity for indicating whether a task, a psm or an action is part of the solution.");
				solutionPart.addRelation(BasicModel.INSTANCE_OF, tmst.getEntity(BasicModel.SYMBOL));			
				//System.out.println(" -- Making new entity: ACTIVATED");
			}catch(NameAlreadyExistException e){
				//Do nothing, it the entity already exist, there is no problem.
			}catch(NoSuchRelationTypeException e){
				//This should never happen.
				e.printStackTrace();
			}
			
		}
		new Relation(this, isSolutionPart, tmst.getEntity("SOLUTION_PART"));
	}	
	
	/**
		Method for checking whether a task is part of the solution space or not.
		@return boolean The method returns true if part of the solution space and false if not.
	 */
	public boolean isSpacePart(){
		try{
			if(this.getLocalRelation("IS_SPACE_PART")!=null){
				return true;
			}
		}catch(NoSuchRelationTypeException e){
			//If there is no such relationtype, this task is not activated.
			//Nothing need to be done and false is returned.
		}catch(NoSuchRelationException e){
			//If there is no such relation, this task is not activated.
			//Nothing need to be done and false is returned.
		}
		return false;
	}
	
	/**
		Method for checking whether a task is part of the solution or not.
		@return boolean The method returns true if part of the solution and false if not.
	 */
	public boolean isSolutionPart(){
		try{
			if(this.getLocalRelation("IS_SOLUTION_PART")!=null){
				return true;
			}
		}catch(NoSuchRelationTypeException e){
			//If there is no such relationtype, this task is not activated.
			//Nothing need to be done and false is returned.
		}catch(NoSuchRelationException e){
			//If there is no such relation, this task is not activated.
			//Nothing need to be done and false is returned.
		}
		return false;
	}
		
	/**
		Method for checking whether a task is activated or not.
		@return boolean The method returns true if activated and false if not.
	*/
	public boolean isActivated(){
		try{
			if(this.getLocalRelation("HAS_ACTIVATION")!=null){
				return true;
			}
		}catch(NoSuchRelationTypeException e){
			//If there is no such relationtype, this task is not activated.
			//Nothing need to be done and false is returned.
		}catch(NoSuchRelationException e){
			//If there is no such relation, this task is not activated.
			//Nothing need to be done and false is returned.
		}
		return false;
	}
	
	/**
		Method returning true if the this task is the initial problem and thus the root node in the
		task structure tree, and false if otherwise.
		@return boolean True if this task is the initial problem, else false.
	*/
	
	public boolean isInitialProblem(){
		try{
			Relation rel = this.getLocalRelation("IS_INITIAL_PROBLEM");
		}catch(NoSuchRelationException e){
			//If this task has no such relation, it is not the initial problem. Thus, return false.
			return false;			
		}catch(NoSuchRelationTypeException e){
			//If the IS_INITIAL_PROBLEM relation type does not exist, this task is not a the initial problem.
			return false;
		}
		return true;
	}
	
	/**
		Method which has to be overriden (from the class EntityType)
	*/
	public static boolean matches(Entity value){
		TMST tmst = (TMST)value.getKnowledgeModel();
		Entity ent = tmst.getEntity("TASK");
		
		if (ent == null) return false; 
		
		Collection subclasses = ent.getAllSubclassesCollection();
		
		if(value.getName().equals("TASK")){
			return false;
		}else if(subclasses.contains(value)){
			return true;
		}
		return false;
	}
}
	